<?php
/**
 * Admin.php
 *
 * This is the Admin Center page. Only administrators
 * are allowed to view this page. This page displays the
 * database table of users and banned users. Admins can
 * choose to delete specific users, delete inactive users,
 * ban users, update user levels, etc.
 *
 * Original by: Jpmaster77 a.k.a. The Grandmaster of C++ (GMC)
 * Overhauled and Extended by: vbaker@engenic.com
 * Last Updated: April 13, 2010
 */
 
// Determine relative location for paths
require("cr_reference.php"); 
require_once($incPath.'LocalSettings.php'); 
require($incPath."include/session.php");
$_SESSION['IncPath'] = $incPath;

require_once($websiteLibrary.'calcWorkUnit.php');   



// Check to see if we have just registered someone new (add / invite)
if ((isset($_SESSION['regsuccess'])) and ($_SESSION["new_user_id"] > 0)) 
{
	unset($_SESSION['regsuccess']);
	unset($_SESSION['reguname']);
	header(sprintf("Location: %s", "cr_admin_invite.php"));      
}


/* Display requested user information */
$req_user_info = $database->getUserInfo($session->username);
/* Get company info */
//$req_company_info = $database->getCompanyInfoByID($req_user_info['company_id']);
/* Get client info */
$req_client_info  = $database->getClientInfo($req_user_info['client_id']);

// Are we allowed to be here? 
// -- User level must be at least 9 admin
if(!$session->isAdmin())
{
	// return to main
	header(sprintf("Location: %s", "cr_main.php"));      
}

// Set company code for add users
$companycode = $appCompanySvcCode;  // was '' 
$form->setValue("company", $companycode);


/**
 * displayCompanies - Displays the companies created by Admin
 * a nicely formatted html table.
 */
function displayCompanies($ClientId)
{
	global $database;
	
	$q = "
SELECT 
	company_id, company_name, company_code
FROM 
	".TBL_COMPANY."
WHERE 
	company_owner_id = ".$ClientId."
ORDER BY company_id";
   // Debug
   //echo "<br>".$q."<br>";
   
	$result = $database->query($q);
	/* Error occurred, return given name by default */
	$num_rows = $database->sqlConvert_sql_numrows($result);
	if(!$result || ($num_rows < 0))
	{
		// "Error displaying info"
		echo providePhrase('cr', 500);
		return;
	}
	if($num_rows == 0)
	{
		// No Subcompanies Currently Exist
		echo "<p>".providePhrase('cr', 1010)."</p>";
		return;
	}
	
	/* Display table contents */
	echo "<table align=\"left\" width=\"550\" border=\"0\" cellspacing=\"1\" cellpadding=\"0\" bgColor=#eeeeee>\n";
	echo "<tr><td bgcolor=\"#29386B\" span class=\"style13\" width=50><b>Code</b></td>"
		   ."<td bgcolor=\"#29386B\" span class=\"style13\" width=200><b>Company</b></td>"
		   ."<td bgcolor=\"#29386B\" span class=\"style13\" width=20><b>Users</b></td>"
		   ."</tr>\n";
		   
	for($i=0; $i<$num_rows; $i++)
	{
		$ccode = $database->sqlConvert_sql_result($result,$i,"company_code");
		$cname = $database->sqlConvert_sql_result($result,$i,"company_name");
		
		// Total users
		$req_total = $database->getCompanyUserTotal($ccode);
		$total = $req_total['total'];
		if ($ccode == '') 
		{
			$ccode = '&nbsp;';
		}
		if ($cname == '') 
		{
			$cname = '&nbsp;';
		}
		echo "<tr><td><p>$ccode</p></td><td><p>$cname</p></td><td><p>$total</p></td></tr>\n";
	}
	echo "</table><br>\n";
}


/**
 * displayUsers - Displays the users database table in
 * a nicely formatted html table.
 */
function displayUsers($ClientId)
{
	global $database;

	$q = "
SELECT 
	user_login_name,
	u_full_name,
	u_permissions_level,
	u_email,
	u_timestamp,
	u_status
FROM ".TBL_USERS."
WHERE 
	client_id = ".$ClientId."
AND u_status != 'DELETED'
ORDER BY u_permissions_level DESC,user_login_name
";
	//echo $q; die;
	$result = $database->query($q);

	/* Error occurred, return given name by default */
	$num_rows = $database->sqlConvert_sql_numrows($result);
	if(!$result || ($num_rows < 0))
	{
		echo "Error displaying info";
		return;
	}
	if($num_rows == 0)
	{
		echo "Database table empty";
		return;
	}
	
	$mDisplay = '';
	/* Display table contents */
	$mDisplay .= "<table width=\"250\" border=\"0\" cellspacing=\"1\" cellpadding=\"1\">";
	$mDisplay .= "<tr><td>";
	$mDisplay .= '<p class="userAdmin">&bull; <b>'.providePhrase('cr', 1000).'</b></p>'; // Super Administrators
	$mDisplay .= '<p class="userAdmin">&bull; '.providePhrase('cr', 1001).'</p>'; // Company Admins
	$mDisplay .= '<p class="userStandard">&bull; '.providePhrase('cr', 1002).'</p>'; // Standard Users
	$mDisplay .= '<p class="userNotYetAuth">&bull; '.providePhrase('cr', 1003).'</p>'; // Users Pending Authorization
	$mDisplay .= "</td></tr>";
	$mDisplay .= "</table><br />";
	
	$mDisplay .= "<table width=\"650\" border=\"0\" cellspacing=\"1\" cellpadding=\"1\" bgColor=#eeeeee>\n";
	$mDisplay .= "<tr>
			<td bgcolor=\"#29386B\" span class=\"style13\">".providePhrase('cr', 71)."</td>
			<td bgcolor=\"#29386B\" span class=\"style13\">".providePhrase('cr', 72)."</td>
			<td bgcolor=\"#29386B\" span class=\"style13\">".providePhrase('cr', 81)."</td>
			<td width=\"50\" bgcolor=\"#29386B\" span class=\"style13\">".providePhrase('cr', 1020)."</td>
		 </tr>\n";

	for($i=0; $i<$num_rows; $i++)
	{
		$uname  = $database->sqlConvert_sql_result($result,$i,"user_login_name");
		$ufull  = $database->sqlConvert_sql_result($result,$i,"u_full_name");
		$ulevel = $database->sqlConvert_sql_result($result,$i,"u_permissions_level");
		$email  = $database->sqlConvert_sql_result($result,$i,"u_email");
		$time   = $database->sqlConvert_sql_result($result,$i,"u_timestamp");
		$usts   = $database->sqlConvert_sql_result($result,$i,"u_status");

		// Only display non-deleted
		if ($usts <> 'DELETED')
		{		
			$mClass = "userStandard";

			// Check for admin
			if (($ulevel == ADMIN_LEVEL) || ($ulevel == SUPER_ADMIN_LEVEL)) // Admin
			{
				$mClass = "userAdmin";
			}
			// Check for invited
			if ($usts == 'CREATE')
			{
				$mClass = "userNotYetAuth";
			}
			

			if ($ufull == '') {
				$ufull = '&nbsp;';
			}
			if ($uname == '') {
				$uname = '&nbsp;';
			}
			if ($ulevel == '') {
				$ulevel = '&nbsp;';
			}
			if ($email == '') {
				$email = '&nbsp;';
			}
			if ($time == '') {
				$time = '&nbsp;';
			}

			$mDisplay .= '<tr class="'.$mClass.'">';			
			//echo "<tr><td bgcolor=".$mcolor."><a href=\"cr_userinfo.php?user=".$uname."\" target=\"_blank\"><p>$uname</p></a></td>"
			if (trim($uname) <> '&nbsp;')
			{
				// See if we should allow and edit (don't allow a connect)
				if (($ulevel == SUPER_ADMIN_LEVEL))
				{
					$mDisplay .= '<td>'.
								'<p><b>'.$uname.'</b></p>'.
								'</td>';
				}
				else
				{
					$mDisplay .= '<td><a href="cr_userinfo.php?user='.$uname.'">'.
								'<p>'.$uname.'<img width="16" height="16" border="0" src="images/icon_edit.gif"></p>'.
								'</a></td>';
				}
			}
			else
			{
				$mDisplay .= "<td>Undefined</td>";			
			}
			$mDisplay .= "<td><p>".$ufull."</p></td>".
						"<td><p>".$email."</p></td>";
			   
			// Put in an edit button
			$mDisplay .= "<td>"; //btnCopy'.$i.'
			$mDisplay .= '<input name="'.$uname.'" width="22" type="button" class="smallbtn" id="btnCopy'.$i.'" value="Select" onclick="selectuser(this.id)"/>';
			$mDisplay .= "</td>";
			
			$mDisplay .= "</tr>\n";
			
		}
	}
	$mDisplay .= "</table><br />\n";
	return $mDisplay;
} // DisplayUsers

// userinfo.php?user=admin

/**
 * displayBannedUsers - Displays the banned users
 * database table in a nicely formatted html table.
 */
function displayBannedUsers($ClientId)
{
	global $database;
	$q = "
SELECT 
	user_login_name, 
	u_timestamp 
FROM 
	".TBL_BANNED_USERS." 
WHERE 
	client_id = ".$ClientId." 
ORDER BY user_login_name";
	//echo $q."<br>";       

	$result = $database->query($q);
	// Error occurred, return given name by default 
	$num_rows = $database->sqlConvert_sql_numrows($result);
	if(!$result || ($num_rows < 0))
	{
		echo "Error displaying info.";
		return;
	}
	if($num_rows == 0)
	{
		echo "No users have been banned.";
		return;
	}
	// Display table contents
	echo "<table align=\"left\" width=\"320\" border=\"0\" cellspacing=\"1\" cellpadding=\"0\" bgColor=#eeeeee>\n";
	echo "<tr><td bgcolor=\"#29386B\" span class=\"style13\" width=140>".providePhrase('cr', 70)."</td>
			 <td bgcolor=\"#29386B\" span class=\"style13\" width=180>".providePhrase('cr', 1021)."</td></tr>\n"; // Time banned
	for($i=0; $i<$num_rows; $i++)
	{
		$uname = $database->sqlConvert_sql_result($result,$i,"user_login_name");
		$time  = $database->sqlConvert_sql_result($result,$i,"u_timestamp");
		echo "<tr><td><p>$uname</p></td><td><p>".date("F j, Y, g:i a",$time)."</p></td></tr>\n";
	}
	echo "</table><br>\n";
}

include($_SESSION['IncPath']."include/header.php"); 
// JavaScript goes here + + +
?>
<script type="text/javascript"> 
//  Set up jQuery - Now is a good timeto see if it will fly / load up the libraries
$(document).ready(function()
{
	// Cancel buttons
	$("#cancel").click(function()
	{	
		window.location = "cr_main.php"
	}); 

	// Cancel buttons
	$("#edit").click(function()
	{	
		window.location = "cr_useredit.php"
	});
	
	<?php
	include('cr_javascript_graphics.php');
	?>	
})
</script>

<script type="text/javascript"> 
function selectuser(x)
{
	var c = document.getElementById(x).name;
	var e = document.getElementById(x).value;
		
	//alert(document.getElementById(x).name);

	// Populate the values
	document.getElementById("deluser").value = c;	
	document.getElementById("upduser").value = c;
}
</script>
<?php
include($_SESSION['IncPath']."include/header_divs.php"); 

// Start - TOP MENU 
// Default names that are replaced from the database
$mDefaultDisplay 	= providePhrase('cr', 1052); //Client Administration
$mDefaultDesc 		= providePhrase('cr', 1053); //Manage User Access
$database->getApplicationNameForWebpage($thisWebPage, $mDefaultDisplay, $mDefaultDesc);
?>
<div id="topbar">
<a class="active"><span><div id="bartitle"><?php echo($mDefaultDisplay);?></div></span></a>	
<?php
echo CheckMenuText($mDefaultDesc );
//echo CheckCurrentMenu($appSysUri, 'cr_register.php', 'Sign Up',  $incPath.'images/tb_span.png');
?>
</div>
<?php
//include($incPath."include/contact_links.php");
// END - TOP MENU
?>
<div id="frontpage" >
<div id="maintext">

<table cellpadding="5" border="0">
<tbody>
<tr>
	<td>
	<table align="left" border="0" cellspacing="5" cellpadding="5">
	<tr>
		<td>
		<?php 
		// Administration Center
		echo(displayContainerOutline('<h1>'.providePhrase('cr', 1050).'</h1>'));
		?>
		</td>
	</tr> 

	<?php
	if ($form->num_errors > 0)
	{
		echo '<tr><td>';
		echo "<font class=dataError>".$form->num_errors.' '.providePhrase('cr', 501)."</font>"; // eg: 1 errors foud
		echo '</td></tr>';
	}
	?>
	<tr>
		<td>
<?php
		echo(displayContainerHeader(1,''));
?>
	
		<TABLE align=center cellSpacing=0 cellPadding=1 width=99% border=0>
		<TR>
			<TD width=115><span class="style5">Name</span></TD>
			<TD><span class="style5"><b><?php echo($req_user_info['u_full_name']); ?></b></span></TD>
		</TR>
		<TR>
			<TD><span class="style5">Company</span></TD>
			<TD><span class="style5"><?php echo(displayCompanyNameTitle(1,$_SESSION['userclientname'])); ?></span></TD>
		</TR>
		<TR>
			<TD><span class="style5">&nbsp;</span></TD>
			<TD></TD>
		</TR>
		</TABLE>	

		</td>
	</tr>
	
	<tr>	
		<td>
<?php	
// Are we allowed to be here? 
// -- User level must be at least 9 admin
echo(displayContainerHeader(1, providePhrase('cr', 1051), ''));  // Administration Menu
if ($req_user_info['u_permissions_level'] >= SUPER_ADMIN_LEVEL)
{
	// Display the menu item, but with no link
	$mMenu = menuItemBuild('cr_access_client.php', 
							providePhrase('cr', 1060), //SUPER ADMIN
							providePhrase('cr', 1061), //Access any client as an Administrator
							'');
	echo(displayContainerMenuItem($mMenu));	
}
// Unlock settings
$mMenu = menuItemBuild('cr_unlock.php', 
							providePhrase('cr', 1062), //Unblock Lockout
							providePhrase('cr', 1063), //Provide Access Locked Users an Unlock Code
							'');
echo(displayContainerMenuItem($mMenu));	
// Teleflow Server views
$mMenu = menuItemBuild($incPath.'open_teleflow/tf_monitor.php', 
							providePhrase('cr', 1064), // TeleFlow Servers
							providePhrase('cr', 1065), //Monitor server functions for each port/line
							'');
echo(displayContainerMenuItem($mMenu));	


// End the jump box
echo(displayContainerFooter());
?>
		<td>
	</tr> 
		
	<tr>
		<td>
<?php
////////////////////// USERS ///////////////////////////
/**
 * Display Users Table
 */
		$mDisplay = "<h2>Users in ".$_SESSION['userclientname']."</h2>";

/**
 * Update User Level
 */
		$mUpdateUser =
		'<h3><font color="#000044">'.providePhrase('cr', 1070).'</font></h3>'.  // Update User Level
		'<p>'.providePhrase('cr', 1071).'</p>'.  // (1) = General access  (9) = Admin level
		$form->error("upduser").
		'<table width="450px">'.
		'<form action="cr_adminprocess.php" method="POST">'.
		'<tr>'.
			'<td>'.
			providePhrase('cr', 70).'<br />'.  // Username
				'<input type="text" id="upduser" name="upduser" size="40" maxlength="60" value="'.$form->value("upduser").'">'.
			'</td>'.		
			'<td>'.
			providePhrase('cr', 1072).'<br />'. // Level
				'<select name="updlevel">'.
				'<option value="1">1'.
				'<option value="9">9'.
				'</select>'.
			'</td>'.		
			'<td>'.
			'<br />'.
			'<input type="hidden" name="subupdlevel" value="1">'.
			'<input type="submit" value="'.providePhrase('cr', 1073).'">'.
			'</td>'.
		'</tr>'.
		'<tr>'.
			'<td>'.$form->error("level").'</td>'.
		'</tr>'.				
		'</form>'.
		'</table>'.
		'<br />';

	
/**
 * Delete user
 */
		$mDeleteUser =
		'<h3><font color="#000044">'.providePhrase('cr', 1080).'</font></h3>'. // Delete User
		$form->error("deluser").
		'<table width="450px">'.
		'<form action="cr_adminprocess.php" method="POST">'.
		providePhrase('cr', 70).'<br />'. // Username
		'<tr>'.
			'<td>'.
			'<input type="text" id="deluser" name="deluser" size="40" maxlength="60" value="'.$form->value("deluser").'">'.
			'</td>'.
			'<td>'.
			'<input type="hidden" name="subdeluser" value="1">'.
			'<input type="submit" value="'.providePhrase('cr', 1081).'">'.
			'</td>'.
		'</tr>'.
		'<tr>'.
			'<td>'.$form->error("delete").'</td>'.
		'</tr>'.		
		'</form>'.
		'</table>'.
		'<br />';
		
		// Display the works
		echo (displayContainerOutline($mDisplay.  //
				displayUsers($req_user_info['client_id']).
				displayContainerShaded($mUpdateUser).
				displayContainerShaded($mDeleteUser)
			));
?>
		</td>
	</tr>

	<tr>
		<td>
		<hr />
		</td>
	</tr>
</tr>

<tr>
	<td>
	<form action="cr_process.php" method="POST">
	<table width="750" align="left" border="0" cellspacing="0" cellpadding="3">
<?php	
/* ============
Add / Invite User - processed with cr_process.php with field code as "subinvite"
 ============ */
?>
	<tr>
	  <td width="180"><h3><font color="#000044"><?php echo(providePhrase('cr', 1090));?></font></h3></td>
	  <td width="220">&nbsp;</td>
	  <td width="10">&nbsp;</td>
	  <td width="250">&nbsp;</td>
	  <td>&nbsp;</td>
	</tr>

	<tr>
	  <td colspan="5">
		<?php echo(providePhrase('cr', 1091));?>	
		<?php echo $_SESSION['userclientname']; // $req_client_info['company_name']?>
		<?php echo(providePhrase('cr', 1092));?>	
	  </td>
	</tr>

	<tr>
	  <td><span class="style5"><?php echo(providePhrase('cr', 1100));?></td>
	  <td><input type="text" name="email" maxlength="50" size="30" value="<?php echo $form->value("email"); ?>"></td>
	  <td>&nbsp;</td>
	  <td><?php echo $form->error("email"); ?></td>
	  <td>&nbsp;</td>
	</tr>

	<tr>
	  <td><span class="style5"><?php echo(providePhrase('cr', 1101));?></td>
	  <td><input type="text" name="namefirst"  maxlength="25"  size="20" value="<?php echo $form->value("namefirst"); ?>"></td>
	  <td>&nbsp;</td>
	  <td><?php echo $form->error("namefirst"); ?></td>
	  <td>&nbsp;</td>
	</tr>

	<tr>
	  <td><span class="style5"><?php echo(providePhrase('cr', 1102));?></td>
	  <td><input type="text" name="namemiddle" maxlength="1"   size="1"  value="<?php echo $form->value("namemiddle"); ?>"></td>
	  <td>&nbsp;</td>
	  <td>&nbsp;</td>
	  <td>&nbsp;</td>
	</tr>

	<tr>
	  <td><span class="style5"><?php echo(providePhrase('cr', 1103));?></td>
	  <td><input type="text" name="namelast"   maxlength="50"  size="20" value="<?php echo $form->value("namelast"); ?>"></td>
	  <td>&nbsp;</td>
	  <td><?php echo $form->error("namelast"); ?></td>
	  <td>&nbsp;</td>
	</tr>

	<tr>
	  <td>&nbsp;</td>
	  <td colspan="2" align="left">
	  <input type="hidden" name="subinvite" value="1">
	  <input type="hidden" name="client" value="<?php echo $_SESSION['userclientid']; ?>">
	  <input type="submit" value="<?php echo(providePhrase('cr', 1104));?>"></td>
	  <td>&nbsp;</td>
	  <td>&nbsp;</td>
	</tr>

	<tr>
	  <td colspan="5"></td>
	</tr>

	</table>
	</form>
	</td>
</tr>

<tr>
	<td>
	<hr />
	</td>
</tr>

<tr>
	<td>
	<?php
	/**
	 * Delete Inactive Users
	 */
	?>
	<h3><font color="#000044"><?php echo(providePhrase('cr', 1110));?></font></h3>
	<p><?php echo(providePhrase('cr', 1111));?></p>
	<table>
	<form action="cr_adminprocess.php" method="POST">
	<tr><td>
	<?php echo(providePhrase('cr', 1112));?><br />
	
	<select name="inactdays">
	<option value="14">14
	<option value="30">30
	<option value="60">60
	<option value="120">120
	<option value="365">365
	</select>
	</td>
	<td>
	<br>
	<input type="hidden" name="subdelinact" value="1">
	<input type="submit" value="<?php echo(providePhrase('cr', 1113));?>">
	</td>
	</tr>
	</form>
	</table>
	</td>
</tr>

<tr>
	<td>
	<hr />
	</td>
</tr>


<?php
/**
 * Ban User
 */
//////////////////////////// HIDE THE REST OF THE PAGE FOR NOW //////////////////////////////////
if (1 == 2)
{
?>
<tr>
<td>
<h3><font color="#000044"><?php echo(providePhrase('cr', 1120));?></font></h3>
<?php echo $form->error("banuser"); ?>
<form action="cr_adminprocess.php" method="POST">
<?php echo(providePhrase('cr', 70));?><br />
<input type="text" name="banuser" maxlength="30" value="<?php echo $form->value("banuser"); ?>">
<input type="hidden" name="subbanuser" value="1">
<input type="submit" value="<?php echo(providePhrase('cr', 1121));?>">
</form>
</td>
</tr>

<tr>
<td>
<hr />
</td>
</tr>


<tr>
<td>
<?php
/**
 * Display Banned Users Table
 */
?>
<h3><font color="#000044"><?php echo(providePhrase('cr', 1122));?></font></h3>
<?php
displayBannedUsers($req_user_info['client_id']);
?>
</td>
</tr>

<tr>
<td>
<hr />
</td>
</tr>

<tr>
<td>
<?php
/**
 * Delete Banned User
 */
?>
<h3><font color="#000044"><?php echo(providePhrase('cr', 1130));?></font></h3>
<?php echo $form->error("delbanuser"); ?>
<form action="cr_adminprocess.php" method="POST">
<?php echo(providePhrase('cr', 70));?><br />
<input type="text" name="delbanuser" maxlength="30" value="<?php echo $form->value("delbanuser"); ?>">
<input type="hidden" name="subdelbanned" value="1">
<input type="submit" value="<?php echo(providePhrase('cr', 1131));?>">
</form>
</td>
</tr>


<tr>
<td>
<br />
<?php
////////////////////// COMPANIES ///////////////////////////
/**
 * Display CompaniesTable
 */
echo ("<h2>".providePhrase('cr', 1140)." ".$req_client_info['company_name']."</h2>");
if(isset($_SESSION['company_added'])) 
{
    // If there are no users for the company, suggesting making an admin
    $req_total = $database->getCompanyUserTotal($_SESSION['company_added']);
    if ($req_total['total'] == 0)
	{
		echo(providePhrase('cr', 1141)." <b>".$_SESSION['company_added']."</b><br />");
		echo("<font color=\"#ff0000\"><p>".providePhrase('cr', 1142)."</p></font> [<a href=\"cr_register.php\">Register</a>]<br /><br />");
    }
}
// Display list of companies in a table
displayCompanies($req_user_info['client_id']);
?>
<br />
<?php
/**
 * Add Company
 */
?>
<h3><font color="#000044"><?php echo(providePhrase('cr', 1150));?></font></h3>
<?php 
echo $form->error("addcompanycode"); 
?>
<p><?php echo(providePhrase('cr', 1151));?></p>
<table>
<form action="cr_adminprocess.php" method="POST">
<tr>
<td>
<?php echo(providePhrase('cr', 1152));?><br />
<input type="text" name="addcompanycode" size=8 maxlength="20" value="<?php echo $form->value("addcompanycode"); ?>">
</td>
<td>
<?php echo(providePhrase('cr', 1153));?><br />
<input type="text" name="addcompanyname" size=35 maxlength="100" value="<?php echo $form->value("addcompanyname"); ?>">
<input type="hidden" name="subaddcompany" value="1">
<input type="hidden" name="subaddcompanyownerid" value="<?php echo $req_user_info['company_id']; ?>">
<input type="submit" value="<?php echo(providePhrase('cr', 1154));?>">
</td>
</tr>
</form>
</table>
</td>
</tr>

<tr>
<td>
<hr />
</td>
</tr>
<?php 
} ///////////////////////// HIDE FOR NOW ////////////////////;
?>


</table>
</TD></TR>


</tbody>
</table>


<!-- Footer standard -->
</div> <!-- maintext -->
</div> <!-- frontpage -->
<?php 
include($_SESSION['IncPath']."include/footer.php");
?>