<?php
/**
 * AdminProcess.php
 * 
 * The AdminProcess class is meant to simplify the task of processing
 * admin submitted forms from the admin center, these deal with
 * member system adjustments.
 *
 * Written by: Jpmaster77 a.k.a. The Grandmaster of C++ (GMC)
 * Over hauled by Vern Baker
 * Last Updated: August 15, 2004
 */
require("cr_reference.php"); 
require_once($incPath.'LocalSettings.php'); 
require($incPath."include/session.php");
$_SESSION['IncPath'] = $incPath;

class AdminProcess
{
	/* Class constructor */
	function AdminProcess()
	{
		global $session;
		/* Make sure administrator is accessing page */
		if(!$session->isAdmin())
		{
			header("Location: cr_main.php");
			return;
		}
		/* Admin submitted add company */
		if(isset($_POST['subaddcompany']))
		{
			$this->procAddCompany();
		}
		/* Admin submitted update user level form */
		if(isset($_POST['subupdlevel']))
		{
			$this->procUpdateLevel();
		}
		/* Admin submitted delete user form */
		else if(isset($_POST['subdeluser']))
		{
			$this->procDeleteUser();
		}
		/* Admin submitted delete inactive users form */
		else if(isset($_POST['subdelinact']))
		{
			$this->procDeleteInactive();
		}
		/* Admin submitted ban user form */
		else if(isset($_POST['subbanuser']))
		{
			$this->procBanUser();
		}
		/* Admin submitted delete banned user form */
		else if(isset($_POST['subdelbanned']))
		{
			$this->procDeleteBannedUser();
		}
		/* Should not get here, redirect to home page */
		else
		{
			echo "Error - No requests processed";
			header("Location: cr_main.php");
		}
	}

/**
* procAddCompany - Will attempt to add a new company
*/
	function procAddCompany()
	{
		global $session, $database, $form;
		/* Companycode error checking */
		$subcompanycode = $this->checkCompanycode("addcompanycode");

		/* Errors exist, have user correct them */
		if($form->num_errors > 0)
		{
			$_SESSION['value_array'] = $_POST;
			$_SESSION['error_array'] = $form->getErrorArray();
			header("Location: ".$session->referrer);
		}
		/* Add Company */
		else
		{
			// Remember the Company
			$_SESSION['company_added'] = $subcompanycode;
			$subcompanyownerid = $_POST['subaddcompanyownerid'];
			$database->addCompany($subcompanycode, $_POST['addcompanyname'], (int)$subcompanyownerid);
			header("Location: ".$session->referrer);
		}
	}


/**
procUpdateLevel - If the submitted username is correct,
their user level is updated according to the admin's
request.
Test: Make sure that there is at least one other Admin before allowing the change.  
*/
	function procUpdateLevel()
	{
		global $session, $database, $form;
		/* Username error checking */
		$subuser = $this->checkUsername("upduser");

		/* Errors exist, have user correct them */
		if($form->num_errors > 0)
		{
			$_SESSION['value_array'] = $_POST;
			$_SESSION['error_array'] = $form->getErrorArray();
			header("Location: ".$session->referrer);
		}
		/* Update user level */
		else
		{
			// Make sure we are not killing off our only admin
			if (((int)$_POST['updlevel'] < ADMIN_LEVEL) && 
			    ($database->getAdminTotal($_SESSION['userclientid']) <= 1))
			{
				$form->setError("level", providePhrase('cr', 600)); // There must be at least one administator
				$_SESSION['value_array'] = $_POST;
				$_SESSION['error_array'] = $form->getErrorArray();
				//echo $database->getAdminTotal($_SESSION['userclientid']).'*'.(int)$_POST['updlevel']; die;
			}
			else
			{
				$database->updateUserField($subuser, "u_permissions_level", (int)$_POST['updlevel']);
			}
			header("Location: ".$session->referrer);
		}
	}


   
/**
* procDeleteUser - If the submitted username is correct,
* the user is deleted from the database.
*/
	function procDeleteUser()
	{
		global $session, $database, $form;
		/* Username error checking */
		$subuser = $this->checkUsername("deluser");

		/* Make sure we are not deleting the last Administrator */
		if ($database->getAdminTotal($_SESSION['userclientid']) <= 1)
		{
			$form->setError("delete", providePhrase('cr', 600)); // There must be at least one administator
		}		
		
		
		/* Errors exist, have user correct them */
		if($form->num_errors > 0)
		{
			$_SESSION['value_array'] = $_POST;
			$_SESSION['error_array'] = $form->getErrorArray();
			header("Location: ".$session->referrer);
		}
		/* Delete user  */
		else
		{
			$q = "
UPDATE 			
	".TBL_USERS."
SET
	u_status = 'DELETED'
WHERE
	user_login_name = '".$subuser."'
AND u_status <> 'DELETED'";
			$database->query($q);
			
			// Now, also remove the entry from the master talbe
			$database->deleteUserFromMaster($subuser);
			
			header("Location: ".$session->referrer);
		}
	} // procDeleteUser
   
   
/**
* procDeleteInactive - All inactive users are deleted from
* the database, not including administrators. Inactivity
* is defined by the number of days specified that have
* gone by that the user has not logged in.
*/
	function procDeleteInactive()
	{
		global $session, $database;
		
		$inact_time = $session->time - $_POST['inactdays']*24*60*60;
		$q = "
UPDATE 			
	".TBL_USERS."
SET
	u_status = 'DELETED'
WHERE
	u_timestamp < ".$inact_time."
AND userlevel != ".ADMIN_LEVEL;
	
		$database->query($q);

		// Now, also remove the entry from the master talbe
		$database->deleteUserFromMaster($subuser);
		
		header("Location: ".$session->referrer);
	} // procDeleteInactive
   
   
/**
* procBanUser - If the submitted username is correct,
* the user is banned from the member system, which entails
* removing the username from the users table and adding
* it to the banned users table.
*/
	function procBanUser()
	{
		global $session, $database, $form;

		/* Username error checking */
		$subuser = $this->checkUsername("banuser");

		/* Errors exist, have user correct them */
		if($form->num_errors > 0)
		{
			$_SESSION['value_array'] = $_POST;
			$_SESSION['error_array'] = $form->getErrorArray();
			header("Location: ".$session->referrer);
		}
		/* Ban user from member system */
		else
		{
			$mclientid = $session->userclientid;
			$q = "
UPDATE 			
	".TBL_USERS."
SET
	u_status = 'DELETED'
WHERE
	user_login_name = '".$subuser."'";			
			$database->query($q);

			// Now, also remove the entry from the master talbe
			$database->deleteUserFromMaster($subuser);

			// Finally, put an entry in the banned list
			$q = "INSERT INTO ".TBL_BANNED_USERS." VALUES ('$subuser', $session->time, $mclientid)";
			$database->query($q);
			header("Location: ".$session->referrer);
		}
	} // procBanUser
   
/**
* procDeleteBannedUser - If the submitted username is correct,
* the user is deleted from the banned users table, which
* enables someone to register with that username again.
*/
	function procDeleteBannedUser()
	{
		global $session, $database, $form;
		
		/* Username error checking */
		$subuser = $this->checkUsername("delbanuser", true);

		/* Errors exist, have user correct them */
		if($form->num_errors > 0)
		{
			$_SESSION['value_array'] = $_POST;
			$_SESSION['error_array'] = $form->getErrorArray();
			header("Location: ".$session->referrer);
		}
		/* Delete user from database */
		else
		{
			$q = "DELETE FROM ".TBL_BANNED_USERS." WHERE user_login_name = '$subuser'";
			$database->query($q);
			header("Location: ".$session->referrer);
		}
	} // procDeleteBannedUser
   
/**
* checkUsername - Helper function for the above processing,
* it makes sure the submitted username is valid, if not,
* it adds the appropriate error to the form.
*/
	function checkUsername($uname, $ban=false)
	{
		global $database, $form;

		/* Username error checking */
		$subuser = $_POST[$uname];
		$field = $uname;  //Use field name for username
		if(!$subuser || strlen($subuser = trim($subuser)) == 0)
		{
			$form->setError($field, providePhrase('cr', 601)); // Username not entered
		}
		else
		{
			/* Make sure username is in database */
			$subuser = stripslashes($subuser);
			if ((strlen($subuser) < 5) ||
				(strlen($subuser) > 30) ||
				(!$ban && !$database->usernameExists($subuser)))
			{
				$form->setError($field, providePhrase('cr', 602)); // Username does not exist
			}
		}
		return $subuser;
	} // checkUsername


/**
* checkCompanycode - Helper function for the above processing,
* it makes sure the submitted Companycode is valid, if not,
* it adds the appropriate error to the form.
*/
	function checkCompanycode($companycode)
	{
		global $database, $form;
		/* Companycode error checking */
		$subcompanycode = $_POST[$companycode];
		$field = $companycode;  //Use field name for company name
		if(!$subcompanycode || strlen($subcompanycode = trim($subcompanycode)) == 0)
		{
			$form->setError($field, providePhrase('cr', 603)); // Company code not entered
		}
		else
		{
			/* Make sure company is in database */
			$subcompanycode = stripslashes($subcompanycode);
			if ((strlen($subcompanycode) < 2) || 
				(strlen($subcompanycode) > 30) ||
				($database->companycodeTaken($subcompanycode)))
			{
				$form->setError($field, providePhrase('cr', 604)); // Company code already exists
			}
		}
		return $subcompanycode;
	}

};

/* Initialize process */
$adminprocess = new AdminProcess;

?>
