<?php
/**
 * cr_process.php
 * 
 * The Process class is meant to simplify the task of processing
 * user submitted forms, redirecting the user to the correct
 * pages if errors are found, or if form is successful, either
 * way. Also handles the logout procedure.
 *
 * Written by: Jpmaster77 a.k.a. The Grandmaster of C++ (GMC)
 * Extended by: vbaker@engenic.com
 * Last Updated: March 18, 2009
 */
require_once("cr_reference.php");
require_once($incPath.'LocalSettings.php'); 
require_once($incPath.$sessionphp);


class Process
{
	/* Class constructor */
	function Process()
	{
		global $session;

		/* User submitted login form */
		if(isset($_POST['sublogin']))
		{
			$this->procLogin();
		}
		/* User submitted forgot password form */
		else if(isset($_POST['resetpass']))
		{
			$this->procResetPass();
		}
		/* User submitted registration form */
		else if(isset($_POST['subjoin']))
		{
			// WARNING: This option allows user to register via a website locked to one Client (ie: 1 client for the website only)
			$this->procRegister();
		}
		/* Admin assisted account lockout - generate an unlock code */
		else if(isset($_POST['subunlockcode']))
		{
			// Generate an unlock code for the user
			$this->procUnlockCode();
		}
		/* Admin assisted account lockout - remove lockout */
		else if(isset($_POST['subunlock']))
		{
			// User has received unlock code
			$this->procUnlock();
		}
		/* Admin submitted registration form for a new user invite*/
		else if(isset($_POST['subinvite']))
		{
			// This option allows invites from any client
			$this->procRegisterInvite();
		}
		/* User submitted  telecommunications settings */
		else if(isset($_POST['extadd']))
		{
			$this->procExtAdd();
		}
		/* User submitted forgot password form */
		else if(isset($_POST['subforgot']))
		{
			$this->procForgotPass();
		}
		/* User submitted new password form */
		else if(isset($_POST['setnewpass']))
		{
			$this->procSetNewPass();
		}
		/* User submitted edit account form */
		else if(isset($_POST['subedit']))
		{
			$this->procEditAccount();
		}
		/* Admin user submitted edit account form for a user */
		else if(isset($_POST['subeditadminuser']))
		{
			$this->procEditAccountAdminUser();
		}
		/**
		* The only other reason user should be directed here
		* is if he wants to logout, which means user is
		* logged in currently.
		*/
		else if($session->logged_in)
		{
			$this->procLogout();
		}

		/**
		* Should not get here, which means user is viewing this page
		* by mistake and therefore is redirected.
		*/
		else
		{
			header("Location: cr_main.php");
		}
	}

   
/**
procLogin - Processes the user submitted login form, if errors
are found, the user is redirected to correct the information,
if not, the user is effectively logged in to the system.
*/
	function procLogin()
	{
		global $session, $form;
				
		/* Login attempt */
		$retval = $session->login($_POST['user'], $_POST['pass'], isset($_POST['remember']));

		/* Login successful */
		if ($retval)
		{
			//echo $session->referrer; die;
			header("Location: ".$session->referrer);
			//header("Location: cr_main.php");
		}
		/* Login failed */
		else
		{
			$_SESSION['value_array'] = $_POST;
			$_SESSION['error_array'] = $form->getErrorArray();
			header("Location: ".$session->referrer);
		}
	} // procLogin
   
   
/**
procLogout - Simply attempts to log the user out of the system
given that there is no logout form to process.
*/
	function procLogout()
	{
		global $session;
		$retval = $session->logout();
		
		header("Location: cr_main.php");
	}
   
   
/**
procUnlockCode - Create and unlock key for the user
*/
	function procUnlockCode()
	{
		global $session, $form;
		
		$retval = $session->accessUnlockCode($_POST['user']);
		
		//echo ($retval); die;
		// Populate where we can
		$_SESSION['value_array'] = $_POST;
		$_SESSION['error_array'] = $form->getErrorArray();
		
		$form->setValue("unlockcode", $retval);
	
		
		header("Location: ".$session->referrer);
		
	} // procUnlockCode
   
/**
procUnlock - Test to see if the user has been able to unlock their system.
*/
	function procUnlock()
	{
		global $session, $form;
		
		$retval = $session->accessUnlock($_POST['user'], $_POST['unlockcode']);
		/* Unblock successful */
		if ($retval)
		{
			//echo $session->referrer; die;
			header("Location: ".$session->referrer);
			//header("Location: cr_main.php");
		}
		/* Unblock failed */
		else
		{
			$_SESSION['value_array'] = $_POST;
			$_SESSION['error_array'] = $form->getErrorArray();
			header("Location: ".$session->referrer);
		}	
	} // procUnlock
	

/**
procResetPass - Check to see if the user has entered
the correct reset request information, and send an email
*/
	function procResetPass()
	{
		global $session, $form;
		
		/* Convert username to all lowercase (by option) */
		if (ALL_LOWERCASE)
		{
			$_POST['email'] = strtolower($_POST['email']);
		}

		/* Registration attempt */
		$retval = $session->resetpass($_POST['email'], 
								   $_POST['namelast'], 
								   $_POST['namefirst']);

		/* Registration Successful */
		if($retval == 0)
		{
			$_SESSION['reguname'] = $_POST['namefirst'];
			$_SESSION['regsuccess'] = true;
			header("Location: ".$session->referrer);
		}
		/* Error found with form */
		else if($retval == 1)
		{
			$_SESSION['value_array'] = $_POST;
			$_SESSION['error_array'] = $form->getErrorArray();
			header("Location: ".$session->referrer);
		}
		/* Registration attempt failed */
		else if($retval == 2)
		{
			$_SESSION['reguname'] = $_POST['namefirst'];
			$_SESSION['regsuccess'] = false;
			header("Location: ".$session->referrer);
		}
		/* User not active in client database */
		else if($retval == 3)
		{
			$_SESSION['reguname'] = $_POST['namefirst'];
			$_SESSION['regsuccess'] = false;
			header("Location: ".$session->referrer);
		}
		/* User does not exist in client database */
		else if($retval == 4)
		{
			$_SESSION['reguname'] = $_POST['namefirst'];
			$_SESSION['regsuccess'] = false;
			header("Location: ".$session->referrer);
		}
	} // procResetPass

   
/**
* procSetNewPass - Processes the user submitted password change
*/
	function procSetNewPass()
	{
		global $database, $session, $form;

		$client_user_id = $_POST['userid']; // This could be Master or Client level (depending on where the password change was called)

		// IMPORTANT!!!!
		// If we are not connected to the Client Database, we are evidently running cr_invite
		if ((!isset($_SESSION['CLI_DB_CONNECTED'])) || (!$_SESSION['CLI_DB_CONNECTED']))
		{
			// Get the user ID for this user.  We need to connect to the Master Database to get the User Id, which we can in turn connect to the Client database with
			$connectedToClientDB = $database->selectClientDatabaseByUserId($client_user_id);
			// Find the client level user name
			$username = $database->getUserFromMasterByUserId($client_user_id);
			// Get the Client value for user id
			$userArray = $database->getUserInfo($username);
			
			// NOTE: Change from Master to Client User id
			$client_user_id = $userArray['u_id'];
		}
		
		// NOTE: We now have the user_id for the client database
		
		/* Password change attempt */
		$retval = $session->setnewpass($client_user_id,   // id, not user name
								   $_POST['pass'], 
								   $_POST['pass2']);
		/* Password change Successful */
		if($retval == 0)
		{
			// echo $session->referrer; die;
			$_SESSION['regsuccess'] = true;
			header("Location: ".$session->referrer);
		}
		/* Error found with form */
		else if($retval == 1)
		{
			$_SESSION['value_array'] = $_POST;
			$_SESSION['error_array'] = $form->getErrorArray();
			header("Location: ".$session->referrer);
		}
		/* Password change attempt failed */
		else if($retval == 2)
		{
			$_SESSION['regsuccess'] = false;
			$_SESSION['error_array'] = $form->getErrorArray();
			header("Location: ".$session->referrer);
		}
	}

   
/**
* procRegister - Processes the user submitted registration form,
* if errors are found, the user is redirected to correct the
* information, if not, the user is effectively registered with
* the system and an email is sent to the newly
* created user.
*/
	function procRegister()
	{
		global $session, $form, $database, $appCompanySvcCode;
		
		/* Convert username to all lowercase (by option) */
		if(ALL_LOWERCASE)
		{
			$_POST['user'] = strtolower($_POST['user']);
		}
		
		// We need to get the client from the installation
		$mClient_Id = $database->getClientIdByClientName($appCompanySvcCode);

		/* Registration attempt */
		$retval = $session->register($_POST['user'], 
								   $mClient_Id, 
								   $_POST['pass'], 
								   $_POST['pass2'], 
								   $_POST['namelast'], 
								   $_POST['namefirst'], 
								   "",  // Passnum - leave blank to not be validated  $_POST['passnum'], 
								   $_POST['email'],
								   REG_BY_SELF);

		/* Registration Successful */
		if($retval == 0)
		{
			$_SESSION['reguser'] 		= $_POST['user'];
			$_SESSION['regnamefirst'] 	= $_POST['namefirst'];
			$_SESSION['regsuccess'] = true;
			header("Location: ".$session->referrer);
		}
		/* Error found with form */
		else if($retval == 1)
		{
			$_SESSION['value_array'] = $_POST;
			$_SESSION['error_array'] = $form->getErrorArray();
			header("Location: ".$session->referrer);
		}
		/* Registration attempt failed */
		else if($retval == 2)
		{
			$_SESSION['reguname'] = $_POST['namefirst'];
			$_SESSION['regsuccess'] = false;
			header("Location: ".$session->referrer);
		}
	}

/**
* procRegisterInvite - Admin is registering a new user, and sending an invite
* if errors are found, the user is redirected to correct the
* information, if not, the user is effectively registered with
* the system and an email is sent to the newly
* created user.
*
* Note: we are only using EMAIL addresses for the user now.  All references to 'user' needs to be 'email'
*/
	function procRegisterInvite()
	{
		global $session, $form;
		
		/* Convert username to all lowercase (by option) */
		if (ALL_LOWERCASE)
		{
			$_POST['email'] = strtolower($_POST['email']);
		}

		$field = 'email';
		/* Registration attempt */
		// Note: ->register overrides 
		$retval = $session->register($_POST['email'], 
								   $_POST['client'], // @@@ This may be a concern.  Should we read the client from the current user signed in only?
								   "Invite000",  // Put a dummy password here.  User will not be able to sign on until verified
								   "Invite000", 
								   $_POST['namelast'], 
								   $_POST['namefirst'], 
								   "",  // Passnum - leave blank to not be validated
								   $_POST['email'],
								   REG_BY_INVITE);

		/* Registration Successful */
		if($retval == 0)
		{
			$_SESSION['reguname'] = $_POST['namefirst'];
			$_SESSION['regsuccess'] = true;
			header("Location: ".$session->referrer);
		}
		/* Error with email address, or email is already in use */
		else if($retval == 1)
		{
			// Errors were populated in session
			$_SESSION['value_array'] = $_POST;
			$_SESSION['error_array'] = $form->getErrorArray();

			$_SESSION['regsuccess'] = false;
			header("Location: ".$session->referrer);
		}
		/* Registration attempt failed */
		else if($retval == 2)
		{
			$form->setError($field, providePhrase('cr', 620)); // User could not be emailed
			$_SESSION['value_array'] = $_POST;
			$_SESSION['error_array'] = $form->getErrorArray();

			$_SESSION['regsuccess'] = false;
			header("Location: ".$session->referrer);
		}
		/* User creation failed */
		else if($retval == 3)
		{
			$form->setError($field, providePhrase('cr', 621)); // Username could not be created
			$_SESSION['value_array'] = $_POST;
			$_SESSION['error_array'] = $form->getErrorArray();
				
			$_SESSION['regsuccess'] = false;
			header("Location: ".$session->referrer);
		}
	} // procRegisterInvite
   
   
/**
* procExtAdd - Processes the user submitted telecommunications settings form,
* if errors are found, the user is redirected to correct the
* information, if not, the information is added
*/
	function procExtAdd()
	{
		global $database, $session, $form;
			  		
		if ($database->selectClientDatabaseByMasterClientId($_SESSION['userclientid']))
		{
			// Determine phone and type
			$mTelephoneNumber	= '';
			$mTelephoneType		= '';
			if (isset($_POST['cellphone']))
			{
				$mTelephoneNumber	= stripNonNumeric($_POST['cellphone']);
				$mTelephoneType		= 'CELL';			
			}
			else if (isset($_POST['telephone']))
			{
				$mTelephoneNumber	= stripNonNumeric($_POST['telephone']);
				$mTelephoneType		= 'LINE';			
			}
			
			// See if a prefix dial code was sent along
			$mPrefix = '';
			if (isset($_POST['user_phone_prefix']))
			{
				// Perhaps a connect dial code like 9 or * (for twityax)
				$mPrefix = $_POST['user_phone_prefix'];
			}
			
			// Test to see if we have been passed a name
			$mExtensionText		= $mPrefix;
			$mExtensionNumber	= $mPrefix;
			if (isset($_POST['user_phone_login']))
			{
				$mExtensionText		.= $_POST['user_phone_login'];
				$mExtensionNumber	.= TTLetterDigits($_POST['user_phone_login']);
			}
			else
			{
				// if a numeric value was passed
				if (isset($_POST['user_extension']))
				{
					$mExtensionNumber .= $_POST['user_extension'];
				}
			}
			
						
			/* Convert username to all lowercase (by option) */
			/* Registration attempt */
			$retval = $session->registerext($_SESSION["useruid"],
										  $mExtensionNumber,
										  $mTelephoneNumber);
			
			/* Registration Successful */
			if($retval == 0)
			{				
				// See if we need additional information for the first logon
				$userArray   = $database->getUserInfo($_SESSION['username']);
				//echo $req_user_info['u_telephone']; die;
				
				// See if we can default some information to the user record
				if ($userArray['u_telephone'] == '')
				{	
					// Goes into the main user record
					// Outside Telephone details
					$database->updateUserField($_SESSION['username'], 
												'u_telephone', 	
												$mTelephoneNumber);
					$database->updateUserField($_SESSION['username'], 
												'u_telephone_type', 	
												$mTelephoneType);
												
					// Inside connection details
					if ($mExtensionNumber <> '')
					{				
						// Save the number
						$database->updateUserField($_SESSION['username'], 
													'user_phone_login',	
													$mExtensionNumber);
						// Put in the name 
						$database->updateUserField($_SESSION['username'], 
													'user_phone_text',	
													$mExtensionText);
					}
					
					if (isset($_POST['passnum']))
					{				
						$database->updateUserField($_SESSION['username'], 
													'user_phone_pass',	
													stripNonNumeric($_POST['passnum']));
					}
				}			

				$_SESSION['regsuccess'] = true;
				header("Location: ".$session->referrer);				
			}
			/* Error found with form */
			else if($retval == 1)
			{
				$_SESSION['value_array'] = $_POST;
				$_SESSION['error_array'] = $form->getErrorArray();
				header("Location: ".$session->referrer);
			}
		}
	} // procExtAdd


   
/**
* procForgotPass - Validates the given username then if
* everything is fine, a new password is generated and
* emailed to the address the user gave on sign up.
*/
	function procForgotPass()
	{
		global $database, $session, $mailer, $form;
		
		/* Username error checking */
		$subuser = $_POST['user'];
		$field = "user";  //Use field name for username
		if(!$subuser || strlen($subuser = trim($subuser)) == 0)
		{
			$form->setError($field, providePhrase('cr', 622)); // Username not entered
		}
		else
		{
			/* Make sure username is in database */
			$subuser = stripslashes($subuser);
			if ((strlen($subuser) < 5) || 
				(strlen($subuser) > 30) ||
				(!eregi("^([0-9a-z])+$", $subuser)) ||
				(!$database->usernameTaken($subuser)))
			{
				$form->setError($field, providePhrase('cr', 623)); //Username does not exist
			}
		}

		/* Errors exist, have user correct them */
		if($form->num_errors > 0)
		{
			$_SESSION['value_array'] = $_POST;
			$_SESSION['error_array'] = $form->getErrorArray();
		}
		/* Generate new password and email it to user */
		else
		{
			/* Generate new password */
			$newpass = $session->generateRandStr(8);

			/* Get email of user */
			$usrinf = $database->getUserInfo($subuser);
			$email  = $usrinf['u_email'];

			/* Attempt to send the email with new password */
			if($mailer->sendNewPass($subuser,$email,$newpass))
			{
				/* Email sent, update database */
				$database->updateUserField($subuser, "password", md5($newpass));
				$_SESSION['forgotpass'] = true;
			}
			/* Email failure, do not change password */
			else
			{
				$_SESSION['forgotpass'] = false;
			}
		}

		header("Location: ".$session->referrer);
	}
   
   
/**
procEditAccount - Attempts to edit the user's account
information, including the password, which must be verified
before a change is made.
*/
	function procEditAccount()
	{
		global $session, $form;
		
		/* Account edit attempt */
		$retval = $session->editAccount($_POST['curpass'], $_POST['newpass'], $_POST['email'], $_POST['pass'], $_POST['telephone']);

		/* Account edit successful */
		if($retval)
		{
			$_SESSION['useredit'] = true;
			header("Location: ".$session->referrer);
		}
		/* Error found with form */
		else
		{
			$_SESSION['value_array'] = $_POST;
			$_SESSION['error_array'] = $form->getErrorArray();
			header("Location: ".$session->referrer);
		}
	}

/**
procEditAccountAdminUser - Attempts to allow an Admin to edit a user's account
information, including the password, which must be verified before a change is made.
*/
	function procEditAccountAdminUser()
	{
		global $session, $form;
		
		/* Account edit attempt */
		$retval = $session->editAccountAdminUser($_POST['user'], $_POST['newpass'], $_POST['email'], $_POST['pass'], $_POST['telephone']);

		/* Account edit successful */
		if($retval)
		{
			$_SESSION['usereditadmin'] = true;
			header("Location: ".$session->referrer);
		}
		/* Error found with form */
		else
		{
			$_SESSION['value_array'] = $_POST;
			$_SESSION['error_array'] = $form->getErrorArray();
			// Return the user that we are editing in the get field, so that repopulation occurs for that person
			header("Location: ".$session->referrer."?user=".$_POST['user']);
		}
	}

	
};

/* Initialize process */
$process = new Process;

?>
