<?php
/*  cr_media_file_positioner.php

This process performs the following functions:

* Scan local files for XML - This is written when a WAV/GSM recording is completed
** eg: C:\TeleFlowRelay\Media\client_00002\MSG-2009-12-01-1259698004.310-URANUS-3.GSM.NEW.XML
* Determine the Wav recording file name 
** eg: MSG-2009-12-01-1259698004.310-URANUS-3.GSM
* Determine date named folder based on file name
** 2009-12-01
* See if the remote date named folder exists (and create if not)
** R:\callcapture_test\KORE\2009-12-01
* Copy local XML, and GSM to remote file (delete first if already exist)
* If successful, see if local move folder exists (and create if not)
** eg: C:\TeleFlowRelay\Media\client_00002\2009-12-01
* Move local XML, and GSM to move folder (delete first if already exist)
* Sleep for 30 seconds
*/

// Determine relative location for paths
require_once("cr_reference.php");
require_once($incPath.'LocalSettings.php'); 
require($incPath.$sessionphp);

require_once($websiteLibrary.'calcWorkUnit.php'); 
// Media
require_once($websiteLibrary.'mediaWorkUnit.php'); 
// Post using stream
//require_once($websiteLibrary.'httpWorkUnit.php');     




/////////////////////////////////////////
// Settings for Media Management
require_once($incPath.'../LocalSettings_MediaManager.php'); 
/////////////////////////////////////////


		
// Simple functions
function output($pLine)
{
	global $incPath;
	
	fwrite(STDOUT, $pLine."\n"); // echo
	//LogTextToFile('cr_transfer_log.log', $pLine."\n");	
	LogTextToDateNamedFileSystem($incPath, 'cr_media_file_positioner', $pLine."\n");	
}


output ("-------------------------------");
output ("Starting File Positioner Process");
output ("-------------------------------");

// Get the computer name - eg:  JUPITER
$mCompName = getHostComputerName(); //php_uname('n');  
output ("Search for files created by ".$mCompName);


$mMediaFolder = mediaFileFolderLocate(false, $mClientId, '');
output ("Directory: ".$mMediaFolder);


$mLoop = 0;
// Loop the entire file
do // forever
{
	// Sleep every loop
	if ($mLoop <> 0)
	{
		sleep($cWaitLoop);
	}
	// Count up
	$mLoop++;


	// Catch any problems: Namely:  "MySQL server has gone away"
	try
	{

		// Set the Files Array to keep track what to delete at the end
		$mFilesTempArray 		= array();
		$mFilesTempCount 		= 0;
		// Array for files used when send via SSLNR
		$mFilesTransferArray	= array();	
		$mFilesTransferCount 	= 0;

		// Leave a break for readability
		output(' '); 

		output('- Testing for files at '.teleflowNowDate().' '.teleflowNowTime());
			
		// See if there are files to load
		if ($mClientMediaNewXmlYN)
		{
			// Determine the folder in which the file should be found (do not create, and in the client_id, root)
			$mMediaFolder = mediaFileFolderLocate(false, $mClientId, '');
			
			output('- XML Reading: '.$mMediaFolder);
			
			// Define where the *.NEW.XML files are to be found (if any)
			$dir = opendir($mMediaFolder);

			$mFileFound = false;
			while (($file = readdir($dir)) !== false)
			{
				// Only files, no dirs
				if (!is_dir($file))
				{
					$mFileFound = true;
					$fs=filesize($mMediaFolder."/". $file);
					$moddate=date('F j, Y',filemtime($mMediaFolder."/". $file));
					
					// Do we have a new file?
					if (rightstr($file,8) == '.NEW.XML')
					{
						output('- Processing XML Media: '.$file);
						
						// Get the create date from the name
						$mParts = explode('_',trim($file));
						
						// Get the count
						$mCount = 0;
						foreach ($mParts as $mEachOne)
						{
							$mCount++;		
						}
						
						// Do we have a file with the number of parts in it?
						//      0        1               2                        3            4
						// eg: MSG_2009-12-01_1259698004.310_URANUS_3.GSM.NEW.XML
						if ($mCount > 1)
						{
							// Get date folder for file
							$mDateFolder = $mParts[1];
							// Get the recording
							// eg: MSG_2009-12-01_1259698004.310_URANUS_3.GSM
							// Get the extentions
							$mExtensionParts = explode('.',$mParts[4]);
							$mExtAdd = $mExtensionParts[0].'.'.$mExtensionParts[1];
							$mWavRecording = $mParts[0].'_'.$mParts[1].'_'.$mParts[2].'_'.$mParts[3].'_'.
											$mExtAdd;
							//echo ($mWavRecording) ; die;
						
							//////////////////////////////
							// REMOTE
							//////////////////////////////
							// Create the path
							if (!is_dir($mRemoteMappedDrive.$mCompName))
							{
								output('- Creating Remote Path: '.$mRemoteMappedDrive.$mCompName );
								make_path($mRemoteMappedDrive.$mCompName);	
							}
							
							$mRemotePath = $mRemoteMappedDrive.$mCompName."/".$mDateFolder."/";
							if (!is_dir($mRemotePath))
							{
								output('- Creating Remote Path: '.$mRemotePath );
								make_path($mRemotePath);
							}
							
							// Delete file on remote system / in case another attempt is being made to fix a problem etc
							if (file_exists($mRemotePath."/".$file))
							{
								unlink($mRemotePath."/".$file);
							}				
							if (file_exists($mRemotePath."/".$mWavRecording))
							{
								unlink($mRemotePath."/".$mWavRecording);
							}				
							
							
							// Copy the files 
							if (copy($mMediaFolder."/".$file,
									$mRemotePath."/".$file)
								&&
								copy($mMediaFolder."/".$mWavRecording,
									$mRemotePath."/".$mWavRecording))
							{											
								//////////////////////////////
								// LOCAL -- Success, so move 
								//////////////////////////////
								$mLocalMovePath = $mMediaFolder."/". $mDateFolder;
								
								// Do we need to create the directory?
								if (!opendir($mLocalMovePath))
								{
									output('- Creating Local Path: '.$mLocalMovePath );
									make_path($mLocalMovePath);
								}
								// Copy it over.
								// -- If successful, move it locally
								output('- Move: '.$mMediaFolder."/".$file);
								output('--- To: '.$mLocalMovePath."/".$file);
									
								// Delete file on local move system / in case another attempt is being made to fix a problem etc
								if (file_exists($mLocalMovePath."/".$file))
								{
									unlink($mLocalMovePath."/".$file);
								}				
								if (file_exists($mLocalMovePath."/".$mWavRecording))
								{
									unlink($mLocalMovePath."/".$mWavRecording);
								}				
									
								// Move the file to a new place
								rename( $mMediaFolder."/".$file, 
										$mLocalMovePath."/".$file);	
								rename( $mMediaFolder."/".$mWavRecording, 
										$mLocalMovePath."/".$mWavRecording);	
								output('-- Successfully processed and moved');
								
							}
						}
						
						
						/*
						// Load the file into a file, and then post it
						$dataLoad  = file_get_contents($mMediaFolder."/". $file);
						if ($dataLoad)
						{
							// Prepare the file, and then move it to a backup (see MediaWorkUnit)
							$mMediaFolderDated = mediaFileFolderLocate(true, $mClientId, teleflowNowDate());
							
							// If file exists, rename 
							if (file_exists($mMediaFolderDated."/".$file))
							{
								unlink($mMediaFolderDated."/".$file);
							}				

							// Copy the file to 

							
						}
						*/
					}
				}
			}
			closedir($dir);					
		}
				
				
		// Test to see if we want to delete
		if ($mClientMediaDeleteYN)
		{
			// Get rid of the Date Named Files in the media folder
			// eg: MSG_2009-12-04_1259885485.791_URANUS_1.GSM
			deleteDateNamedFiles($mMediaFolder,1,$mDeleteFilesAtDays);
			// Destroy the entire date named folder at a later time
			// eg: 2009-09-24
			deleteDateFolders($mMediaFolder,0, $mDeleteFoldersAtDays);

		} //  if

		output (" "); // A blank space

		// WARNING: Break the loop if this is wrapped by a service (svcany)
		sleep($cWaitLoop);
		break;
		
	}
	catch(Exception $e)
	{
		output (" "); // A blank space
		output ("ERROR Encountered");
		output ("* Message: ".$e->getMessage());
		output ("* Waiting to continue loop...");
		sleep($cWaitLoop);
		output (" "); // A blank space
		break;
		
	}


} while (TRUE);
  
output ("Done! \n");
?>