<?php
/*  cr_media_file_importer.php

This process performs the following functions:
* For each client with MEDIA_XFER as a subscribed application
* Scan the remote date named folders (created by cr_media_file_positioner) for XML statement files  (NEW.XML)
* Will scan through last couple day of folders (set by a variable)
** eg: 2009-12-01, 2009-12-02, 2009-12-03
** Note: Could run x days on the first loop, and then 2 days afterward to ensure that if a system was down, that it pick up older files
* Create 'xml_processed' folder in the date named folder if not found
* Read contents of each .NEW.XML file
** Load them into the database (using xml_media_publish_body.php)
* Rename NEW.XML to the 'xml_processed' folder in the date named folder
* Sleep for 30 seconds, and repeat (or die if this is wrapped by a command file which has been serviced)

*/


// Determine relative location for paths
require_once("cr_reference.php");
require_once($incPath.'LocalSettings.php'); 
require($incPath.$sessionphp);

require_once($websiteLibrary.'calcWorkUnit.php'); 
// Media
require_once($websiteLibrary.'mediaWorkUnit.php'); 



/////////////////////////////////////////
// Settings for Media Management
require_once($incPath.'../LocalSettings_MediaManager.php'); 
/////////////////////////////////////////




// Post using stream
//require_once($websiteLibrary.'httpWorkUnit.php');     
require_once('xml_media_publish_functions.php');

/////////////////////////// XML_MEDIA_PUBLISH BODY ///////////////////////////////
// NOTE: This is a little trick to create a function, whic  then works with the value in $data
function xml_media_publish($data)
{
	global $database;
	// All the contents become the function
	// - Note: requires a decho function
	include('xml_media_publish_body.php');
	$doc->loadXML($domobject->saveHTML());
	if($doc->getElementsByTagName('SUCCESS')->item(0))
	{
		// Element was found, so we are good
		return true;
	}
	return false;
} // xml_media_publish
/////////////////////////// XML_MEDIA_PUBLISH BODY ///////////////////////////////


		
// Simple functions
function output($pLine)
{
	global $incPath;
	
	fwrite(STDOUT, $pLine."\n"); // echo
	//LogTextToFile('cr_transfer_log.log', $pLine."\n");	
	LogTextToDateNamedFileSystem($incPath, 'cr_media_file_importer', $pLine."\n");	
}


output ("-------------------------------");
output ("Starting File Import Process");
output ("-------------------------------");

// Get the computer name - eg:  JUPITER
$mCompName = getHostComputerName(); //php_uname('n');  
output ("Search for files created by ".$mCompName);


$mMediaFolder = mediaFileFolderLocate(false, $mClientId, '');
output ("Directory: ".$mMediaFolder);


$mLoop = 0;
// Loop the entire file
do // forever
{
	// Count up
	$mLoop++;

	
	// Catch any problems: Namely:  "MySQL server has gone away"
	try
	{

		//////////////////////////////////////
		// Find all clients that are active, and 
		//////////////////////////////////////
		$q =
"SELECT 
	a.client_id client_id,
	client_name,
	app_type_name,
	subscription_status
FROM 
	".$database->sqlConvert_FROM($database->DATABASE_MASTER,"client a").",
	".$database->sqlConvert_FROM($database->DATABASE_MASTER,"client_application_subscription b")."
WHERE
	a.client_id = b.client_id
AND a.client_status = 'ACTIVE'
AND b.subscription_status = 'ACTIVE'
AND app_type_name like '".PROCESSTYPE."'
ORDER BY client_id
";		
		//echo $q; die;

		$database->switchToClientDatabase();
		
		// Do we have any clients needing processsing
		$clientResult = $database->sqlConvert_sql_query($q, $database->connectionSystem);
		if (!$clientResult || ($database->sqlConvert_sql_numrows($clientResult) < 1))
		{
			output("* ".teleflowNowTimeStamp()."(".$mLoop."): No clients are current set with ".PROCESSTYPE." to ACTIVE. Sleeping ".$cWaitLoop." seconds...");
			sleep($cWaitLoop);
		}
		else
		{
			output("* ".$mLoop.": Found ".$database->sqlConvert_sql_numrows($clientResult)." clients with ".PROCESSTYPE." as ACTIVE.");
			output(" "); // A blank space
			
			//////////////////////////////////////
			//  MEDIA - For Each Client
			//////////////////////////////////////
			while (($row_rsClientResult = $database->sqlConvert_sql_fetch_array($clientResult)))
			{		
				// Array for files found to process
				$mFilesTransferArray	= array();	
				$mFilesTransferCount 	= 0;
				
				// Remember the client
				$mClientId = $row_rsClientResult['client_id'];
				
				// Leave a break for readability
				output(' '); 
			
			
				//////////////////////////////////////
				//  SELECT CLIENT DATABASE
				//////////////////////////////////////		
				output('Client: '.$mClientId.' - '.$row_rsClientResult['client_name']);		
				if ($database->selectClientDatabaseByMasterClientId($mClientId))
				{
					output('- Selected database & testing for remote NEW.XML files');

					// Loop back a couple of folder directories
					// @@@ perhaps we should have a last run option, to go back a few days if the system was turned off for some reason - Or, first loop, goes back 14 days.
					for($mLoop=-1;$mLoop<2;$mLoop++)
					{
						$mDateFolder = teleflowDateAdd(teleflowNowDate(), 0-$mLoop);
						$mRemotePath = $mRemoteMappedDrive.$mCompName."/".$mDateFolder."/";
						output('- Scanning: '.$mRemotePath);
					
						if (is_dir($mRemotePath))
						{
							// Load the files to process into an array
							$dir = opendir($mRemotePath);
							
							$mFileFound = false;
							while (($file = readdir($dir)) !== false)
							{
								// Only files, no dirs
								if (!is_dir($file))
								{
									// Do we have a new file?
									if (rightstr($file,8) == '.NEW.XML')
									{
										output('- Located XML Media: '.$file);
										
										// Track the file_id in order to update that the transfer occurred
										$mFilesTransferArray[$mFilesTransferCount] = $mRemotePath.$file;
										$mFilesTransferCount++;
								
									}
								}
							}					
							// And close it
							closedir($dir);	
						}
					} // For last couple days
					
					
					// Run through the list of files that are found
					foreach ($mFilesTransferArray as $file)
					{					
						// Do we have a new file?
						if (rightstr($file,8) == '.NEW.XML')
						{
							// Get the path details about the file.
							$path_parts = pathinfo($file);
						
							$mFileName 	= getFileNameFromPath($file);
							$mRemotePath = $path_parts['dirname'].'/';
						
							output('- Processing Path: '.$mRemotePath);
						
							output('- Processing XML Media: '.$mFileName);
							
							// Get the create date from the name
							$mParts = explode('_',trim($mFileName));
							
							// Get the count
							$mCount = 0;
							foreach ($mParts as $mEachOne)
							{
								$mCount++;		
							}
							
							// Do we have a file with the number of parts in it?
							//      0        1               2                        3            4
							// eg: MSG_2009-12-01_1259698004.310_URANUS_3.GSM.NEW.XML
							if ($mCount > 1)
							{
								// Get date folder for file
								$mDateFolder = $mParts[1];
								// Get the recording
								// eg: MSG_2009-12-01_1259698004.310_URANUS_3.GSM
								// Get the extentions
								$mExtensionParts = explode('.',$mParts[4]);
								$mExtAdd = $mExtensionParts[0].'.'.$mExtensionParts[1];
								$mWavRecording = $mParts[0].'_'.$mParts[1].'_'.$mParts[2].'_'.$mParts[3].'_'.
												$mExtAdd;
												
								//echo ($mRemotePath."/".$mFileName.addCRLF()) ;
								//echo ($mRemotePath."/".$mWavRecording.addCRLF()) ;
						
								// If both exist, load them
								if ((file_exists($mRemotePath."/".$mFileName)) &&
									(file_exists($mRemotePath."/".$mWavRecording)))
								{						
									// A folder to move the NEW.XML 
									$mRemotePathProcessed = $mRemotePath.'xml_processed/';
									if (!is_dir($mRemotePathProcessed))
									{
										output('- Creating Remote Path: '.$mRemotePathProcessed );
										make_path($mRemotePathProcessed);
									}
																
									output('- Importing XML Media: '.$mFileName);
									
									// Run POST XML / write to databaes
									$dataLoad  = file_get_contents($mRemotePath."/". $mFileName);
									if ($dataLoad)
									{
										// Create a media entry
										//output('-- Create: '.$dataLoad);
									
										// Build on the action
										$dataCmd = '<REQUEST action="MediaFileInsertMediaEntry" client_id="'.$mClientId.'"  media_folder_name="'.$mDateFolder.'">'.
												xml_sanitize($dataLoad).
												'</REQUEST>';
										//echo $dataCmd; die;
										
										// If we successfully created a media entry, then move the file
										if (xml_media_publish($dataCmd))
										{
											// Move the file to a new place
											rename( $mRemotePath."/".$mFileName, 
													$mRemotePathProcessed."/".$mFileName);	
											output('-- Successfully processed and moved to: '.$mMediaFolderDated);
										}
										else
										{
											output('- ERROR: Could not create Media entry for: '.$mFileName);
										}
									}
									
									
								}				

							}
						}					
					}
				}
			}  // While each client

		}
				
		output (" "); // A blank space


		// WARNING: Break the loop if this is wrapped by a service (svcany)
		sleep($cWaitLoop);
		break;
	
	
	//catch exception
	}
	catch(Exception $e)
	{
		output (" "); // A blank space
		output ("ERROR Encountered");
		output ("* Message: ".$e->getMessage());
		output ("* Waiting to continue loop...");
		sleep($cWaitLoop);
		output (" "); // A blank space
	}


} while (TRUE);
  
output ("Done! \n");
?>