<?php
/*  cr_media_file_transfer.php

Used to convert and transfer media files ( recordings) to their destination
eg:
* Dictations
* Voice Messages (should be added here)
* Call Recordings (should be added here)

*/

// Determine relative location for paths
require_once("cr_reference.php");
require_once($incPath.'LocalSettings.php'); 
require($incPath."include/session.php");
require_once($websiteLibrary.'calcWorkUnit.php'); 
// Media
require_once($websiteLibrary.'mediaWorkUnit.php'); 
// Post using stream
require_once($websiteLibrary.'httpWorkUnit.php');     

/////////////////////////////////////////
// Settings for Media Management
require_once($incPath.'../LocalSettings_MediaManager.php'); 
/////////////////////////////////////////

//////////////////
// Notes
//////////////////
/*
REQUIRED:  The File MUST be named
-- 4_ZZZZ_6046382650_2000_Spears_032408_060329_0_065423_orig.wav


-- xxx_orig.wav is created by TeleFlow Dictation environment
-- xxx.wav is the GSM (compressed file) that is created by this software
-- xxx.wav is renamed to xxx.wa1
-- xxx.wa1 is ftp'd to the remote location
-- xxx.wa1 size local and remote is compared
if the size is the same, then
-- xxx.wa1 is renamed to xxx.wav remote
else
-- xxx.wa1 is deleted remote
end
-- xxx.wa1 is deleted local

*/

//////////////////
// Settings
//////////////////

// Note: These values should eventually come from a database or algorithm that determines what to do

// Transfer types
define("FTPLOCATION",	'FTPLOCATION');	// FTP to other company
define("WEB",			'WEB');				// FTP to our web environment
define("TELEFLOWRELAY",	'TELEFLOWRELAY');	// HTTP Post to a TeleFlowRelay host


// FTP listing fields
define("FTPACCESS", 	0);
define("FTPLEVEL",  	1);
define("FTPOWNER",  	2);
define("FTPGROUP",  	3);
define("FTPSIZE",   	4);
define("FTPTIME1",  	5);
define("FTPTIME2",  	6);
define("FTPTIME3",  	7);
define("FTPFILENAME", 	8);

// === Constants ===
// === Constants ===
// -- See operationconstants for details


// How many files are we allowed to process in a cycle
// -- Only do 20 at a time so that we are not overwhelming the server
$cProcessMAX 			= 20;  
$mProcessLimit 			= $cProcessMAX;  
$cWaitLoop				= 5;	// Time for next SQL check
		
// Simple functions
function output($pLine)
{
	global $incPath;
	
	fwrite(STDOUT, $pLine."\n"); // echo
	//LogTextToFile('cr_transfer_log.log', $pLine."\n");	
	LogTextToDateNamedFileSystem($incPath, 'cr-recording-transfer', $pLine."\n");	
}


output ("-------------------------------");
output ("Starting PHP transfer process");
output ("-------------------------------");

// Get the computer name - eg:  JUPITER
$mCompName = getHostComputerName(); //php_uname('n');  
output ("Search for files created by ".$mCompName);

// Create the path for the temporary FTP command files
// Location for database
$commandFilesPath = $incPath.'../'.SYSTEM_TEMPFILES.'/';
if (!file_exists($commandFilesPath))
{
	// Create path if not exist
	make_path($commandFilesPath);
}
output("FTP Command Files created here: ".$commandFilesPath);

$mLoop = 0;
// Loop the entire file
do // forever
{
$mLoop++;

// Catch any problems: Namely:  "MySQL server has gone away"
try
{


//////////////////////////////////////
// Find all clients that have files that may need to have media file transfers done
//////////////////////////////////////
$q =
"SELECT 
	a.client_id client_id,
	client_name,
	app_type_name,
	subscription_status
FROM 
	".DATABASE_MASTER.".client a,
	".DATABASE_MASTER.".client_application_subscription b
WHERE
	a.client_id = b.client_id
AND a.client_status = 'ACTIVE'
AND b.subscription_status = 'ACTIVE'
AND app_type_name like '".PROCESSTYPE."'
AND ((last_data_change_dtm_utc > last_data_scan_dtm_utc) or (last_data_scan_dtm_utc is null))
";		
//echo $q; die;

// Do we have any clients needing processsing
$clientResult = mysql_query($q, $database->connectionSystem);
if (!$clientResult || (mysql_numrows($clientResult) < 1))
{
	output("* ".teleflowNowTimeStamp()."(".$mLoop."): No clients with ".PROCESSTYPE." activity. Sleeping ".$cWaitLoop." seconds...");
	sleep($cWaitLoop);
}
else
{
	output("* ".$mLoop.": Found ".mysql_numrows($clientResult)." clients with ".PROCESSTYPE." activity.");
	output(" "); // A blank space
	
	//////////////////////////////////////
	//  MEDIA - For Each Client
	//////////////////////////////////////
	while (($row_rsClientResult = mysql_fetch_assoc($clientResult)))
	{
		// Clear information
		$ftp_dest_server    	= '';
		$ftp_dest_user_name 	= '';
		$ftp_dest_user_pass 	= '';
		$ftp_dest_folder    	= '';
		$ftp_dest_SSL			= false;
		$ftp_web_server     	= '';
		$ftp_web_user_name  	= '';
		$ftp_web_user_pass  	= '';
		$ftp_web_folder     	= '';
		$ftp_web_SSL			= false;
		
		// Set the Files Array to keep track what to delete at the end
		$mFilesTempArray 		= array();
		$mFilesTempCount 		= 0;
		// Array for files used when send via SSLNR
		$mFilesTransferArray	= array();	
		$mFilesTransferCount 	= 0;
		
		// Remember the client
		$mClientId = $row_rsClientResult['client_id'];
		
		// Leave a break for readability
		output(' '); 

		//////////////////////////////////////
		//  SELECT CLIENT DATABASE
		//////////////////////////////////////		
		output('Client: '.$mClientId.' - '.$row_rsClientResult['client_name']);		
		if ($database->selectClientDatabaseByMasterClientId($mClientId))
		{
			output('- Selected database & marking scan');
			
			// Mark that we did a scan 
			$database->systemDatabaseLastScannedDate($mClientId, PROCESSTYPE);
			
			
			//////////////////////////////////////
			//  Find all Media to process
			//////////////////////////////////////
			// Rules 
			// - The medai file is initially set to CREATED
			// - The media file is set to NEW for the file_app_status once the use has indicated the file is ready for processing
			// - Once file_app_status is NEW, then the file_xfer status becomes the trigger for the rest of the processing
			$query_statement = (sprintf(
"SELECT 
	fat_id,
	fat_app_type_name, 
	fat_wav_type,
	fat_gsm_type,
	fat_mp3_type,
	media_file_id,
	media_owner_u_id,
	media_file_directory,
	media_file_name, 
	file_reference_id, 
	app_type_name,
	file_duration,
	file_size
FROM
	file_application_transfer,
	media_file
WHERE
	(fat_status 			= 'ACTIVE')
AND (fat_app_type_name		= app_type_name)
AND (file_app_status 		= '%s') 
AND (file_transfer_status 	= '%s') 
AND (ivr_computer_name  	= '%s')", FA_NEW, FT_NEW, $mCompName));
			//output($query_statement);	die; // Read them 
			
			$rsSearchResult = mysql_query($query_statement); // or die(mysql_error());
			$totalRows_rsSearchResult = mysql_num_rows($rsSearchResult);  
			output ("- Total media recordings for processing on ".$mCompName." = ".$totalRows_rsSearchResult);
			
			// Only process if we have something to do
			if ($totalRows_rsSearchResult > 0)
			{
				// Mark that we only do so much work per client per cycle
				$mProcessLimit 			= $cProcessMAX;  
				
				// For each media file to be processed...				
				while (($row_rsSearchResult = mysql_fetch_assoc($rsSearchResult)) && ($mProcessLimit > 0))
				{   
					// Get the fields from file_application_transfer
					$mFat_id					= $row_rsSearchResult['fat_id'];
					$mFat_app_type_name			= $row_rsSearchResult['fat_app_type_name'];
					$mFat_wav_type				= $row_rsSearchResult['fat_wav_type'];			
					$mFat_gsm_type				= $row_rsSearchResult['fat_gsm_type'];			
					$mFat_mp3_type 				= $row_rsSearchResult['fat_mp3_type']; 			

					// Get the fields from media_file
					$mMedia_file_id				= $row_rsSearchResult['media_file_id'];
					$mMedia_owner_u_id			= $row_rsSearchResult['media_owner_u_id'];
					$mMedia_file_directory		= $row_rsSearchResult['media_file_directory'];
					$mMedia_file_name			= $row_rsSearchResult['media_file_name'];
					$mFile_reference_id			= $row_rsSearchResult['file_reference_id'];
					$mApp_type_name				= $row_rsSearchResult['app_type_name'];
					$mFile_duration				= $row_rsSearchResult['file_duration'];
					$mFile_size					= $row_rsSearchResult['file_size'];

					
					// Determine the folder in which the file should be found
					$mMediaFolder = mediaFileFolderLocate(false, $mClientId, $mMedia_file_directory);
					// The original dictation recording from TeleFlow
					$fileWAV   = $mMediaFolder.'/'.$mMedia_file_name;
					
					// Remove if exists
					if (file_exists($fileWAV))
					{
						output("-- File review for ".$mFat_app_type_name);
					
						// We are processing this, so reduce the count
						$mProcessLimit = $mProcessLimit -1;
					
						// Determine filenames for compressed versions
						$fileMP3   = getMediaFileNameMP3($fileWAV); 
						$fileGSM   = getMediaFileNameGSM($fileWAV); 
						
						// Test conversions to ensure they worked 
						$SuccessConversion = true;
						
						// Convert the files as needed
						// - MP3 can be done now and transfered in a later loop
						if (($mFat_mp3_type == 'YES') && ($SuccessConversion))
						{
							output("-- Converting to MP3: ". $mMedia_file_name);
					
							$cmd = ('lame\lame -h -V 4 '.$fileWAV.' '.$fileMP3);
							output('--Converting using: '.$cmd);
							$result = shell_exec($cmd);		

							// Did we get it?
							$SuccessConversion = (file_exists($fileMP3));
						}
						
						// for FTP formats, we will convert to GSM and start processing right away
						if (($mFat_gsm_type == 'YES') && ($SuccessConversion))
						{
							output("--Converting to GSM: ". $mMedia_file_name);

							// CONVERT: Convert the WAV FAT file to WAV GSM  (also see SOX:  http://sox.sourceforge.net/)
							$cmd = ('libsndfile\sndfile-convert.exe -gsm610 '.$fileWAV.' '.$fileGSM);
							output('--Converting using: '.$cmd);
							$result = shell_exec($cmd);
							
							// If the file wasnt created, then, copy the original in order to send the fat wave at least
							if (!file_exists($fileGSM)) 
							{
								// Copy file for transfer 
								copy($fileWAV, $fileGSM);
							}
							else
							{
								// Check to see if the file is big enough
								clearstatcache();
								if (filesize($fileGSM) < 1000)
								{
									// Copy the original over top the (too small to bother transferring) file.
									copy($fileWAV, $fileGSM);			
								}
							}

							// Did we get it?
							$SuccessConversion = (file_exists($fileMP3));							
						}

						// If all files converted correctly, then insert rows into media_file_transfer
						if ($SuccessConversion)
						{
							// Remove any old (failed) entries
							$delete_statement = (sprintf(
"DELETE FROM 
	media_file_transfer
WHERE 
	fat_id = '%s'
AND media_file_id = '%s'
", $mFat_id, $mMedia_file_id));
							$res = mysql_query($delete_statement);
							
							// Insert the new entry
							$insert_statement = (sprintf(
"INSERT INTO media_file_transfer
(
	fat_id,
	media_file_id,
	mft_status
)
VALUES
(
	'%s',
	'%s',
	'%s'
)", $mFat_id, $mMedia_file_id, FT_NEW));
							if ($res = mysql_query($insert_statement))
							{
								////////////////////
								// SUCCESS
								////////////////////
								// We are good to go
								$mUpdateCode = FT_PROCESSING;	
								output('--SUCCESS: Marked Media File Id for processing: '.$mMedia_file_id);
							}
							else
							{
								$mUpdateCode = FT_ERROR;	
								output("--ERROR: Inserting to 'media_file_transfer' of fat_id: ". $mFat_id);
								output($insert_statement);
							}
						}
						else
						{
							$mUpdateCode = FT_ERROR;	
							output("--ERROR: Conversion failed for: ". $fileWAV);						
						}
					}	
					else
					{
						// Failure because we have nothing to process.  
						$mUpdateCode = FT_MISSING;
						output('--ERROR: Missing file for Media File Id: '.$mMedia_file_id);
						output(filePathForOS($mMediaFolder.'/'.$mMedia_file_name));
					} // if (file_exists($fileWAV))
					
					// Update the details in Database
					$result = $database->mediaEntryUpdate($mClientId, 
								$mMedia_file_id,
								'', // no need to update file_app_status
								$mUpdateCode,
								$mFile_duration,
								$mFile_size);
					
				} // while
			} // if ($totalRows_rsSearchResult > 0)

			
			//////////////////////////////////////
			//  START: Transfer Loop: Process files for each transfer to location
			//////////////////////////////////////
			$query_transfer = (			
"SELECT
	fat_id,
	fat_wav_type, 
	fat_gsm_type, 
	fat_mp3_type, 
	fat_dest_type, 
	fat_dest_host_name, 
	fat_dest_ssl_type, 
	fat_dest_user_name, 
	fat_dest_user_pass, 
	fat_dest_folder
FROM
	file_application_transfer
WHERE
	fat_status = 'ACTIVE'");

			$rsTransferResult = mysql_query($query_transfer); // or die(mysql_error());
			$totalTransferRows_rsSearchResult = mysql_num_rows($rsTransferResult);  
			output ("MEDIA TRANSFER TO LOCATIONS: = ".$totalTransferRows_rsSearchResult);
			
			// For each transfer to location...
			while (($row_rsTransferResult = mysql_fetch_assoc($rsTransferResult)))
			{   
				// For each media to be processed...
				// -- Only do 20 at a time so that we are not overwhelming the server
				$mProcessLimit 	= $cProcessMAX;  

				// Determine what work we need to do
				$mTransferTeleFlowRelay 	= false;
				$transferToTeleFlowRelay	= 0;
				$mTransferFTPLocation		= false;
				$transferToFTPLocation		= 0;

				// Type of transfer for FTP (put only is drop and forget, the simple way(
				$ftp_put_only	= false;
				
			
			
				// Get the fields from the fat talbe
				$mFat_id				= $row_rsTransferResult['fat_id'];			
				$mFat_wav_type			= $row_rsTransferResult['fat_wav_type'];			
				$mFat_gsm_type			= $row_rsTransferResult['fat_gsm_type'];			
				$mFat_mp3_type 			= $row_rsTransferResult['fat_mp3_type']; 			
				$mFat_dest_type 		= $row_rsTransferResult['fat_dest_type']; 		
				$mFat_dest_host_name	= $row_rsTransferResult['fat_dest_host_name'];	
				$mFat_dest_ssl_type		= $row_rsTransferResult['fat_dest_ssl_type'];		
				$mFat_dest_user_name	= $row_rsTransferResult['fat_dest_user_name'];	
				$mFat_dest_user_pass 	= $row_rsTransferResult['fat_dest_user_pass'];
				$mFat_dest_folder      	= $row_rsTransferResult['fat_dest_folder'];

				//////////////////////////////////////
				//  TRANSFER LOCATION - For Each Transfer Record
				//////////////////////////////////////
				// For FTPLOCATION, set up the initial command block for FTP connection, and location to folder etc
				
				// Each row will be one of WEB, FTPLOCATION
				if ($mFat_dest_type == FTPLOCATION)
				{
					$mTransferFTPLocation = true;

					// Determine what to do
					output('- [X] - Processing '.$mFat_dest_type);
					// FTPLOCATION transfer			
					$ftp_dest_server        = $mFat_dest_host_name;
					$ftp_dest_user_name     = $mFat_dest_user_name;
					$ftp_dest_user_pass     = $mFat_dest_user_pass;
					$ftp_dest_folder        = $mFat_dest_folder;
					$ftp_dest_SSL			= false;
					if ($mFat_dest_ssl_type == 'YES')
					{
						$ftp_dest_SSL = true;				
					}
					else if ($mFat_dest_ssl_type == 'SSLNR')  // SSL No Read
					{
						$ftp_dest_SSL = true;				
						$ftp_put_only = true;				
					}

					// Create the command for the Transcriptist FTP site 
					$ftpFileFTPLOCATION  = $commandFilesPath.
											PROCESSTYPE.'_transfer_FTPLOCATION_'.
											$row_rsClientResult['client_id'].'.ftp';
					$ftpTextFTPLOCATION  = 'passive'.chr(13).chr(10);
					$ftpTextFTPLOCATION .= 'bin'.chr(13).chr(10);
					// Get to the folder where the files are to be worked from
					$goFolderList = explode("/", $mFat_dest_folder );
					foreach ($goFolderList as $goFolder)
					{
						if ($goFolder <> '')
						{
							$ftpTextFTPLOCATION .= 'cd '.$goFolder.chr(13).chr(10);
						}
					}	
					
				}
				// HTTP post
				else if ($mFat_dest_type == TELEFLOWRELAY)
				{
					// Web transfer determined
					$mTransferTeleFlowRelay = true;
					
					// Determine what to do
					output('- [X] - Processing '.$mFat_dest_type);
					
					// Web server php for POST transfer to
					$ftp_web_server         = $mFat_dest_host_name;	
				}

				
				//////////////////////////////////////
				//  Pass 1: Set up for FTP connection, and get list of all files
				//////////////////////////////////////
				// Note: This first pass allows us to build the scripts for en-mass transfers to FTP locations.
				// -- The same query will be run a second time to do the transfer for one by one, or en-mass
				// -- Copy file to transfer name for FTP
				// -- Transfer for FTP
				
				// Now, process the files for the fat
//////////////////// USED TWICE:  2 Passes ///////////////////////				
				$query_statement = (sprintf(			
"SELECT            
	a.media_file_id media_file_id,
	media_owner_u_id,
	media_file_directory,
	media_file_name,
	app_type_name, 
	media_file_type,
	file_duration, 
	file_size,
	file_app_status,  
	file_transfer_status,                 
	mft_id
FROM
	media_file_transfer a,
	media_file b,
	file_application_transfer c
WHERE
	a.media_file_id = b.media_file_id 
and fat_status = 'ACTIVE'
and fat_app_type_name = app_type_name
and a.fat_id = c.fat_id
and c.fat_id = %s
and b.file_transfer_status = '%s'
and mft_status != '%s'
", $mFat_id, FT_PROCESSING, FT_COMPLETED));
//////////////////// USED TWICE:  2 Passes  ///////////////////////				
				//output($query_statement); die;

				$rsSearchResult = mysql_query($query_statement); // or die(mysql_error());
				$totalRows_rsSearchResult = mysql_num_rows($rsSearchResult);  
				output ("FAT LOCATION ".$mFat_id." - Total media files = ".$totalRows_rsSearchResult);
				// Only process if we have something to do
				if ($totalRows_rsSearchResult > 0)
				{				
					// Set some defaults
					// Reset holder 
					$ftpTextWeb		= '';
					
					// For each media recording to be processed...
					while (($row_rsSearchResult = mysql_fetch_assoc($rsSearchResult)) && ($mProcessLimit > 0))
					{   
						// Get the fields
						$mMedia_file_id			= $row_rsSearchResult['media_file_id'];
						$mMedia_owner_u_id		= $row_rsSearchResult['media_owner_u_id'];
						$mMedia_file_directory	= $row_rsSearchResult['media_file_directory'];
						$mMedia_file_name		= $row_rsSearchResult['media_file_name'];		
						$mApp_type_name			= $row_rsSearchResult['app_type_name'];			
						$mMedia_file_type		= $row_rsSearchResult['media_file_type'];		
						$mFile_duration			= $row_rsSearchResult['file_duration'];			
						$mFile_size				= $row_rsSearchResult['file_size'];				
						$mFile_app_status		= $row_rsSearchResult['file_app_status'];		
						$mFile_transfer_status	= $row_rsSearchResult['file_transfer_status'];
						$mMft_id				= $row_rsSearchResult['mft_id'];
												   
						// Add a file for ftp location
						if ($mTransferFTPLocation)
						{
							// Determine the folder in which the file should be found
							$mMediaFolder = mediaFileFolderLocate(false, $mClientId, $mMedia_file_directory);
							// The original dictation recording from TeleFlow
							$fileWAV   = $mMediaFolder.'/'.$mMedia_file_name;
							
							output("- Preprocessing ". $mMedia_file_name);
						
							// Determine filenames for compressed versions
							$fileMP3   = getMediaFileNameMP3($fileWAV); 
							$fileGSM   = getMediaFileNameGSM($fileWAV); 
							
							// If the file was created, then, transfer
							if (file_exists($fileGSM))
							{
								// Rename the file so that we can place it temporarily
								// -  mediaWorkUnit
								$fileXFER = getFileNameForXfer($fileGSM);
								if (file_exists($fileXFER))
								{
									unlink($fileXFER);
								}
								
								if (copy($fileGSM, $fileXFER))
								{					
									/////////////////////////
									// SEND to FTPLOCATION via FTP command file
									/////////////////////////
									// Create the FTP command list
									if ($ftp_put_only) // SSLNR
									{
										// Send as the final name.  Assume no review for problems.
										$ftpTextFTPLOCATION .= 'put '.slashToBackslash($fileGSM).chr(13).chr(10);

										// Track the file_id in order to update that the transfer occurred
										$mFilesTransferArray[$mFilesTransferCount] = $mMft_id;
										$mFilesTransferCount++;
									
									}
									else
									{
										// Send using a transfer file name, and rename later
										$ftpTextFTPLOCATION .= 'put '.slashToBackslash($fileXFER).chr(13).chr(10);
									}
									// Add one to send
									$transferToFTPLocation++;							

								}
								
							} // file exists GSM
						}
					} // while (transfer types)
					
					
					//////////////////////////////////////
					//  FTP: Transfer the files securely
					//////////////////////////////////////
					// Transfer to the FTPLOCATION location
					if (($transferToFTPLocation > 0) && ($ftp_dest_server <> ''))
					{
						///////////////// TRANSFER ATTEMPT ///////////////

						$ftpTextFTPLOCATION .= 'quit'.chr(13).chr(10);
					
						// Write the FTP command file
						$handleIdx  = fopen($ftpFileFTPLOCATION, "w");  // Open for Write from beginning
						fwrite($handleIdx, $ftpTextFTPLOCATION);
						fclose($handleIdx);
						
						// Figure out the flag for SSL
						$ftpSSLFlags = '-e:off';
						if ($ftp_dest_SSL)
						{
							$ftpSSLFlags = '-e:on';
						}
						
						// File was converted.  Send
						$ftpsend = 'ftp\ftps.exe '.$ftp_dest_server.
								' -user:'.$ftp_dest_user_name.
								' -password:'.$ftp_dest_user_pass.
								' -z '.$ftpSSLFlags.' -s:'.$ftpFileFTPLOCATION;
						output($ftpsend);
						
						// Do the ftp connect and send
						$result = shell_exec($ftpsend);
						
						///////////////// TEST for TRANSFER SUCCESS ///////////////
						// if we are SSLNR, (SSL No Read), then 
						if ($ftp_put_only)
						{
							// Clean up the local processing copies of the files
							foreach ($mFilesTransferArray as $mMft_id)
							{
								$update_statement = (sprintf(
"UPDATE         
	media_file_transfer
SET             
	mft_status = '%s'
WHERE 
	mft_id = %s", FT_COMPLETED, $mMft_id));
								if (!$res = mysql_query($update_statement))
								{
									output("ERROR: SSLNR Transfer: Updating Media File Transfer ID: ". $mMft_id);
									output($update_statement);
								}
								else
								{
									output("SSLNR Transfer: Updating Media File Transfer ID: ". $mMft_id);
								}

							}				
						}
						// Connect to host (also, if SSL is enables, see ftp_ssl_connect)
						else if ($conn = ftp_connect($ftp_dest_server))
						{
							ftp_login($conn, $ftp_dest_user_name, $ftp_dest_user_pass);
							ftp_pasv($conn, true);				
							output("FTP-Dest: Confirmation login");

							// Get to the folder where the files are to be worked from
							$goFolderList = explode("/", $ftp_dest_folder );
							foreach ($goFolderList as $goFolder)
							{
								output("FTP-Dest: Cd: ".$goFolder);
								if ($goFolder <> '')
								{
									ftp_chdir($conn, $goFolder);
								}
							}
								
							// This doesnt work with IIS apparently
							$files = ftp_rawlist($conn, ".");  // Gets a detailed list
							foreach ($files as $remotedetail)
							{
								//echo "File: ".$remotefile."\n";
								// Remote details will return space padded information allowing it to draw nicely, but not easy to explode, so trim extra space

								$filedetails = explode(" ", stripRepeating(' ',$remotedetail));
								//echo $filedetails[FTPACCESS]."---\n";
								
								// see if we have a file (d = dir0)   eg:  drw-rw-rw-
								if (substr($filedetails[FTPACCESS],0,1) != 'd')
								{
									// File Size
									$mFileSize = $filedetails[FTPSIZE];
									$mFileName = $filedetails[FTPFILENAME];
									
									// Is this a transferred wav file name (.wa1?)
									if (strpos($mFileName, FTPXFEREXT))
									{
										output("FTP-Dest ".$mFileSize."\t".$mFileName);

										$fileFtpOrig = str_replace(FTPXFEREXT, FTPWAVEXT, $mFileName);   // Lower case
										
										// Return the file name to something we can look up
										// - Note: the file transfer status should be indexed to make lookups fast
										$query_lookup = (sprintf(			
"SELECT            
	media_file_id,
	media_file_directory,
	media_file_name,
	mft_id
FROM
	media_file_transfer a,
	media_file b,
	file_application_transfer c
WHERE
	a.media_file_id = b.media_file_id 
abd fat_status = 'ACTIVE'	
and fat_app_type_name = app_type_name
and a.fat_id = c.fat_id
and b.file_transfer_status = '%s'
and c.fat_dest_type = 'FTPLOCATION'
and a.media_file_name = '%s'
", FT_PROCESSING, $fileFtpOrig));
										//output($query_lookup); die;

										$rsSearchResult = mysql_query($query_lookup); // or die(mysql_error());
										$totalRows_rsSearchResult = mysql_num_rows($rsSearchResult);  
										// For each media recording to be processed...
										if ($row_rsSearchResult = mysql_fetch_assoc($rsSearchResult))
										{
											// Get the fields
											$mMedia_file_id			= $row_rsSearchResult['media_file_id'];
											$mMedia_file_directory	= $row_rsSearchResult['media_file_directory'];
											$mMedia_file_name		= $row_rsSearchResult['media_file_name'];											
											$mMft_id 				= $row_rsSearchResult['mft_id'];
											
											// Determine the folder in which the file should be found
											$mMediaFolder = mediaFileFolderLocate(false, $mClientId, $mMedia_file_directory);
											// The original dictation recording from TeleFlow
											$fileWAV   = $mMediaFolder.'/'.$mMedia_file_name;
											
											// Do we have the original file
											if (file_exists($fileWAV))
											{
													$fileWAV   	= str_replace(FTPXFEREXT, FTPWAVEXT, $fileWAV);   // Lower case
													$fileFtpWAV	= str_replace(FTPXFEREXT, FTPWAVEXT, $mFileName);   // Lower case

													// Compare the size.  If they are  identical, then rename, commit in the db, and remove locally
													if ($mFileSize == filesize($fileWAV))
													{
														// Make sure the original is gone
														if (@ftp_delete($conn, $fileFtpWAV))
														{
															output("FTP-Delete ".$fileFtpWAV);
														}
														// Rename on that end 
														// Note; the FTPLOCATIONist is ready to go
														ftp_rename($conn, $mFileName, $fileFtpWAV);
														
														// Update the table 
														$update_statement = (sprintf(
"UPDATE         
	media_file_transfer
SET             
	mft_status = '%s'
WHERE 
	mft_id = %s", FT_COMPLETED, $mMft_id));  
														if (!$res = mysql_query($update_statement))
														{
															output("ERROR: Failure Media File Transfer Id: ". $mMft_id);
															output($update_statement);
														}
													
													}
													// Otherwise, destroy, let the next attemp try again.
													else
													{
														// Update the table 
														$update_statement = (sprintf(
"UPDATE         
	media_file_transfer
SET             
	mft_status = '%s'
WHERE 
	mft_id = %s", FT_RETRY, $mMft_id));  
														if (!$res = mysql_query($update_statement))
														{
															output("ERROR: Failure Media File Transfer Id: ". $mMft_id);
															output($update_statement);
														}

														ftp_delete($conn, $mFileName);
														output("FTP-Dest ".filesize($fileWAV)."\t".$mFileName." FILESIZE ERROR");
													}					
													
													// Finally, track to delete the local copy
													$mFilesTempArray[$mFilesTempCount] = $ftp_location.$mFileName;
													$mFilesTempCount++;

											} // if exists
										} // if fetch
									} // if (strpos($mFileName, FTPXFEREXT))
								} // if not a dir
							} // for each

							ftp_close($conn);
							
						} // if Connected	
					} // if ($transferToFTPLocation > 0)
				} // if ($totalRows_rsSearchResult > 0)
				

				//////////////////////////////////////
				//  START: Pass 2: Perform the processing for media TeleFlowRelay HTTP post types
				//////////////////////////////////////
				// Note: This first pass allows us to build the scripts for en-mass transfers to FTP locations.
				// -- The same query will be run a second time to do the transfer for one by one, or en-mass
				// Re uses last SQL statement
				$rsSearchResult = mysql_query($query_statement); // or die(mysql_error());
				$totalRows_rsSearchResult = mysql_num_rows($rsSearchResult);  
				output ("FAT LOCATION ".$mFat_id." - Total media files = ".$totalRows_rsSearchResult);
				// Only process if we have something to do
				if ($totalRows_rsSearchResult > 0)
				{
					output ("- Total media transfers reviewed = ".$totalRows_rsSearchResult);
				
					// Transfer to the web location using HTTP Post
					// - Post of XML will be sent to $ftp_web_server
					if (($mTransferTeleFlowRelay) && ($ftp_web_server <> ''))
					{				
						// Set some defaults
						$mTransferToLocationsCount = 0;
						// Reset holder 
						$ftpTextWeb		= '';
						
						// For each media recording to be processed...
						while (($row_rsSearchResult = mysql_fetch_assoc($rsSearchResult)) && ($mProcessLimit > 0))
						{   
							// Get the fields - again
							$mMedia_file_id			= $row_rsSearchResult['media_file_id'];
							$mMedia_owner_u_id		= $row_rsSearchResult['media_owner_u_id'];
							$mMedia_file_directory	= $row_rsSearchResult['media_file_directory'];
							$mMedia_file_name		= $row_rsSearchResult['media_file_name'];		
							$mApp_type_name			= $row_rsSearchResult['app_type_name'];			
							$mMedia_file_type		= $row_rsSearchResult['media_file_type'];		
							$mFile_duration			= $row_rsSearchResult['file_duration'];			
							$mFile_size				= $row_rsSearchResult['file_size'];				
							$mFile_app_status		= $row_rsSearchResult['file_app_status'];		
							$mFile_transfer_status	= $row_rsSearchResult['file_transfer_status'];
							$mMft_id				= $row_rsSearchResult['mft_id'];

							output("-- Sending MFT_id: ".$mMft_id." - ".$mMedia_file_name);

							// Determine the folder in which the file should be found
							$mMediaFolder = mediaFileFolderLocate(false, $mClientId, $mMedia_file_directory);
							// The original dictation recording from TeleFlow
							$fileWAV   = $mMediaFolder.'/'.$mMedia_file_name;

							// Application transfer mechanism
							$mAppXferType = "FileUpload";
							
							// Do we need to send OAUTH information for a later post?
							$mOauthKey1 = '';
							$mOauthKey2 = '';
							if ($mApp_type_name == 'TWITYAX')
							{
								// Return the file name to something we can look up
								// - Note: the file transfer status should be indexed to make lookups fast
								$req_user_info = $database->getUserIdData($mMedia_owner_u_id);
								if ($req_user_info)
								{
									// Change the transfer type
									$mAppXferType = "FileUploadTwitYax";
								
									$mOauthKey1 = $req_user_info['u_oauth_access_key'];
									$mOauthKey2 = $req_user_info['u_oauth_secret_access_key'];
									output("-- Oauth required for ".$mApp_type_name.": ".$mOauthKey1);
								}
								else
								{
									output("-- WARNING: Oauth required for ".$mApp_type_name.", but not found");
								}
							}
							
							// Test all files gone
							$mSuccessSend = true;
							$fileToSendGSM = '';
							$fileToSendMP3 = '';
							
							// Send each file as needed.
							if ($mFat_gsm_type == 'YES')
							{
								$fileToSendGSM = getMediaFileNameGSM($fileWAV);
								if (file_exists($fileToSendGSM))
								{
									output("-- Posting GSM to: ".$ftp_web_server);
									// Call to mediaWorkUnit
									$mVerboseMessage = sendTeleFlowRelayFileViaHTTP(
														$mClientId, 
														$mAppXferType,
														$ftp_web_server, 
														$mApp_type_name, 
														$mMedia_file_id, 
														$mMedia_file_directory, 
														$fileToSendGSM, 
														'GSM',
														-1,
														$mOauthKey1,
														$mOauthKey2,
														$mFileSize
														);
									output("-- Remote Message: ".$mVerboseMessage);
									output("-- GSM FileSize Remote: ".$mFileSize.".");
									$mFileSizeHere = filesize($fileToSendGSM);
									// Compare the size.  If they are  identical, then rename, commit in the db, and remove locally
									output("-- GSM FileSize Here..: ".$mFileSizeHere.".");
									if (($mFileSize) <> ($mFileSizeHere))
									{	
										output("-- ERROR - transfer failed size check.");
										$mSuccessSend = false;
									}
								}
								else
								{
									$mSuccessSend = false;
								}
							}
				
							if ($mFat_mp3_type == 'YES')
							{
								$fileToSendMP3 = getMediaFileNameMP3($fileWAV);
								if (file_exists($fileToSendMP3))
								{
									output("-- Posting MP3 to: ".$ftp_web_server);
									$mVerboseMessage = sendTeleFlowRelayFileViaHTTP(
														$mClientId, 
														$mAppXferType,
														$ftp_web_server, 
														$mApp_type_name, 
														$mMedia_file_id, 
														$mMedia_file_directory, 
														$fileToSendMP3,
														'MP3',
														-1,
														$mOauthKey1,
														$mOauthKey2,
														$mFileSize
														);
									output("-- Remote Message: ".$mVerboseMessage);
									output("-- MP3 FileSize Remote: ".$mFileSize.".");
									$mFileSizeHere = filesize($fileToSendMP3);
									// Compare the size.  If they are  identical, then rename, commit in the db, and remove locally
									output("-- MP3 FileSize Here..: ".$mFileSizeHere.".");
									if (($mFileSize) <> ($mFileSizeHere))
									{	
										output("-- ERROR - transfer failed size check."); 
										$mSuccessSend = false;
									}
								}
								else
								{
									$mSuccessSend = false;
								}								
							}
							
							// If all bytes got to where they needed to, then update the file media transfer status
							if ($mSuccessSend)
							{
								// Update the table 
								$update_statement = (sprintf(
"UPDATE         
	media_file_transfer
SET             
	mft_status = '%s'
WHERE 
	mft_id = %s", FT_COMPLETED, $mMft_id));
								if ($res = mysql_query($update_statement))								
								{
									// Mark a success
									$mTransferToLocationsCount++;									
								}
								else
								{
									output("-- ERROR: Failure Updating MFT ID: ". $mMft_id);
									output($update_statement);
								}
							
							}
							// Otherwise, let the next attempt try again.
							else
							{
								// Update the table 
								$update_statement = (sprintf(
"UPDATE         
	media_file_transfer
SET             
	mft_status = '%s'
WHERE 
	mft_id = %s", FT_RETRY, $mMft_id));
								if (!$res = mysql_query($update_statement))
								{
									output("ERROR: Failure Updating Web Conf Num: ". $mfile_conf_num);
									output($update_statement);
								}
								output("POST-Local ".filesize($fileToSend)."\t".basename($fileToSend)."\n- FILESIZE ERROR");
							}					

						} // While media files
				
					}
					else
					{
						// The join between file_application_transfer and media_file found nothing.  As the system was indicated that a new file is ready something is 
						// not configured correctly.
						output('-- WARNING: Nothing to do - check that an active entry exists in file_application_transfer');
					
					} // if (($mTransferTeleFlowRelay) && ($ftp_web_server <> ''))

				} // if($totalRows_rsSearchResult > 0)
				//////////////////////////////////////
				// END: Pass 2
				//////////////////////////////////////
				
				
				
				//////////////////////////////////////
				// REVIEW: See if all the files transfered correctly, and mark if successful with COMPLETED
				//////////////////////////////////////
				// Now, process the files for the fat
				$query_statement = (sprintf(			
"SELECT            
	media_file_id,
	media_owner_u_id,
	file_duration,
	file_size
FROM
	media_file
WHERE
	file_transfer_status = '%s'
", FT_PROCESSING));
				//output($query_statement); die;

				$rsSearchResult = mysql_query($query_statement); // or die(mysql_error());
				$totalRows_rsSearchResult = mysql_num_rows($rsSearchResult);  
				output ("-- Review for Completed: ".$totalRows_rsSearchResult);
				// Only process if we have something to do
				if ($totalRows_rsSearchResult > 0)
				{				
					// Set some defaults
					// Reset holder 
					$ftpTextWeb		= '';
					
					// For each media recording to be processed...
					while (($row_rsSearchResult = mysql_fetch_assoc($rsSearchResult)))
					{   
						// Get the fields
						$mMedia_file_id			= $row_rsSearchResult['media_file_id'];
						$mMedia_owner_u_id		= $row_rsSearchResult['media_owner_u_id'];
						$mFile_duration			= $row_rsSearchResult['file_duration'];			
						$mFile_size				= $row_rsSearchResult['file_size'];				
						
						// Find completed entries in the next 
						$query_statement_review = (sprintf(
"SELECT            
	count(*) total
FROM
	media_file_transfer
WHERE
	media_file_id = %s
and mft_status != '%s'
", $mMedia_file_id, FT_COMPLETED));
						$rsReviewResult = mysql_query($query_statement_review); // or die(mysql_error());
						if (($row_rsReviewResult = mysql_fetch_assoc($rsReviewResult)))
						{   
							// Get the count
							$mTotal	= $row_rsReviewResult['total'];
							
							// Do a check to make sure all the mft entries are COMPLETED (new, completed, retry)
							// - If successful transfers = the number to transfer... we are done
							// - Update media_file from PROCESSING to COMPLETED		
							if ($mTotal == 0)
							{
								// Update the details in Database
								$result = $database->mediaEntryUpdate($mClientId, 
											$mMedia_file_id,
											'', // no need to update file_app_status
											FT_COMPLETED,
											$mFile_duration,
											$mFile_size);
											
								output ("-- Recording completed for Media File: ".$mMedia_file_id);
							}
							else
							{
								output ("-- Warning: Could not complete Media File: ".$mMedia_file_id);
							}							
						}

					} // for each file to process
				}
				
			} // while each FAT location
			//////////////////////////////////////
			//  END: Transfer Loop
			//////////////////////////////////////
			
		} //  if $database select client
		
		output (" "); // A blank space
		
	} // while clients with media transfer applications

} // If clients found

//output ("DEBUG - Shutdown");
//die;
//catch exception
}
catch(Exception $e)
{
	output (" "); // A blank space
	output ("ERROR Encountered");
	output ("* Message: ".$e->getMessage());
	output ("* Waiting to continue loop...");
	sleep($cWaitLoop);
	output (" "); // A blank space
}


} while (TRUE);
  
output ("Done! \n");
?>