<?php
/*  cr_recording_transfer.php

Used to convert and transfer call recordings to their destination

* Dictations
* Voice Messages (should be added here)
* Call Recordings (should be added here)

*/

// Determine relative location for paths
require_once("cr_reference.php");
require_once($incPath.'LocalSettings.php'); 
require($incPath."include/session.php");
require_once($websiteLibrary.'calcWorkUnit.php'); 
// Dictation specific
require_once($websiteLibrary.'dictationWorkUnit.php'); 
// Post using stream
require_once($websiteLibrary.'httpWorkUnit.php');     

//////////////////
// Notes
//////////////////
/*
REQUIRED:  The File MUST be named
-- 4_ZZZZ_6046382650_2000_Spears_032408_060329_0_065423_orig.wav


-- xxx_orig.wav is created by TeleFlow Dictation environment
-- xxx.wav is the GSM (compressed file) that is created by this software
-- xxx.wav is renamed to xxx.wa1
-- xxx.wa1 is ftp'd to the remote location
-- xxx.wa1 size local and remote is compared
if the size is the same, then
-- xxx.wa1 is renamed to xxx.wav remote
else
-- xxx.wa1 is deleted remote
end
-- xxx.wa1 is deleted local

*/

//////////////////
// Settings
//////////////////

// Note: These values should eventually come from a database or algorithm that determines what to do

// Process type
define("PROCESSTYPE",	'DICTATION_XFER');
// Transfer types
define("TRANSCRIPTION",	'TRANSCRIPTION');	// FTP to other company
define("WEB",			'WEB');				// FTP to our web environment
define("TELEFLOWRELAY",	'TELEFLOWRELAY');	// HTTP Post to a TeleFlowRelay host


// FTP listing fields
define("FTPACCESS", 	0);
define("FTPLEVEL",  	1);
define("FTPOWNER",  	2);
define("FTPGROUP",  	3);
define("FTPSIZE",   	4);
define("FTPTIME1",  	5);
define("FTPTIME2",  	6);
define("FTPTIME3",  	7);
define("FTPFILENAME", 	8);

// For file transfer name
define("FTPXFEREXT",	'.wa1');
define("FTPWAVEXT",		'.wav');


// Constants
$cPending				= 'Pending';
$cPendingRetry			= 'PendingRetry';
$cUploaded  			= 'Uploaded';
$cAbandoned				= 'Abandoned';


// Simple functions
function output($pLine)
{
	global $incPath;
	
	fwrite(STDOUT, $pLine."\n"); // echo
	//LogTextToFile('cr_transfer_log.log', $pLine."\n");	
	LogTextToDateNamedFileSystem($incPath, 'cr-recording-transfer', $pLine."\n");	
}


output ("-------------------------------");
output ("Starting PHP transfer process");
output ("-------------------------------");

// Get the computer name - eg:  JUPITER
$mCompName = getHostComputerName(); //php_uname('n');  
output ("Search for files created by ".$mCompName);

// Create the path for the temporary FTP command files
// Location for database
$commandFilesPath = $incPath.'../'.SYSTEM_TEMPFILES.'/';
if (!file_exists($commandFilesPath))
{
	// Create path if not exist
	make_path($commandFilesPath);
}
output("FTP Command Files created here: ".$commandFilesPath);

$mLoop = 0;
// Loop the entire file
do // forever
{
$mLoop++;
output ("- Process Loop count: ".$mLoop);


//////////////////////////////////////
// Find all clients that have files that need to have transfers done
//////////////////////////////////////
$q =
"SELECT 
	a.client_id client_id,
	client_name,
	app_type_name,
	subscription_status
FROM 
	".DATABASE_MASTER.".client a,
	".DATABASE_MASTER.".client_application_subscription b
WHERE
	a.client_id = b.client_id
and app_type_name like '".PROCESSTYPE."'
";		
//echo($q);

$clientResult = mysql_query($q, $database->connectionSystem);
if (!$clientResult || (mysql_numrows($clientResult) < 1))
{
	output("No clients currently set for ".PROCESSTYPE." systems!");
}
else
{
	output("Found ".mysql_numrows($clientResult)." clients who have ".PROCESSTYPE);
	output(" "); // A blank space

	//////////////////////////////////////
	//  DICTATIONS - For Each Client
	//////////////////////////////////////
	while (($row_rsClientResult = mysql_fetch_assoc($clientResult)))
	{
		// Clear information
		$ftp_dest_server    	= '';
		$ftp_dest_user_name 	= '';
		$ftp_dest_user_pass 	= '';
		$ftp_dest_folder    	= '';
		$ftp_dest_SSL			= false;
		$ftp_web_server     	= '';
		$ftp_web_user_name  	= '';
		$ftp_web_user_pass  	= '';
		$ftp_web_folder     	= '';
		$ftp_web_SSL			= false;
		// Determine processing in from the file_app_trans table
		$proc_to_web      		= false;
		$proc_include_mp3 		= false;
		// @@@ Add processing determination for physicians later
		$proc_physicians  		= false;
		
		// Set the Files Array to keep track what to delete at the end
		$mFilesTempArray 		= array();
		$mFilesTempCount 		= 0;
		// Array for files used when send via SSLNR
		$mFilesTransferArray	= array();	
		$mFilesTransferCount 	= 0;
		
		// Remember the client
		$mClientId = $row_rsClientResult['client_id'];
		
		// Leave a break for readability
		output(' '); 
		output('Client: '.$mClientId.' - '.$row_rsClientResult['client_name']);
		if ($database->selectClientDatabaseByMasterClientId($mClientId))
		{
			output('- Selected database');
		}

		
		//////////////////////////////////////
		//  Communications details - create the ftp batch command files 
		//////////////////////////////////////
		// Do not include the web
		$q = 
"SELECT                               
	fat_wav_type, 
	fat_gsm_type, 
	fat_mp3_type, 
	fat_dest_type, 
	fat_dest_host_name, 
	fat_dest_ssl_type, 
	fat_dest_user_name, 
	fat_dest_user_pass, 
	fat_dest_folder
FROM                                 
	file_application_transfer                 
WHERE
	fat_status = 'ACTIVE'";
	
		//output($q); die;
		$mTransferToLocationsCount = 0;

		// See what locations we find
		$mTransferLocationWeb 			= false;	// FTP to web
		$mTransferLocationTranscription = false;	// FTP to transcription
		$mTransferLocationTeleFlowRelay	= false;	// HTTP POST to TeleFlow system
		//$mTransferLocationArchiver 		= false;	
		
		// Type of transfer
		$ftp_put_only	= false;
		// Reset holder 
		$ftpTextWeb		= '';
		
		
		// Loop 
		$rsFatResult = mysql_query($q); // or die(mysql_error());

		
		//////////////////////////////////////
		//  TRANSFER LOCATION - For Each Transfer Record
		//////////////////////////////////////
		// Loop to generate a list of files to transfer for all ftp locations at the same time.  
		// - Max 1 (one) of each type: WEB / TRANSCRIPTION
		while ($row_rsFatResult = mysql_fetch_assoc($rsFatResult))
		{
			// Each row will be one of WEB, TRANSCRIPTION
			if ($row_rsFatResult['fat_dest_type'] == TRANSCRIPTION)
			{
				// Transcription transfer determined
				$mTransferLocationTranscription = true;
		
				// Determine what to do
				output('- [X] - Processing '.$row_rsFatResult['fat_dest_type']);
				// Transcription transfer			
				$ftp_dest_server        = $row_rsFatResult['fat_dest_host_name'];
				$ftp_dest_user_name     = $row_rsFatResult['fat_dest_user_name'];
				$ftp_dest_user_pass     = $row_rsFatResult['fat_dest_user_pass'];
				$ftp_dest_folder        = $row_rsFatResult['fat_dest_folder'];
				$ftp_dest_SSL			= false;
				if ($row_rsFatResult['fat_dest_ssl_type'] == 'YES')
				{
					$ftp_dest_SSL = true;				
				}
				else if ($row_rsFatResult['fat_dest_ssl_type'] == 'SSLNR')  // SSL No Read
				{
					$ftp_dest_SSL = true;				
					$ftp_put_only = true;				
				}

				
				// We will attempt to ftp to this location
				$mTransferToLocationsCount++;
				
				// Create the command for the Transcriptist FTP site 
				$ftpFileTranscription  = $commandFilesPath.PROCESSTYPE.'_transfer_transcription_'.$row_rsClientResult['client_id'].'.ftp';
				$ftpTextTranscription  = 'passive'.chr(13).chr(10);
				$ftpTextTranscription .= 'bin'.chr(13).chr(10);
				// Get to the folder where the files are to be worked from
				$goFolderList = explode("/", $ftp_dest_folder );
				foreach ($goFolderList as $goFolder)
				{
					if ($goFolder <> '')
					{
						$ftpTextTranscription .= 'cd '.$goFolder.chr(13).chr(10);
					}
				}	
				
			}
			else if ($row_rsFatResult['fat_dest_type'] == WEB)
			{
				// Web transfer determined
				$mTransferLocationWeb = true;
				
				// Determine what to do
				output('- [X] - Processing '.$row_rsFatResult['fat_dest_type']);
				$proc_to_web = true;
				// Do we also want to convert to mp3?
				if ($row_rsFatResult['fat_mp3_type'] == 'YES')
				{
					$proc_include_mp3 = true;				
				}				
				
				// We will attempt to FTP to this location
				$mTransferToLocationsCount++;
				
				// Web server for copy to
				$ftp_web_server         = $row_rsFatResult['fat_dest_host_name'];
				$ftp_web_user_name      = $row_rsFatResult['fat_dest_user_name'];
				$ftp_web_user_pass      = $row_rsFatResult['fat_dest_user_pass'];
				$ftp_web_folder       	= $row_rsFatResult['fat_dest_folder'];
				$ftp_web_SSL			= false;
				if ($row_rsFatResult['fat_dest_ssl_type'] == 'YES')
				{
					$ftp_web_SSL = true;				
				}

				// Create the command for the Web FTP site 
				$ftpFileWeb  = $commandFilesPath.PROCESSTYPE.'_transfer_web_'.$mClientId.'.ftp';
				$ftpTextWeb  = 'passive'.chr(13).chr(10);
				$ftpTextWeb .= 'bin'.chr(13).chr(10);
				// Get to the folder where the files are to be worked from
				$goFolderList = explode("/", $ftp_web_folder );
				foreach ($goFolderList as $goFolder)
				{
					if ($goFolder <> '')
					{
						$ftpTextWeb .= 'cd '.$goFolder.chr(13).chr(10);
					}
				}
				
				// For the web side, also include the media section and client subfolder
				$ftpTextWeb .= 'cd '.CLIENT_MEDIA_AREA.chr(13).chr(10);
				// Client subfolder will include a make dir just in case
				$docClientRoot = 'client_'.zeroFill($mClientId,5);				
				$ftpTextWeb .= 'mkdir '.$docClientRoot.chr(13).chr(10);
				$ftpTextWeb .= 'cd '.$docClientRoot.chr(13).chr(10);	
				// For the web side (always our control), create an upload folder
				$ftpTextWeb .= 'mkdir upload'.chr(13).chr(10);
				$ftpTextWeb .= 'cd upload'.chr(13).chr(10);	
				
			}
			// HTTP post
			else if ($row_rsFatResult['fat_dest_type'] == TELEFLOWRELAY)
			{
				// Web transfer determined
				$mTransferLocationTeleFlowRelay = true;
				
				// Determine what to do
				output('- [X] - Processing '.$row_rsFatResult['fat_dest_type']);
				$proc_to_web = true;
				// Do we also want to convert to mp3?
				if ($row_rsFatResult['fat_mp3_type'] == 'YES')
				{
					$proc_include_mp3 = true;				
				}				
				
				// We will attempt to POST to this location
				$mTransferToLocationsCount++;
				
				// Web server php for POST transfer to
				$ftp_web_server         = $row_rsFatResult['fat_dest_host_name'];	
			}
		} // while (transfer types)
		
		// Make sure we have something to do
		if ($mTransferToLocationsCount == 0)
		{
			output('ERROR: There are no File Application Transfer FTP sites defined for this client.');
			continue;
		}
		
		
		
		//////////////////////////////////////
		//  Gather and Convert Files
		//////////////////////////////////////
		$transferToTranscription	= 0;
		$transferToWeb 				= 0;
		$transferToTeleFlowRelay	= 0;
		
		
		// Gather all entries that need to be worked on
		// - Are we going to process two locations?
		if (($proc_to_web == true) && ($ftp_web_server <> ''))
		{
			$query_statement = (sprintf(
"SELECT                               
   a.file_id,                        
   a.file_name,                      
   a.file_status_transcription,      
   a.file_status_web,                
   b.dir_loc
FROM                                 
   dictate_file a,                   
   filepath b                        
WHERE                                
     a.filepath_id = b.filepath_id	 
and (   (a.file_status_transcription = '%s') 
     or (a.file_status_transcription = '%s')
     or (a.file_status_web           = '%s'))
and (ivr_computer_name               = '%s')", $cPending, $cPendingRetry, $cPending, $mCompName));
		}
		else
		{
			// Do not include the web
			$query_statement = (sprintf(
"SELECT                               
   a.file_id,                        
   a.file_name,                      
   a.file_status_transcription,      
   a.file_status_web,                
   b.dir_loc
FROM                                 
   dictate_file a,                   
   filepath b                        
WHERE                                
     a.filepath_id = b.filepath_id   
and (   (a.file_status_transcription = '%s') 
     or (a.file_status_transcription = '%s'))
and (ivr_computer_name               = '%s')", $cPending, $cPendingRetry, $mCompName));
		}
		
		//output($query_statement);	die; // Read them 
		
		$rsSearchResult = mysql_query($query_statement); // or die(mysql_error());
		$totalRows_rsSearchResult = mysql_num_rows($rsSearchResult);  
		output ("- Total dictation recordings for processing on ".$mCompName." = ".$totalRows_rsSearchResult);

		// For each dictation to be processed...
		$mProcessLimit = 20;  // Only do 20 at a time so that we are not overwhelming the server
		while (($row_rsSearchResult = mysql_fetch_assoc($rsSearchResult)) && ($mProcessLimit > 0))
		{   
			// Get the fields
			$mfile_id                  = $row_rsSearchResult['file_id'];
			$mfile_name                = $row_rsSearchResult['file_name'];
			$mfile_status_transcription= $row_rsSearchResult['file_status_transcription'];
			$mfile_status_web          = $row_rsSearchResult['file_status_web'];
			$mdir_loc                  = $row_rsSearchResult['dir_loc'];
			$mweb_local_dirloc         = $row_rsSearchResult['web_local_dirloc'];
		
			// Where should the file be on this computer?
			$ftp_location = $mdir_loc.'/';
		
			// See if we need to transfer using FTP SSL
			if (((($mfile_status_transcription == $cPending) ||
				  ($mfile_status_transcription == $cPendingRetry))
				  && ($mTransferLocationTranscription))
				||
				((($mfile_status_web 			 == $cPending) ||
				  ($mfile_status_web			 == $cPendingRetry))
				  && ($mTransferLocationWeb)) 
				||
				// HTTP Post
				((($mfile_status_web 			 == $cPending) ||
				  ($mfile_status_web			 == $cPendingRetry))
				  && ($mTransferLocationTeleFlowRelay)) 
				)
			{
				// The original dictation recording from TeleFlow
				$fileWAV   = $ftp_location.$row_rsSearchResult['file_name'];
				//echo "LOOKING FOR: ".$fileWAV; die;
				
				// Do we have the original file
				if (file_exists($fileWAV))
				{
					// We are processing this, so reduce the count
					$mProcessLimit = $mProcessLimit -1;
				
					// Compressed version (wav)
					$fileGSM   = $ftp_location.untack_orig($row_rsSearchResult['file_name']);
					// Mp3 version 
					$fileMP3   = $ftp_location.untack_orig($row_rsSearchResult['file_name']);
					$fileMP3   = str_replace(FTPWAVEXT, '.mp3', $fileMP3);   // Lower case


					// CONVERT: Convert the WAV FAT file to WAV GSM  (also see SOX:  http://sox.sourceforge.net/)
					$cmd = ('libsndfile\sndfile-convert.exe -gsm610 '.$fileWAV.' '.$fileGSM);
					output('Converting: '.$cmd);
					$result = shell_exec($cmd);
					if ($proc_include_mp3)
					{
						$cmd = ('lame\lame -h -V 4 '.$fileWAV.' '.$fileMP3);
						output('Converting: '.$cmd);
						$result = shell_exec($cmd);
					}

					
					// If the file wasnt created, then, copy the original in order to send the fat wave at least
					if (!file_exists($fileGSM)) 
					{
						// Copy file for transfer 
						copy($fileWAV, $fileGSM);
					}
					else
					{
						// Check to see if the file is big enough
						clearstatcache();
						if (filesize($fileGSM) < 1000)
						{
							// Copy the original over top the (too small to bother transferring) file.
							copy($fileWAV, $fileGSM);			
						}
					}


					// If the file was created, then, transfer
					if (file_exists($fileGSM))
					{
						// Rename the file so that we can place it temporarily
						$fileXFER = filename_xfer($fileGSM);
						if (file_exists($fileXFER))
						{
							unlink($fileXFER);
						}
						
						if (copy($fileGSM, $fileXFER))
						{					
							/////////////////////////
							// SEND to Transcription via FTP command file
							/////////////////////////
							// TRANSFER: To the transcription site
							if (($mfile_status_transcription == $cPending) ||
								($mfile_status_transcription == $cPendingRetry))
							{
								// Create the FTP command list
								if ($ftp_put_only) // SSLNR
								{
									// Send as the final name.  Assume no review for problems.
									$ftpTextTranscription .= 'put '.slashToBackslash($fileGSM).chr(13).chr(10);

									// Track the file_id in order to update that the transfer occurred
									$mFilesTransferArray[$mFilesTransferCount] = $mfile_id;
									$mFilesTransferCount++;
								
								}
								else
								{
									// Send using a transfer file name, and rename later
									$ftpTextTranscription .= 'put '.slashToBackslash($fileXFER).chr(13).chr(10);
								}
								// Add one to send
								$transferToTranscription++;							
							}

							
							// Upload original file to web server
							if (($proc_to_web == true) && ($mTransferLocationWeb))
							{
								/////////////////////////
								// SEND to Web Site as Backup
								/////////////////////////
								if (($mfile_status_web == $cPending) ||
									($mfile_status_web == $cPendingRetry))
								{
									// Create the FTP command list
									$ftpTextWeb .= 'put '.slashToBackslash($fileXFER).chr(13).chr(10);
									// Add one to send
									$transferToWeb++;
									
									// Send the MP3 optional
									if ($proc_include_mp3)
									{
										// Create the FTP command list
										$ftpTextWeb .= 'put '.slashToBackslash($fileMP3).chr(13).chr(10);					
									}
								}								
							} // file exists WAV
							
							// Tests for a HTTP Post to web server
							if (($proc_to_web == true) && ($mTransferLocationTeleFlowRelay))
							{
								/////////////////////////
								// SEND to Web Site via TFR as Backup
								/////////////////////////
								if (($mfile_status_web == $cPending) ||
									($mfile_status_web == $cPendingRetry))
								{
									$ftpTextWeb  .= $fileGSM.chr(13).chr(10);
									$transferToTeleFlowRelay++;
									
									// Send the MP3 optional
									if ($proc_include_mp3)
									{
										// Create the FTP command list
										$ftpTextWeb .= $fileMP3.chr(13).chr(10);					
									}
								}
							}
						}
						
					} // file exists GSM

				}
				else
				{
					/////////////////////
					// ERROR Condition
					/////////////////////
					// We have a file that was abandoned or not existing for some other reason
					
					// @@@ Put an error condition in the master table somehow though
					// Update the table 
					$update_statement = (sprintf(
"UPDATE         
	dictate_file  
SET             
	file_status_web           = '$cAbandoned',
	file_status_transcription = '$cAbandoned'
WHERE 
	file_id = %s", $mfile_id));

					if (!$res = mysql_query($update_statement))
					{
						output("ERROR: (File Missing) Updating to 'Abandoned' fo FileID: ". $mfile_id);
						output($update_statement);
					}

					output("ERROR: (File Missing) Updating to 'Abandoned' fo FileID: ". $mfile_id);
					output("  - : ". $fileWAV);
				} // if wav file exists
				
			} // if Pending or PendingRetey
			
		} // while ($row_rsSearchResult = mysql_fetch_assoc($rsSearchResult));

		
		//////////////////////////////////////
		//  Transfer the files securely
		//////////////////////////////////////
		// Transfer to the transcription location
		if (($transferToTranscription > 0) && ($ftp_dest_server <> ''))
		{
			///////////////// TRANSFER ATTEMPT ///////////////

			$ftpTextTranscription .= 'quit'.chr(13).chr(10);
		
			// Write the FTP command file
			$handleIdx  = fopen($ftpFileTranscription, "w");  // Open for Write from beginning
			fwrite($handleIdx, $ftpTextTranscription);
			fclose($handleIdx);
			
			// Figure out the flag for SSL
			$ftpSSLFlags = '-e:off';
			if ($ftp_dest_SSL)
			{
				$ftpSSLFlags = '-e:on';
			}
			
			// File was converted.  Send
			$ftpsend = 'ftp\ftps.exe '.$ftp_dest_server.
					' -user:'.$ftp_dest_user_name.
					' -password:'.$ftp_dest_user_pass.
					' -z '.$ftpSSLFlags.' -s:'.$ftpFileTranscription;
			output($ftpsend);
			
			// Do the ftp connect and send
			$result = shell_exec($ftpsend);
			
			///////////////// TEST for TRANSFER SUCCESS ///////////////
			// if we are SSLNR, (SSL No Read), then 
			if ($ftp_put_only)
			{
				// Clean up the local processing copies of the files
				foreach ($mFilesTransferArray as $mfile_id)
				{
					$update_statement = (sprintf(
"UPDATE         
  dictate_file  
SET             
  file_status_transcription = '$cUploaded'
WHERE 
  file_id = %s", $mfile_id));

					if (!$res = mysql_query($update_statement))
					{
						output("ERROR: SSLNR Transfer: Updating Uploaded File ID: ". $mfile_id);
						output($update_statement);
					}
					else
					{
						output("SSLNR Transfer: Updating Uploaded status File ID: ". $mfile_id);
					}

				}				
			}
			// Connect to host (also, if SSL is enables, see ftp_ssl_connect)
			else if ($conn = ftp_connect($ftp_dest_server))
			{
				ftp_login($conn, $ftp_dest_user_name, $ftp_dest_user_pass);
				ftp_pasv($conn, true);				
				output("FTP-Dest: Confirmation login");

				// Get to the folder where the files are to be worked from
				$goFolderList = explode("/", $ftp_dest_folder );
				foreach ($goFolderList as $goFolder)
				{
					output("FTP-Dest: Cd: ".$goFolder);
					if ($goFolder <> '')
					{
						ftp_chdir($conn, $goFolder);
					}
				}
					
				// This doesnt work with IIS apparently
				$files = ftp_rawlist($conn, ".");  // Gets a detailed list
				foreach ($files as $remotedetail)
				{
					//echo "File: ".$remotefile."\n";
					// Remote details will return space padded information allowing it to draw nicely, but not easy to explode, so trim extra space

					$filedetails = explode(" ", stripRepeating(' ',$remotedetail));
					//echo $filedetails[FTPACCESS]."---\n";
					
					// see if we have a file (d = dir0)   eg:  drw-rw-rw-
					if (substr($filedetails[FTPACCESS],0,1) != 'd')
					{
						// File Size
						$mFileSize = $filedetails[FTPSIZE];
						$mFileName = $filedetails[FTPFILENAME];
						
						// Is this a transferred wav file
						if (strpos($mFileName, FTPXFEREXT))
						{
							output("FTP-Dest ".$mFileSize."\t".$mFileName);
						
							$fileWAV   	= $ftp_location.$mFileName;
							$fileWAV   	= str_replace(FTPXFEREXT, FTPWAVEXT, $fileWAV);   // Lower case
							$fileFtpWAV	= str_replace(FTPXFEREXT, FTPWAVEXT, $mFileName);   // Lower case

							// Get the local file conf number to look up in the database
							// eg:  4_ZZZZ_6046382650_2000_Spears_032408_060329_0_065423_orig.wav
							// 065423 = confirm number (8)
							$filenameparts = explode("_", basename(changeFileExtension($mFileName,'')));
							foreach ($filenameparts as $fileBit)
							{
								// Get the last part that is numeric
								if (is_numeric($fileBit))
								{
									$mfile_conf_num = intval($fileBit);
								}
							}								
								
							// Do we have a local copy of what we found at the remote FTP site?
							if (file_exists($fileWAV))
							{
								// Compare the size.  If they are  identical, then rename, commit in the db, and remove locally
								if ($mFileSize == filesize($fileWAV))
								{
									// Make sure the original is gone
									if (@ftp_delete($conn, $fileFtpWAV))
									{
										output("FTP-Delete ".$fileFtpWAV);
									}
									// Rename on that end 
									// Note; the transcriptionist is ready to go
									ftp_rename($conn, $mFileName, $fileFtpWAV);
									
									// Update the table 
									$update_statement = (sprintf(
"UPDATE
  dictate_file                     
SET                                
  file_status_transcription = '%s'
WHERE                              			
  file_conf_num = %s", $cUploaded , $mfile_conf_num));  
									if (!$res = mysql_query($update_statement))
									{
										output("ERROR: Failure Updating Transcription Conf Num: ". $mfile_conf_num);
										output($update_statement);
									}
								
								}
								// Otherwise, destroy, let the next attemp try again.
								else
								{
									// Update the table 
									$update_statement = (sprintf(
"UPDATE
  dictate_file                     
SET                                
  file_status_transcription = '%s'
WHERE                              			
  file_conf_num = %s", $cPendingRetry , $mfile_conf_num));  
									if (!$res = mysql_query($update_statement))
									{
										output("ERROR: Failure Updating Transcription Conf Num: ". $mfile_conf_num);
										output($update_statement);
									}

									ftp_delete($conn, $mFileName);
									output("FTP-Dest ".filesize($fileWAV)."\t".$mFileName." FILESIZE ERROR");
								}					
								
								// Finally, track to delete the local copy
								$mFilesTempArray[$mFilesTempCount] = $ftp_location.$mFileName;
								$mFilesTempCount++;
							} // If local file copy exists
							else
							{
								// Other systems may be FTPing files, so when not found on this system, its being handled by another 
								output("Ignoring: Not on this system: ".$fileWAV);
							}
						}
						// Just update as success if we have fudge factored it
						else if ($ftp_put_only)
						{
							// Break apart the filename and update as success
							
							// Get the local file conf number to look up in the database
							// eg:  4_ZZZZ_6046382650_2000_Spears_032408_060329_0_065423_orig.wav
							// 065423 = confirm number (8)
							$filenameparts = explode("_", basename(changeFileExtension($mFileName,'')));
							foreach ($filenameparts as $fileBit)
							{
								// Get the last part that is numeric
								if (is_numeric($fileBit))
								{
									$mfile_conf_num = intval($fileBit);
								}
							}
							
							output("FTP-Dest Fudge Update: \t".$mfile_conf_num."\t".$mFileName);
							
							// Update the table 
							$update_statement = (sprintf(
"UPDATE
  dictate_file                     
SET                                
  file_status_transcription = '%s'
WHERE                              			
  file_conf_num = %s", $cUploaded , $mfile_conf_num));  
							if (!$res = mysql_query($update_statement))
							{
								output("ERROR: Failure Updating Transcription Conf Num: ". $mfile_conf_num);
								output($update_statement);
							}
						
						}
						
					}
				}

				ftp_close($conn);
				
			} // if Connected	
		} // if ($transferToTranscription > 0)


		// Transfer to the web location using FTP
		if (($transferToWeb > 0) && ($ftp_web_server <> ''))
		{
			$ftpTextWeb .= 'quit'.chr(13).chr(10);

			// Write it 
			$handleIdx  = fopen($ftpFileWeb, "w");  // Open for Write from beginning
			fwrite($handleIdx, $ftpTextWeb);
			fclose($handleIdx);
				
			// Figure out the flag for SSL
			$ftpSSLFlags = '-e:off';
			if ($ftp_web_SSL)
			{
				$ftpSSLFlags = '-e:on';
			}
				
			// File was converted.  Send
			$ftpsend = 'ftp\ftps.exe '.$ftp_web_server.
					' -user:'.$ftp_web_user_name.
					' -password:'.$ftp_web_user_pass.
					' -z '.$ftpSSLFlags.' -s:'.$ftpFileWeb;
			output($ftpsend);
			
			// Do the ftp connect and send
			$result = shell_exec($ftpsend);
		
			
			///////////////// TEST for TRANSFER SUCCESS ///////////////
			
			// Connect to host (also, if SSL is enables, see ftp_ssl_connect)
			if ($conn = ftp_connect($ftp_web_server))
			{
				ftp_login($conn, $ftp_web_user_name, $ftp_web_user_pass);
				// ftp_pasv($conn, true);	  If you cant get a direcotry listing, try turning this on
				output("FTP-Web: Confirmation login");

				// Get to the folder where the files are to be worked from
				$goFolderList = explode("/", $ftp_web_folder );
				foreach ($goFolderList as $goFolder)
				{
					output("FTP-Web: Cd: ".$goFolder);
					ftp_chdir($conn, $goFolder);					
				}
				// Now work through the standard teleflow relay location
				output("FTP-Web: Cd: ".CLIENT_MEDIA_AREA);
				ftp_chdir($conn, CLIENT_MEDIA_AREA);
				$docClientRoot = 'client_'.zeroFill($mClientId,5);				
				output("FTP-Web: Cd: ".$docClientRoot);
				ftp_chdir($conn, $docClientRoot);
				output("FTP-Web: Cd: upload");
				ftp_chdir($conn, 'upload');
					
					
				// This doesnt work with IIS apparently
				$files = ftp_rawlist($conn, ".");  // Gets a detailed list
				foreach ($files as $remotedetail)
				{
					//echo "File: ".$remotefile."\n";
					// Remote details will return space padded information allowing it to draw nicely, but not easy to explode, so trim extra space

					$filedetails = explode(" ", stripRepeating(' ',$remotedetail));
					
					// see if we have a file (d = dir0)   eg:  drw-rw-rw-
					if (substr($filedetails[FTPACCESS],0,1) != 'd')
					{
						// File Size on the FTP site
						$mFileSize = $filedetails[FTPSIZE];
						$mFileName = $filedetails[FTPFILENAME];
						
						// Is this a transferred wav file
						if (strpos($mFileName, FTPXFEREXT))
						{
							//echo $filedetails."---\n";
							output("FTP-Web ".$mFileSize."\t".$mFileName);
						
							$fileWAV   	= slashToBackslash($ftp_location.$mFileName);
							$fileWAV   	= str_replace(FTPXFEREXT, FTPWAVEXT, $fileWAV);   // Lower case
							$fileFtpWAV	= str_replace(FTPXFEREXT, FTPWAVEXT, $mFileName);   // Lower case

							// Get the local file conf number to look up in the database
							// eg:  4_ZZZZ_6046382650_2000_Spears_032408_060329_0_065423_orig.wav
							// 065423 = confirm number (8)
							$filenameparts = explode("_", basename(changeFileExtension($mFileName,'')));
							foreach ($filenameparts as $fileBit)
							{
								// Get the last part that is numeric
								if (is_numeric($fileBit))
								{
									$mfile_conf_num = intval($fileBit);
								}
							}
							
							// Do we have a local copy of what we found at the remote FTP site?
							if (file_exists($fileWAV))
							{
								// Compare the size.  If they are  identical, then rename, commit in the db, and remove locally
								if ($mFileSize == filesize($fileWAV))
								{
									// Make sure the original is gone
									if (@ftp_delete($conn, '../'.$fileFtpWAV))
									{
										output("FTP-Delete ../".$fileFtpWAV);
									}
									// Rename on that end 
									// Note; the transcriptionist is ready to go
									// - Rename the file to the previous folder
									ftp_rename($conn, $mFileName, '../'.$fileFtpWAV);
									// @@@ Note: We might want to see if there is a file there first, and remove before we rename down.
									
									// Update the table 
									$update_statement = (sprintf(
"UPDATE
	dictate_file                     
SET                                
	file_status_web = '%s'
WHERE                              			
	file_conf_num = %s", $cUploaded , $mfile_conf_num));  
									if (!$res = mysql_query($update_statement))
									{
										output("ERROR: Failure Updating Web Conf Num: ". $mfile_conf_num);
										output($update_statement);
									}
								
								}
								// Otherwise, destroy, let the next attempt try again.
								else
								{
									// Update the table 
									$update_statement = (sprintf(
"UPDATE
  dictate_file                     
SET                                
  file_status_web = '%s'
WHERE                              			
  file_conf_num = %s", $cPendingRetry , $mfile_conf_num));  
									if (!$res = mysql_query($update_statement))
									{
										output("ERROR: Failure Updating Web Conf Num: ". $mfile_conf_num);
										output($update_statement);
									}
									ftp_delete($conn, $mFileName);
									output("FTP-Local ".filesize($fileWAV)."\t".$fileWAV."\n- FILESIZE ERROR");
								}					
								
								// Finally, track to delete the local copy
								$mFilesTempArray[$mFilesTempCount] = $ftp_location.$mFileName;
								$mFilesTempCount++;
								
							} // If local file copy exists
							else
							{
								// Other systems may be FTPing files, so when not found on this system, its being handled by another 
								output("Ignoring: Not on this system: ".$fileWAV);
							}
						}
					}
				} // foreach file at FTP

				ftp_close($conn);
			}
		}

		
		// Transfer to the web location using HTTP Post
		// - Post of XML will be sent to $ftp_web_server
		if (($mTransferLocationTeleFlowRelay) && ($ftp_web_server <> ''))
		{
			$goTransferList = explode(chr(13).chr(10), $ftpTextWeb );
			foreach ($goTransferList as $fileToSend)
			{
				if (trim($fileToSend) <> '')
				{
					output("Sending ".$fileToSend);
					
					$handle = fopen($fileToSend, "rb");  // windows MUST have a 'b' for binary
					$contents = fread($handle, filesize($fileToSend));
					fclose($handle);
					//output('Size: '.strlen($contents));
					//output(base64_encode($contents));
					//output('Size: '.strlen($contents));
					//output('End');
					
					$data = '<REQUEST action="FileUpload" name="'.basename($fileToSend).'" client_id="'.$mClientId.'"><FileContents>'.base64_encode($contents).'</FileContents></REQUEST>';
					$result = ffl_XMLPostStream($ftp_web_server, $data);
					
					// Break up the result, and check success
					$mResultCode	='Nothing';
					$mFileSize 		= 0;
					$xml = new SimpleXMLElement($result['body']);
					foreach($xml->children() as $child)
					{
						//echo $child->getName() . ": " . $child . "<br />";
						if ($child->getName() == "SUCCESS")
						{
							// Get the name
							foreach($child->attributes() as $a => $b)
							{
								if ($a == "result")
								{
								   $mResultCode = $b;
								}
								if ($a == "filesize")
								{
								   $mFileSize = intval($b);
								}
							}
						}
					}					
					output("FileSize Remote: ".$mFileSize.".");
					// Compare the size.  If they are  identical, then rename, commit in the db, and remove locally
					output("FileSize __Here: ".filesize($fileToSend).".");
					//die;
					
					// Get the local file conf number to look up in the database
					// eg:  4_ZZZZ_6046382650_2000_Spears_032408_060329_0_065423_orig.wav
					// 065423 = confirm number (8)
					$filenameparts = explode("_", basename(changeFileExtension($fileToSend,'')));
					foreach ($filenameparts as $fileBit)
					{
						// Get the last part that is numeric
						if (is_numeric($fileBit))
						{
							$mfile_conf_num = intval($fileBit);
						}
					}

					// Compare the size.  If they are  identical, then rename, commit in the db, and remove locally
					output("Conf ID: ".$mfile_conf_num);
					
					if (($mFileSize) == (filesize($fileToSend)))
					{
					
						// Update the table 
						$update_statement = (sprintf(
"UPDATE
	dictate_file                     
SET                                
	file_status_web = '%s'
WHERE                              			
	file_conf_num = %s", $cUploaded , $mfile_conf_num));  
						if (!$res = mysql_query($update_statement))
						{
							output("ERROR: Failure Updating Web Conf Num: ". $mfile_conf_num);
							output($update_statement);
						}
					
					}
					// Otherwise, destroy, let the next attempt try again.
					else
					{
						// Update the table 
						$update_statement = (sprintf(
"UPDATE
  dictate_file                     
SET                                
  file_status_web = '%s'
WHERE                              			
  file_conf_num = %s", $cPendingRetry , $mfile_conf_num));  
						if (!$res = mysql_query($update_statement))
						{
							output("ERROR: Failure Updating Web Conf Num: ". $mfile_conf_num);
							output($update_statement);
						}
						output("POST-Local ".filesize($fileToSend)."\t".basename($fileToSend)."\n- FILESIZE ERROR");
					}					
					
					// Finally, track to delete the local copy
					$mFilesTempArray[$mFilesTempCount] = $fileToSend;
					$mFilesTempCount++;
					
				}
			}		
		}

		
		// If we were supposed to have done something 
		if ($totalRows_rsSearchResult > 0)
		{
			// Did anything happen?
			if (($transferToTranscription == 0) &&
				($transferToTeleFlowRelay == 0) &&
				($transferToWeb == 0))
			{
				// It could be that another computer has the recording.  We assume that it will pick it up.
				// - Also, it could be that the is no process in place to transfer either or the transcription or web files
				output (" ");
				output ("NOTE: Several files were marked as to processed, but nothing was found on this computer");
				output (" ");
			}
			
			// Clean up the local processing copies of the files
			foreach ($mFilesTempArray as $mFileDel)
			{
				// Finally, remove the local copy
				if (file_exists($mFileDel))
				{
					unlink($mFileDel);			
				}
			}					
		}
		

		//////////////////////////////////////
		//  PHYSICIANS - Names to Website
		//////////////////////////////////////
		if (($proc_physicians == true)  && ($proc_to_web == true))
		{
			// Location of the Recorded name of Physicians  @@@
			$ftp_loc_physnames = 'c:\\junk\\'; 			

			$query_statement = (sprintf(
"SELECT   
	dp_id,              
	dictate_client_id,  
	physician_name_file 
FROM 
	dictate_physician   
WHERE
	physician_status = 'Enabled'
and physician_name_file <> ''
and file_transfer_status = '%s'
and ivr_computer_name    = '%s'", $cPending, $mCompName));

			 
			//output $query_statement;	 
			$rsSearchResult = mysql_query($query_statement, $dictation) or die(mysql_error());
			$totalRows_rsSearchResult = mysql_num_rows($rsSearchResult);  

			output ("Total company recordings for processing = ".$totalRows_rsSearchResult);
			while ($row_rsSearchResult = mysql_fetch_assoc($rsSearchResult))			
			{   
				// Get the fields
				$mdp_id                   = $row_rsSearchResult['dp_id'];
				$mdictate_client_id       = $row_rsSearchResult['dictate_client_id'];
				$mphysician_name_file     = $row_rsSearchResult['physician_name_file'];
			
				$fileWAV   = $ftp_loc_physnames.$mphysician_name_file;
				
				// If the file was created, then, transfer
				if (file_exists($fileWAV))
				{
					// Create the FTP command list
					$ftptxtcmd = ftpcmd($fileWAV);
					$ftptext  = 'passive'.chr(13).chr(10);
					$ftptext .= 'bin'.chr(13).chr(10);
					$ftptext .= 'put '.slashToBackslash($fileWAV).chr(13).chr(10);
					$ftptext .= 'quit'.chr(13).chr(10);

					// Write it 
					$handleIdx  = fopen($ftptxtcmd, "w");  // Open for Write from beginning
					fwrite($handleIdx, $ftptext);
					fclose($handleIdx);
						
					// File was converted.  Send
					$ftpsend = 'ftp\ftps.exe '.$ftp_web_server.
						' -user:'.$ftp_web_user_name.
						' -password:'.$ftp_web_user_pass.
						' -z -s:'.$ftptxtcmd;
					output($ftpsend);
					
					$result = shell_exec($ftpsend);
					
					// @@@ Save to OUT file, and then parse for success flags
					// ASSUMING SUCCESS...
					
					// Update the table 
$update_statement = (sprintf(
"UPDATE
	dictate_physician                     
SET                                
	file_transfer_status = '%s'
WHERE                              			
	dp_id = %s", $cUploaded , $mdp_id));
		  
					output($update_statement);
					//if (!$res = mysql_query($update_statement))
					/*
					{
						output("ERROR: Failure Updating FileID: ". $mfile_id);
						output($update_statement);
					}
					*/
				} // file exists GSM
				else
				{
					output("ERROR: Could not find ".$fileWAV);		
				}
			} // while ($row_rsSearchResult = mysql_fetch_assoc($rsSearchResult));
			
		} // if proc_physicians
		else
		{
			output ("*** NOTE: PHYSICIAN NAME PROCESSING - DISABLED");
		}
		
		output (" "); // A blank space
		
	} // while clients with dictation

} // If clients found

output ("Sleeping for 15 seconds...");
sleep(15);

} while (TRUE);
  
output ("Done! \n");
?>