<?php
/**
 * Database.php
 * 
 * The Database class is meant to simplify the task of accessing
 * information from the website's database.
 *
 * Started by: Jpmaster77 a.k.a. The Grandmaster of C++ (GMC)
 * Redeveloped: vbaker@engenic.com
 * Last Updated: May 7, 2009
 */     
 
///////////////////
// ESSENTIAL:
// Note:  You must use DATABASE_MASTER in SQL queries for the master database.  This is because only one database can be active per server
///////////////////

/* 
===PROBLEMS and ISSUES===
Both MySQL and MS SQL Server have inherent flaws when it comes to working with
multiple database spaces simulateously.  With MySQL, you must reference the Database Space 
within the SQL Command (eg: database.table).  You cannot do with with SQL Server.  With MySQL,
you can only reference one database with the mysql_select_db.  With SQL Server,
it is assumed you can access multiple spaces with that method.  Unfortunately, the 
two together make creating SQL Statements painful for multiple spaces.  Therefore,
MySQL will be used for external systems with multiple clients, and SQL Server will have no
client create capabilities.  

To accomodate this compromise, sqlConvert_* functions have been imposed on the application
*/

 
// This must be found relative to the php include_path  eg: c:\php5\pear
require_once 'Crypt/Blowfish.php'; // Crypto class
// Standard Library
require_once($websiteLibrary.'calcWorkUnit.php');


class MySQLDB
{
	// START OF CLASS

	var $connectionSystem;   //The database connection for the System level Client/User authentication and system level
	var $connectionClient;   //The database connection for the Client Databse
	var $num_active_users;   //Number of active users viewing site
	var $num_active_guests;  //Number of active guests viewing site
	var $num_members;        //Number of signed-up users
	
	// Global connect for SQL Server only
	var $globalSQLSrv_conn;
	
	// Global data connection information
	var $db_client_id;
	var $db_driver;
	var $db_type;
	var $db_host;
	var $db_host_remote;
	var $db_name;
	var $db_user;
	var $db_password;
	var $db_connect_options;
	/* Note: call getNumMembers() to access $num_members! */
			
	/* Class constructor */
	function MySQLDB()
	{
		/* Make connection to database at the SYSTEM level*/
		$this->sqlConvert_connect_db_master();
		//$this->connectionSystem = mssql_connect(SYS_DB_HOST, SYS_DB_USER, SYS_DB_PASS);
		//echo ' ---@@@@@ddd---- '. $this->connectionSystem."\n";
		
		$this->sqlConvert_select_db(SYS_DB_NAME, $this->connectionSystem);
		//mssql_select_db(SYS_DB_NAME, $this->connectionSystem);

		
		// Unset the connected flow
		unset($_SESSION['CLI_DB_CONNECTED']);
		
		// If we are logged in, then use the Client database, otherwise, go with the SYS database
		if ((isset($_SESSION['username'])) &&
			//($this->selectClientDatabaseByUsername($_SESSION['username']))) // Way before putting in admin ID override... needed to go with pre-determined client id
			($this->selectClientDatabaseByMasterClientId($_SESSION['userclientid'])))
		{
			/* Make connection to database at the CLIENT level*/
			// Mark that we are connected
			$_SESSION['CLI_DB_CONNECTED'] = true;
		}

		/**
		* Only query database to find out number of members
		* when getNumMembers() is called for the first time,
		* until then, default value set.
		*/
		$this->num_members = -1;

		// Only see visitors or guests once into the CLIENT tables
		if ((TRACK_VISITORS) && (isset($_SESSION['CLI_DB_HOST'])))
		{
			/* Calculate number of users at site */
			$this->calcNumActiveUsers();

			/* Calculate number of guests at site */
			$this->calcNumActiveGuests();
		}
	}

		
	// Conversion functions be MySQL and MSSQL interoperability

	// Build a functional FROM section
	function sqlConvert_FROM($pDatabase, $pTable)
	{
		/* Make connection to database at the SYSTEM level*/
		if (SYS_DATABASE_SYSTEM == "SQLServer")
		{
			// SQL Server
			return	$pTable.' ';  // Give a space for following SQL
		}
		else if (SYS_DATABASE_SYSTEM == "MS_PHP_SQLSvr")
		{
			// SQL Server
			return	$pTable.' ';  // Give a space for following SQL
		}
		else
		{
			// MySQL
			// Fully functional part
			if ($pDatabase <> '')
			{
				return	$pDatabase.'.'.$pTable.' ';  // Give a space for following SQL
			}
			else
			{
				return	$pTable.' ';  // Give a space for following SQL		
			}
		}
	} // sqlConvert_FROM


	// Connect to a host database at master 
	function sqlConvert_connect_db_master()
	{		
		global $globalSQLSrv_conn;
	
		if (SYS_DATABASE_SYSTEM == "SQLServer")
		{
			// SQL Server
			$this->connectionSystem = mssql_connect(SYS_DB_HOST, SYS_DB_USER, SYS_DB_PASS) or die();
			if (!$this->connectionSystem)
			{
				echo (providePhrase('cr', 520)); // Unable to connect
			}
		}
		else if (SYS_DATABASE_SYSTEM == "MS_PHP_SQLSvr")
		{
			// SQL Server
			// WARNING: !!!!! We must always select connectionSystem and not client for SQL Server due to inadequacies with PHP handling !!!!
			// -- Fix when SQL Server issues are repaired
			$connectionInfo = array("Database"=> DATABASE_MASTER);
			
			// IMPORTANT:  We are using a global connection for SQL Server in order to get it working.  However, the entire system should adopt this method
			$globalSQLSrv_conn = sqlsrv_connect( SYS_DB_HOST, $connectionInfo);
			if (!$globalSQLSrv_conn )
			{
				echo (providePhrase('cr', 521)); // Server Access Error
				die();
			}
			return $globalSQLSrv_conn;	
			//return mssql_select_db('['.$pDatabaseName.']', $pConnection) or die();		
			// Database names with '-' must be enclosed in [ ] 		
		}
		else
		{
			// MySQL
			$this->connectionSystem = mysql_connect(SYS_DB_HOST, SYS_DB_USER, SYS_DB_PASS) or die(mysql_error());
		}
		return true;
	} // sqlConvert_connect_db_master

	
	
	// Connect to a host database for client 
	function sqlConvert_connect_db_client($db_type, $db_host, $db_user, $db_password)
	{		
		if (SYS_DATABASE_SYSTEM == "SQLServer")
		{
			// SQL Server
			// WARNING: !!!!! We must always select connectionSystem and not client for SQL Server due to inadequacies with PHP handling !!!!
			// -- Fix when SQL Server issues are repaired
			$this->connectionClient = $this->connectionSystem;
			//$this->connectionClient = mssql_connect($db_host, $db_user, $db_password) or die();
		}
		else if (SYS_DATABASE_SYSTEM == "MS_PHP_SQLSvr")
		{
			// MS PHP SQL Server service account
			// WARNING: !!!!! We must always select connectionSystem and not client for SQL Server due to inadequacies with PHP handling !!!!
			// -- Fix when SQL Server issues are repaired
			$this->connectionClient = $this->connectionSystem;
			//$this->connectionClient = mssql_connect($db_host, $db_user, $db_password) or die();
		}
		else
		{
			// MySQL
			$this->connectionClient = mysql_connect($db_host, $db_user, $db_password) or die(mysql_error());
		}
		return true;
	} // sqlConvert_connect_db_system

	
	
	// Build a functional FROM section
	function sqlConvert_select_db($pDatabaseName, $pConnection)
	{		
		global $globalSQLSrv_conn;

		//echo 'here!!'.$pDatabaseName; die;
		if (SYS_DATABASE_SYSTEM == "SQLServer")
		{
			// SQL Server
			// WARNING: !!!!! We must always select connectionSystem and not client for SQL Server due to inadequacies with PHP handling !!!!
			// -- Fix when SQL Server issues are repaired
			//if (!$mLink = mssql_select_db($pDatabaseName,  $this->connectionSystem))
			if (!$mLink = mssql_select_db($pDatabaseName,  $this->connectionSystem))
			{
				echo (providePhrase('cr', 521)); // Server Access Error
				die();
			}
			return $mLink;	
			//return mssql_select_db('['.$pDatabaseName.']', $pConnection) or die();		
			// Database names with '-' must be enclosed in [ ] 			
		}
		else if (SYS_DATABASE_SYSTEM == "MS_PHP_SQLSvr")
		{
			// SQL Server
			// WARNING: !!!!! We must always select connectionSystem and not client for SQL Server due to inadequacies with PHP handling !!!!
			// -- Fix when SQL Server issues are repaired
			$connectionInfo = array("Database"=> $pDatabaseName);
			
			// IMPORTANT:  We are using a global connection for SQL Server in order to get it working.  However, the entire system should adopt this method
			$globalSQLSrv_conn = sqlsrv_connect( SYS_DB_HOST, $connectionInfo);
			if (!$globalSQLSrv_conn )
			{
				echo (providePhrase('cr', 521)); // Server Access Error
				die();
			}
			return $globalSQLSrv_conn;	
			//return mssql_select_db('['.$pDatabaseName.']', $pConnection) or die();		
			// Database names with '-' must be enclosed in [ ] 			
		}
		else
		{
			// MySQL
			return mysql_select_db($pDatabaseName, $pConnection) or die(mysql_error());
		}
	} // sqlConvert_select_db

	// Return a cursor
	function sqlConvert_sql_query($pQuery, $pConnection)
	{	
		global $cGlobalDebug, $globalSQLSrv_conn;
	
		// Log database entries in debug
		if ($cGlobalDebug)
		{
			// For debugging, lets catch the SQL statements being run.
			//LogTextToFileXMLTimeStamp('../'.SYSTEM_LOGFILES."Database-".Date('Y-m-d').".log", $pQuery);			
			LogTextToFileXMLTimeStamp(getRelativeSystemLogFilesPath()."Database-".Date('Y-m-d').".log", 
										'-- '.SYS_DATABASE_SYSTEM.chr(13).chr(10).
										$pQuery);		
		}

		// Database type
		if (SYS_DATABASE_SYSTEM == "SQLServer")
		{
			// SQL Server
			try
			{
				return mssql_query($pQuery, $pConnection);
			}
			catch(Exception $e)
			{
				// Log error
				LogTextToFileXMLTimeStamp(getRelativeSystemLogFilesPath()."Database-Error-".Date('Y-m-d').".log", 
											'-- '.SYS_DATABASE_SYSTEM.chr(13).chr(10).
											$pQuery);		
				echo (providePhrase('cr', 522)); // SQL Statement failure				
				die;				
			}
		}
		else if (SYS_DATABASE_SYSTEM == "MS_PHP_SQLSvr")
		{
			// SQL Server
			try
			{
				return sqlsrv_query($globalSQLSrv_conn, doubleQuoteToSingle($pQuery), array(), array( "Scrollable" => SQLSRV_CURSOR_KEYSET ));
			}
			catch(Exception $e)
			{
				LogTextToFileXMLTimeStamp(getRelativeSystemLogFilesPath()."Database-Error-".Date('Y-m-d').".log", 
											'-- '.SYS_DATABASE_SYSTEM.chr(13).chr(10).
											$pQuery);		
				echo (providePhrase('cr', 522)); // SQL Statement failure
				die;				
			}
		}
		else
		{
			// MySQL
			try
			{
				return mysql_query($pQuery, $pConnection);
			}
			catch(Exception $e)
			{
				LogTextToFileXMLTimeStamp(getRelativeSystemLogFilesPath()."Database-Error-".Date('Y-m-d').".log", 
											'-- '.SYS_DATABASE_SYSTEM.chr(13).chr(10).
											$pQuery);		
				echo (providePhrase('cr', 522)); // SQL Statement failure
				die;				
			}				
		}
	} // sqlConvert_sql_query

	// Counts number rows in a cursor
	function sqlConvert_sql_numrows($pResultSet)
	{
		if (SYS_DATABASE_SYSTEM == "SQLServer")
		{
			// SQL Server
			return mssql_num_rows($pResultSet);
		}
		else if (SYS_DATABASE_SYSTEM == "MS_PHP_SQLSvr")
		{
			// SQL Server
			return sqlsrv_num_rows($pResultSet);
		}
		else
		{
			// MySQL
			return mysql_numrows($pResultSet);
		}
	} // sqlConvert_sql_numrows

	
	// Once an insert has occured, get its id
	function sqlConvert_sql_insert_id($pConnection)
	{
		global $cGlobalDebug, $globalSQLSrv_conn;

		if (SYS_DATABASE_SYSTEM == "SQLServer")
		{
			// SQL Server - version of mysql_insert_id
			$id = false;
			$res = mssql_query('SELECT @@identity AS id', $pConnection);
			if ($row = mssql_fetch_row($res))
			{
				$id = trim($row[0]);
			}
			mssql_free_result($res);
			return $id;
		}
		else if (SYS_DATABASE_SYSTEM == "MS_PHP_SQLSvr")
		{
			// SQL Server - version of mysql_insert_id
			$id = false;
			$res = sqlsrv_query($globalSQLSrv_conn, 'SELECT @@identity AS id', array(), array( "Scrollable" => SQLSRV_CURSOR_KEYSET ));

			
			//if ($row = sqlsrv_fetch($res))
			if ($mRowArray = sqlsrv_fetch_array($res, SQLSRV_FETCH_ASSOC))
			{
				$id = trim($mRowArray['id']);
			}
			return $id;
		}
		else
		{
			// MySQL
			return mysql_insert_id($pConnection);
		}
	} // sqlConvert_sql_numrows

		
	// Access a field of a result set
	function sqlConvert_sql_result($pResultSet, $pRow, $pFieldName)
	{
		if (SYS_DATABASE_SYSTEM == "SQLServer")
		{
			// SQL Server
			return mssql_result($pResultSet, $pRow, $pFieldName);
		}
		else if (SYS_DATABASE_SYSTEM == "MS_PHP_SQLSvr")
		{
			// SQL Server
			$mRowArray = sqlsrv_fetch_array($pResultSet, SQLSRV_FETCH_ASSOC, SQLSRV_SCROLL_ABSOLUTE, $pRow);
			return $mRowArray[$pFieldName];
		}
		else
		{
			// MySQL
			return mysql_result($pResultSet, $pRow, $pFieldName);
		}		
	}
	
	
	// Fetches a row from a cursor
	function sqlConvert_sql_fetch_array($pResultSet)
	{
		if (SYS_DATABASE_SYSTEM == "SQLServer")
		{
			// SQL Server
			return mssql_fetch_array($pResultSet);
		}
		else if (SYS_DATABASE_SYSTEM == "MS_PHP_SQLSvr")
		{
			// SQL Server
			return sqlsrv_fetch_array($pResultSet, SQLSRV_FETCH_ASSOC);
		}
		else
		{
			// MySQL
			return mysql_fetch_array($pResultSet);
		}
	} // sqlConvert_sql_fetch_array

	// Standard UTC timestamp function
	function sqlConvert_UTC_timestamp()
	{
		if (SYS_DATABASE_SYSTEM == "SQLServer")
		{
			// SQL Server
			return "GetUTCDate() ";
		}
		else if (SYS_DATABASE_SYSTEM == "MS_PHP_SQLSvr")
		{
			// SQL Server
			return "GetUTCDate() ";
		}
		else
		{
			// MySQL
			return "UTC_Timestamp() ";
		}
	} // sqlConvert_UTC_timestamp

	// Create a textual date output etc
	// - YYY-MM-DD
	function sqlConvert_DateFormat_Date($pFieldName)
	{
		if (SYS_DATABASE_SYSTEM == "SQLServer")
		{
			// SQL Server
			return "CONVERT(varchar(10), ".$pFieldName.", 120) ";
		}
		else if (SYS_DATABASE_SYSTEM == "MS_PHP_SQLSvr")
		{
			// SQL Server
			return "CONVERT(varchar(10), ".$pFieldName.", 120) ";
		}
		else
		{
			// MySQL
			return "DATE_FORMAT(".$pFieldName.",'%Y-%m-%d') ";
		}
	} // sqlConvert_DateFormat_Date


	// Create a textual date  output etc
	// - HH:MM:SS
	function sqlConvert_DateFormat_Time($pFieldName)
	{
		if (SYS_DATABASE_SYSTEM == "SQLServer")
		{
			// SQL Server
			return "CONVERT(varchar(8), ".$pFieldName.", 108) ";
		}
		else if (SYS_DATABASE_SYSTEM == "MS_PHP_SQLSvr")
		{
			// SQL Server
			return "CONVERT(varchar(8), ".$pFieldName.", 108) ";
		}
		else
		{
			// MySQL
			return "DATE_FORMAT(".$pFieldName.",'%H:%i:%s') ";
		}
	} // sqlConvert_DateFormat_Time

	// Drop in a full timestamp for current time
	// - YYYY-MM-DD HH:MM:SS
	function sqlConvert_DateFormat_FullTimeStamp_Now()
	{
		// Eg: YYYY-MM-DD HH:MM:SS  (19 chars)
		if (SYS_DATABASE_SYSTEM == "SQLServer")
		{
			// SQL Server
			return "CONVERT(varchar(19), GetUTCDate(), 120) ";
		}
		else if (SYS_DATABASE_SYSTEM == "MS_PHP_SQLSvr")
		{
			// SQL Server
			return "CONVERT(varchar(19), GetUTCDate(), 120) ";
		}
		else
		{
			// MySQL
			return "DATE_FORMAT(UTC_Timestamp(),'%Y-%m-%d %H:%i:%s') ";
		}
	} // sqlConvert_DateFormat_FullTimeStamp_Now

	// Drop in a full timestamp for current time
	function sqlConvert_DateAdd_Seconds($pFieldName, $pSeconds)
	{
		// Eg: YYYY-MM-DD HH:MM:SS  (19 chars)
		if (SYS_DATABASE_SYSTEM == "SQLServer")
		{
			// SQL Server
			return "DATEADD(ss, ".$pSeconds.", ".$pFieldName.") ";
		}
		else if (SYS_DATABASE_SYSTEM == "MS_PHP_SQLSvr")
		{
			// SQL Server
			return "DATEADD(ss, ".$pSeconds.", ".$pFieldName.") ";
		}
		else
		{
			// MySQL
			return "DATE_ADD(".$pFieldName.", INTERVAL ".$pSeconds." SECOND) ";
		}
	} // sqlConvert_DateAdd_Seconds
	
	// Drop in a full timestamp for current time
	function sqlConvert_DateAdd_Days($pFieldName, $pDays)
	{
		// Eg: YYYY-MM-DD HH:MM:SS  (19 chars)
		if (SYS_DATABASE_SYSTEM == "SQLServer")
		{
			// SQL Server
			return "DATEADD(dd, ".$pDays.", ".$pFieldName.") ";
		}
		else if (SYS_DATABASE_SYSTEM == "MS_PHP_SQLSvr")
		{
			// SQL Server
			return "DATEADD(dd, ".$pDays.", ".$pFieldName.") ";
		}
		else
		{
			// MySQL
			return "DATE_ADD(".$pFieldName.", INTERVAL ".$pDays." DAY) ";
		}
	} // sqlConvert_DateAdd_Seconds
	
	// Drop in a full datetime for current time
	function sqlConvert_DayName($pFieldName)
	{
		if (SYS_DATABASE_SYSTEM == "SQLServer")
		{
			// SQL Server
			return "DATENAME(weekday, ".$pFieldName.") ";
		}
		else if (SYS_DATABASE_SYSTEM == "MS_PHP_SQLSvr")
		{
			// SQL Server
			return "DATENAME(weekday, ".$pFieldName.") ";
		}
		else
		{
			// MySQL
			return "DAYNAME(".$pFieldName.") ";
		}
	} // sqlConvert_DayName	

	// Attempt to find a string in another
	function sqlConvert_Locate($pFieldName, $pInString)
	{
		if (SYS_DATABASE_SYSTEM == "SQLServer")
		{
			// SQL Server
			return "CharIndex(".$pFieldName.", '".$pInString."', 1) ";
		}
		else if (SYS_DATABASE_SYSTEM == "MS_PHP_SQLSvr")
		{
			// SQL Server
			return "CharIndex(".$pFieldName.", '".$pInString."', 1) ";
		}
		else
		{
			// MySQL
			return "locate(".$pFieldName.", '".$pInString."') ";
		}	
	} // sqlConvert_Locate
	
	// Catenate padding around a field (used for locate generally)
	function sqlConvert_Concat($pC1,$pFieldName,$pC2)
	{
		if (SYS_DATABASE_SYSTEM == "SQLServer")
		{
			// SQL Server
			return $pC1."+".$pFieldName."+".$pC2." ";
		}
		else if (SYS_DATABASE_SYSTEM == "MS_PHP_SQLSvr")
		{
			// SQL Server
			return $pC1."+".$pFieldName."+".$pC2." ";
		}
		else
		{
			// MySQL
			return "CONCAT('".$pC1."',".$pFieldName.",'".$pC2."') ";
		}
	} // sqlConvert_DayName	
	
	// Limit the report 
	function sqlConvert_Limit_MySQL($pReportLines, $pReportPage)
	{
		if (SYS_DATABASE_SYSTEM == "SQLServer")
		{
			// For SQL Server, each report will need to scroll and scan (see xml_mediacmdbody.php for example)
			return " ";
		}
		else if (SYS_DATABASE_SYSTEM == "MS_PHP_SQLSvr")
		{
			// For SQL Server, each report will need to scroll and scan (see xml_mediacmdbody.php for example)
			return " ";
		}
		else
		{
			// MySQL
			return "LIMIT ".$pReportLines*($pReportPage-1).",".$pReportLines." ";
		}
	}

	// Error number - Assumes an error has been caught
	function sqlConvert_sql_errno()
	{
		if (SYS_DATABASE_SYSTEM == "SQLServer")
		{
			// For SQL Server
			return 1;  // Force an error number for now...  @@@ What function do we use here? @@@
		}
		else if (SYS_DATABASE_SYSTEM == "MS_PHP_SQLSvr")
		{
			// For SQL Server
			// @@@ Update with this:  sqlsrv_errors()
			return 1;  // Force an error number
		}
		else
		{
			// MySQL
			return mysql_errno();
		}		
	}
	// Error decscription - Assumes an error has been caught
	function sqlConvert_sql_error()
	{
		if (SYS_DATABASE_SYSTEM == "SQLServer")
		{
			// For SQL Server
			return mssql_get_last_message();  // This will show details about the last error encountered
		}
		else if (SYS_DATABASE_SYSTEM == "MS_PHP_SQLSvr")
		{
			$mResult = '';
			foreach( sqlsrv_errors() as $error)
			{
				//echo "SQLSTATE: ".$error[ 'SQLSTATE']."\n";
				//echo "code: ".$error[ 'code']."\n";
				//echo "message: ".$error[ 'message']."\n";
				$mResult = $error['SQLSTATE'].' '.$error['code'].' '.$error['message'];
			}
			return $mResult;
		}
		else
		{
			// MySQL
			return mysql_error();
		}		
	}
	
	
	// Quick method to switch back to system database
	function switchToSystemDatabase()
	{
		return $this->sqlConvert_select_db(SYS_DB_NAME, $this->connectionSystem);
	}
	
	// Quick method to switch back to system database
	function switchToClientDatabase()
	{
		return $this->sqlConvert_select_db(SYS_DB_NAME, $this->connectionClient);
	}
	
/**
testDatabaseSystemLevel - Checks whether or not the 
database is active.  Returns "1" if available  0 if not
*/
	function selectClientDatabase($db_type, $db_host, $db_name, $db_user, $db_password, $db_connect_options)
	{	
		///////////////////////////////////////////
		// DATABASE CHANGE FUNCTION
		///////////////////////////////////////////
		// Connect options not currently used - $db_connect_options
		$this->sqlConvert_connect_db_client($db_type, $db_host, $db_user, $db_password);
		return $this->sqlConvert_select_db($db_name, $this->connectionClient);
	}		

/* 	
selectClientDatabaseByUserId - Selects the database associated with a username
*/
	function selectClientDatabaseByUserId($u_id)
	{	
		// The UserID is the ID from the user_master table
		$username = $this->getUserFromMasterByUserId($u_id);
		$connected = $this->selectClientDatabaseByUsername($username);
		if ($connected)
		{
			// Mark that we are connected
			$_SESSION['CLI_DB_CONNECTED'] = true;
		}
		return $connected;
	}

/* 	
selectClientDatabaseByMasterClientId - Selects the database associated with a  master client id
*/
	function selectClientDatabaseByMasterClientId($master_client_id)
	{	
		// Make sure the information exists in the system table
		$q =
"SELECT 
	client_name,
	client_id,
	client_db_driver, 
	client_db_type,
	client_db_host, 
	client_db_host_remote,
	client_db_name, 
	client_db_user, 
	client_db_password, 
	client_db_connect_options
FROM 
 	".$this->sqlConvert_FROM(DATABASE_MASTER,'client c')."
WHERE 
	client_id = ".$master_client_id;
		//echo($q); die;

		$result = $this->sqlConvert_sql_query($q, $this->connectionSystem);
		if(!$result || ($this->sqlConvert_sql_numrows($result) < 1))
		{
			return false; //Indicates username not found
		}
		else
		{
			// Note: This will select the database 
			return $this->selectDecryptedClientDatabase($result);
		}		
	} // selectClientDatabaseByMasterClientId
	
/* 	
selectClientDatabaseByUsername - Selects the database assoicated with a username
*/
	function selectClientDatabaseByUsername($username)
	{	
		// Make sure the information exists in the system table
		$q =
"SELECT 
	u.user_master_id, 
	u.user_login_name, 
	u.client_id client_id, 
	client_name,
	client_db_driver, 
	client_db_type, 
	client_db_host, 
	client_db_host_remote,
	client_db_name, 
	client_db_user, 
	client_db_password, 
	client_db_connect_options
FROM 
	".$this->sqlConvert_FROM(DATABASE_MASTER,'user_master u').",
	".$this->sqlConvert_FROM(DATABASE_MASTER,'client c')."	
WHERE 
	u.client_id = c.client_id 
and user_login_name = '".$username."'";		
		//echo($q); die;

		$result = $this->sqlConvert_sql_query($q, $this->connectionSystem);
		//echo ('here:'.$this->sqlConvert_sql_numrows($result) ); die;
		if(!$result || ($this->sqlConvert_sql_numrows($result) < 1))
		{
			return false; //Indicates username not found
		}
		else
		{
			// Note: This will select the database 
			return $this->selectDecryptedClientDatabase($result);
		}
	} // selectClientDatabaseByUsername

	
/* getClientIdByClientName - allows software to determine which client is the active client for sign ups
or other purposes.  The default client name is stored in the variable $appCompanySvcCode  in LocalSettings_Appearnce.php
eg: $mClient_Id = $database->getClientIdByClientName($appCompanySvcCode);
*/
	function getClientIdByClientName($pClientName)
	{	
		// See if the information exists in the system table
		$q = "
SELECT 
	client_id 
FROM 
	".$this->sqlConvert_FROM(DATABASE_MASTER,'client')."	
WHERE
	client_name = '".$pClientName."' and client_status = 'ACTIVE'";
	
		$result = $this->sqlConvert_sql_query($q, $this->connectionSystem);
		if(!$result || ($this->sqlConvert_sql_numrows($result) < 1))
		{
			return -1; //Indicates client not found
		}
		else
		{
			if ($dbarray = $this->sqlConvert_sql_fetch_array($result))
			{
				return $dbarray['client_id'];
			}
			else
			{
				return -1; //Indicates client not found
			}
		}
	} // getClientIdByClientName
	
/*
selectByIPAddressClientDatabase - Select the client automatically based on the IP address it originated.
This way, an automated system from an IP address can run without needing to sign in using a password etc.  

NOTE: This would be found at the start of the program, and override other code
	$mIP = getClientIpAddress();
	$mClientId = 0
	if (!$database->selectByIPAddressClientDatabase($mIP, $mClientId))
	{
		$responsecode = "ERROR: Failed to Authorize IP address ".$mIP;
	}
	echo "client id: ".$mClientId
*/
	function selectByIPAddressClientDatabase($pIpAddress, &$varClientId)
	{		
		// Default to unknown
		$varClientId = -1;
	
		// Is the IP address in the master IP table
		$this->switchToSystemDatabase();
		
		$q =
"SELECT 
	cia_id, 
	client_id, 
	cia_ip_address_range_start,
	cia_ip_address_range_end, 
	cia_ip_address_description
FROM 
	".$this->sqlConvert_FROM(DATABASE_MASTER,'client_ip_access c')."		
WHERE 
	cia_status = 'ACTIVE' ";
		//echo($q);

		// Get the LONG values for the IP address recevied from CLIENT
		$ipClientLong = ip2long($pIpAddress);
		if ($ipClientLong)
		{
			$result = $this->sqlConvert_sql_query($q, $this->connectionSystem);
			while ($dbarray = $this->sqlConvert_sql_fetch_array($result))
			{
				// Get the long values
				$ipRangeStart = ip2long($dbarray['cia_ip_address_range_start']);
				$ipRangeEnd   = ip2long($dbarray['cia_ip_address_range_end']);
				
				// Did we find it? 
				if (($ipClientLong >= $ipRangeStart) &&
					($ipClientLong <= $ipRangeEnd))
				{
					// Found it, now lets try and select it
					$varClientId = $dbarray['client_id'];
					
					// Remember that this session is an autologon
					$_SESSION['CLIENTID_BY_IP_ADDR'] = $varClientId;
					// Note: This will select the database 
					// This will return: true or false
					return $this->selectClientDatabaseByMasterClientId($dbarray['client_id']);				
				}
			}			
		}
		else
		{
			return false; //Indicates invalid ip address
		}
	}  // selectByIPAddressClientDatabase

	
/*
Crypt functions to 
*/		
	function padEncryptToHex($cKey, $pStringToCrypt)
	{
		// Only crypt non-blank data
		if ($pStringToCrypt <> '')
		{
			$blowfish = new Crypt_Blowfish($cKey);  
			
			$mPaddedStr = pkcs5_pad($pStringToCrypt, 8);
			$encrypted_str 	= bin2hex($blowfish->encrypt($mPaddedStr));
			return $encrypted_str;
		}
		else
		{
			return '';
		}
	} // padEncryptToHex
	
	function padHexToDecrypt($cKey, $pStringToDeCrypt)
	{
		// Only decrypt non-blank data
		if ($pStringToDeCrypt <> '')
		{
			$blowfish = new Crypt_Blowfish($cKey); 
			
			// Crypt to Hex
			$mBinStr = '';
			$len = strlen($pStringToDeCrypt);
			for($i=0;$i<$len;$i+=2) 
			{
				$mBinStr .= pack("C",hexdec(substr($pStringToDeCrypt,$i,2)));
			}
			

			// NOTE: @@@ THIS SHOULD BE THE WAY WE DO IT, HOLD OFF UNTIL SYSTEM UPGRADE @@@
			$decrypted_str = pkcs5_unpad($blowfish->decrypt($mBinStr));
			//$decrypted_str = trim(trim($blowfish->decrypt($mBinStr)), "\x00..\x1F");

			return $decrypted_str;
		}
		else
		{
			return '';
		}
	} // padHexToDecrypt
	
/*
selectDecryptedClientDatabase - Select the client database by decrypting the connection information
*/	
	function selectDecryptedClientDatabase($clientRecord)
	{
		// * We now have to shift to the CLIENT database from the SYSTEM database to see if the password is a match
		// * If the login is correct, we set the CLI SESSION variables and contine
		
		// First, get the database details
		$dbarray = $this->sqlConvert_sql_fetch_array($clientRecord);
	
		$this->db_client_id			= stripslashes($dbarray['client_id']);
		$this->db_driver			= stripslashes($dbarray['client_db_driver']);
		$this->db_type				= stripslashes($dbarray['client_db_type']);
		$this->db_host				= stripslashes($dbarray['client_db_host']);
		$this->db_host_remote		= stripslashes($dbarray['client_db_host_remote']);
		$this->db_name				= stripslashes($dbarray['client_db_name']);
		$this->db_user				= stripslashes($dbarray['client_db_user']);
		$this->db_password			= stripslashes($dbarray['client_db_password']);  // encrypted
		$this->db_connect_options	= stripslashes($dbarray['client_db_connect_options']);

		$mLoc = '../'.SYS_CRYPTO_LOCATE_KEYFILE;  // @@@ Should use some kind of session home variable. eg: $incPath
		if (!file_exists($mLoc))
		{
			$mLoc = '../../'.SYS_CRYPTO_LOCATE_KEYFILE;
		}

		// Get the crypt key to decrypt the stored password
		$cBinUser = '';
		$cBinPass = '';
		$cKey=$this->getCryptKey($mLoc); // Define found in database file
		if ($cKey) 
		{
			// This encyption uses padding 
			$decrypted_user		= $this->padHexToDecrypt($cKey, $this->db_user);
			$decrypted_password = $this->padHexToDecrypt($cKey, $this->db_password);
		}
		else
		{
			// Password was not encrypted, or someone has removed the cryptkey
			$decrypted_user = $this->db_user;
			$decrypted_password = $this->db_password;
		}
		//echo ('---'.$this->db_host.'---'.$this->db_name.'---'.$this->db_user.'---'.$decrypted_user.'---'.$decrypted_password.'---'); die;

		// select the new database
		if (DATABASE_LOCAL == "YES")
		{
			return $this->selectClientDatabase($this->db_type, $this->db_host, $this->db_name, $decrypted_user, $decrypted_password, '');
		}
		else
		{
			return $this->selectClientDatabase($this->db_type, $this->db_host_remote, $this->db_name, $decrypted_user, $decrypted_password, '');
		}
		
	}  // selectDecryptedClientDatabase
			

/*			
// Load the systems crypt key
*/
	function getCryptKey($pFile)
	{
		$mKey = '';
		$xml = simplexml_load_file($pFile);

		//echo $xml->getName() . "<br />";
		foreach($xml->children() as $child)
		{
			//echo $child->getName() . ": " . $child . "<br />";
			foreach($child->attributes() as $a => $b) 
			{
				//echo $a,'=',$b,"<br />";
				if (stristr("DataBaseKey",$a))
				{
					$mKeyFile = $b;
					$mKey = $b;
					
					// Load the crypt key
					// Open the Log file
					$mCryptFile = '../'.$b;

					$mCryptFile = '../'.$b;  // @@@ Should use some kind of session home variable. eg: $incPath
					if (!file_exists($mCryptFile))
					{
						$mCryptFile = '../../'.$b;
						if (!file_exists($mCryptFile))
						{
							echo 'ERROR: Encyption system missing key file';
							die;
						}
					}
					// Open the key file
					$handleIdx  = fopen($mCryptFile, "r");  // Open for Read
					// Load the Key
					$mKey = fgets($handleIdx, 1024*16);
					fclose($handleIdx);
				}
			}
		}	
		return $mKey;
	} // getCryptKey

/*
createClientTables - runs a SQL script 
*/
function createClientTables($sqlCreateScriptArray, &$varErrorDescription)
{
	$sqlErrorCode = '';
	
	//Process the sql file by statements
	foreach ($sqlCreateScriptArray as $stmt) 
	{
		if (strlen($stmt)>3)
		{
			//echo ($stmt.'<br /><br />');

			// Remove any comments (--), that will kill the SQL executor
			$cleanStmt = SQLCommandClean($stmt);

			// Log the create script
			LogTextToFileXMLTimeStamp(getRelativeSystemLogFilesPath()."Database-".Date('Y-m-d').".log", 
										'-- '.SYS_DATABASE_SYSTEM.chr(13).chr(10).
										$cleanStmt);	
			
			// Run it
			$result = $this->sqlConvert_sql_query($cleanStmt, $this->connectionClient);
			if (!$result)
			{
				$sqlErrorCode = $this->sqlConvert_sql_errno();
				$sqlErrorText = $this->sqlConvert_sql_error();

				// Log the error
				LogTextToFileXMLTimeStamp(getRelativeSystemLogFilesPath()."Database-".Date('Y-m-d').".log", 
											'-- ERROR: '.$sqlErrorCode.' - '.$sqlErrorText.chr(13).chr(10)
										);	
				// Return the code for alert
				$varErrorDescription = $sqlErrorText;
			}
		}
	}
	
	return $sqlErrorCode;
} // createClientTables

	
/**
testDatabaseSystemLevel - Checks whether or not the 
database is active.  Returns "1" if available  0 if not
*/
function testDatabaseSystemLevel()
{
	// Make sure the information exists in the system table
	$q = "
SELECT
	count(*) total 
FROM 
	".$this->sqlConvert_FROM(DATABASE_MASTER,'client')."		
";
	//echo($q);

	$result = $this->sqlConvert_sql_query($q, $this->connectionSystem);
	if(!$result || ($this->sqlConvert_sql_numrows($result) < 1))
	{
		return 0; //Indicates username failure
	}

	return 1;
} // testDatabaseSystemLevel


/**
systemDatabaseIndicateNewData - Lets the master database know that  
there is new data to process.  Client and Application will be updated, and if not existing, created.
eg: systemDatabaseIndicateNewData($_SESSION['userclientid'],'APPOINTMENT_REMINDER');
*/
function systemDatabaseIndicateNewData($pClientId, $pApplicationSubscription, $pOrder = 0)
{
	// See if an entry exists
	$q = "
SELECT
	app_type_name
FROM 
	".$this->sqlConvert_FROM(DATABASE_MASTER,'client_application_subscription')."
WHERE
	app_type_name = '".$pApplicationSubscription."'
AND client_id = ".$pClientId;
	
	// NOTE: If an entry is not active, we are still going to count it 
	$result = $this->sqlConvert_sql_query($q, $this->connectionSystem);
	if ($dbarray = $this->sqlConvert_sql_fetch_array($result))
	{
		// We have an entry already
		// -- Update the entry to indicate new data
		$qUpdate = "
UPDATE
	".$this->sqlConvert_FROM(DATABASE_MASTER,'client_application_subscription')."			
SET
	last_data_change_dtm_utc = ".$this->sqlConvert_DateFormat_FullTimeStamp_Now()."   
WHERE
	app_type_name = '".$pApplicationSubscription."' 
AND client_id = ".$pClientId."
AND subscription_status = 'ACTIVE'";	
	
		$result = $this->sqlConvert_sql_query($qUpdate, $this->connectionSystem);		
		return true;
	}
	else
	{
		// An entry does not currently exist
		// -- Create one
		$qInsert = "
INSERT INTO 
	".$this->sqlConvert_FROM(DATABASE_MASTER,'client_application_subscription')."			
(
	`client_id`, 
	`app_type_name`, 
	`app_type_group_menu_order`,
	`subscription_status`,
	`last_data_change_dtm_utc`
) 
VALUES 
(
	'$pClientId', 
	'$pApplicationSubscription',
	$pOrder,
	'ACTIVE',
	".$this->sqlConvert_DateFormat_FullTimeStamp_Now()."
)";		
	
		$result = $this->sqlConvert_sql_query($qInsert, $this->connectionSystem);
		if ($result)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
} // systemDatabaseIndicateNewData

/**
systemDatabaseLastScannedDate - Lets the master database know that  
the data was reviewed for an application type
eg: systemDatabaseLastScannedDate($_SESSION['userclientid'],'APPOINTMENT_REMINDER');
*/
function systemDatabaseLastScannedDate($pClientId, $pApplicationSubscription)
{
	$qUpdate = "
UPDATE
	".$this->sqlConvert_FROM(DATABASE_MASTER,'client_application_subscription')."			
SET
	last_data_scan_dtm_utc = ".$this->sqlConvert_DateFormat_FullTimeStamp_Now()."   
WHERE
	app_type_name = '".$pApplicationSubscription."' AND
	client_id = ".$pClientId;

	$result = $this->sqlConvert_sql_query($qUpdate, $this->connectionSystem);
} // systemDatabaseLastScannedDate


/**
installApplicationCodesForSystem - Returns list of applications, and enters them into the subscription table for the client
*/
function installApplicationCodesForSystem($subclient, $pSystemCode)
{
	// Grab entries that have the code (works for both MySQL and SQL Server )
	$q = "
SELECT
	app_type_name,
	app_type_group_menu_order
FROM 
	".$this->sqlConvert_FROM(DATABASE_MASTER,'application_type')."				
WHERE
	app_type_group_codes != replace(app_type_group_codes, '".$pSystemCode."', '')
AND app_type_status = 'ACTIVE'
ORDER BY 
	app_type_group_menu_order
";
	//echo($q);

	$mCount = 0;
	$result = $this->sqlConvert_sql_query($q, $this->connectionSystem);
	while ($dbarray = $this->sqlConvert_sql_fetch_array($result))
	{
		// Get the long values
		$app_type_name 				= $dbarray['app_type_name'];
		$app_type_group_menu_order	= $dbarray['app_type_group_menu_order'];
		
		// First, add the application to the list so that it shows on the menu in cr_main etc (make sure it does not exist first)
		// Note: This reuses code from the system that indicates new data to process in order to create the application code
		if ($this->testApplicationCodeExists($app_type_name))
		{
			if ($this->systemDatabaseIndicateNewData($subclient,
									  $app_type_name,
									  $app_type_group_menu_order))
			{
				// Will add up if the entry exists or not.  Effectively provides a count as well as an insert
				$mCount = $mCount+1;
				
				// Finally, add any additional entries for CallOut settings
				$this->addClientApp($subclient,	$app_type_name);
			}
		}
	}
	
	return $mCount;
} // installApplicationCodesForSystem


/**
testDatabaseSystemLevel - Checks whether or not the 
database is active.  Returns "1" if available  0 if not
*/
function testApplicationCodeExists($pApplicationSubscriptionCode)
{
	// Make sure the information exists in the system table
	$q = "
SELECT
	count(*) total 
FROM 
	".$this->sqlConvert_FROM(DATABASE_MASTER,'application_type')."				
WHERE
	app_type_name = '".$pApplicationSubscriptionCode."'
and app_type_status = 'ACTIVE'	
";
	//echo($q);

	$result = $this->sqlConvert_sql_query($q, $this->connectionSystem);
	if(!$result || ($this->sqlConvert_sql_numrows($result) < 1))
	{
		return false; //Indicates database failure
	}

	$dbarray = $this->sqlConvert_sql_fetch_array($result);
	if ($dbarray['total'] > 0)
	{
		return true;
	}
	
	return false; //Indicates application not available for this system	
} // testApplicationCodeExists


// Get the file for the application code 
// eg:  $mProgram = $database->getWebLinkForApplicationCode($mClientId, "APPOINTMENT_REMINDER");
function getWebLinkForApplicationCode($pClientId, $pApplicationSubscriptionCode)
{
	// Get the application for the client
	$query_rsApp = sprintf("
SELECT 
	a.app_type_name,
	subscription_management_file,
	app_type_display,
	app_type_desc,
	app_type_tf_app,
	app_type_web_app
FROM
	".$this->sqlConvert_FROM(DATABASE_MASTER,'application_type a').", 	
	".$this->sqlConvert_FROM(DATABASE_MASTER,'client_application_subscription b')."   
WHERE 
    a.app_type_name = b.app_type_name
and b.app_type_name	= '%s'
and b.client_id  	= %s
and b.subscription_status	= 'ACTIVE'
and a.app_type_status		= 'ACTIVE'
order by  
   app_type_display", $pApplicationSubscriptionCode, $pClientId);

	//echo $query_rsApp; die;
   	$result = $this->sqlConvert_sql_query($query_rsApp, $this->connectionSystem);
	if(!$result || ($this->sqlConvert_sql_numrows($result) < 1))
	{
		// If we didnt find it, return to main application
		return 'cr_main.php';
	}

	// if we found it
	$rsApp = $this->sqlConvert_sql_query($query_rsApp, $this->connectionSystem) or die(mysql_error());
	if ($row_rsApp = $this->sqlConvert_sql_fetch_array($rsApp))
	{
		return $row_rsApp['app_type_web_app'];
	}
	
	// If we didnt find anything.
	return 'cr_main.php';
	
} // getWebLinkForApplicationCode

//  Media file updates
function UpdateMediaFileTable($mMediaFileId,$mClient_id,$mDuration)
{	
	$q = 
"SELECT
	media_file_name,
	app_type_name,
	media_file_type,
	ivr_computer_name,
	ivr_port,
	file_create_start_dtm_utc,
	file_duration,
	file_size,
	file_app_status,
	file_transfer_status,
	call_direction,
	called_number,
	calling_number
FROM  
	media_file
WHERE
	media_file_id = ".$mMediaFileId;
			//echo($q); die;
	
			$result = $this->sqlConvert_sql_query($q);// or die(mysql_error());                              
			$row = $this->sqlConvert_sql_fetch_array($result);
			
			if($row == true)
			{	//echo $row['app_type_name'];
								// Generate a file name for the application to use
				$mMediaFilename = mediaFileNameGenerate($mClient_id, 
										$row['app_type_name'],
										$row['media_file_type'],
										$row['ivr_port'],  
										$row['called_number'],
										$row['calling_number']);
				// Find the file size
				$mFileSize = -1;
				// echo $mMediaFilename; 
				if (file_exists($mMediaFilename)) //==ask about how this works since no file path is specified && filename will never exists due to timestamp differences. 
				{
					clearstatcache();
					$mFileSize = (filesize($mMediaFilename));
				}
				// Note: if we have -1... weve got a problem

				//temporary test
				$mFileSize = 200; 
				// If we do not have a passed duration, then lets determine it
				if ($mDuration == 0)
				{
					//$mDuration = now() - $row['file_create_start_dtm_utc']; now() function. 
					$mDuration = strtotime('now') - $row['file_create_start_dtm_utc'];//@@@ not working properly
				}
				
				//$mDuration = 12; 
				// Update the details
				$result =$this->mediaEntryUpdate($mClient_id, 
							$mMediaFileId,
							'SAVED',
							$mDuration,
							$mFileSize);				
				return true; 
			}
			else
			{			
				return false;
			}
}// UpdateMediaFileTable



//function added by daniel, used by the teleflow app to authorize caller. 
function ConfirmCaller($callerID, $clientID, $callerPassword, &$varUserID)
{
	if(!get_magic_quotes_gpc()) 
	{
	    $callerID = addslashes($callerID);
    }
	

	$selected = $this->selectClientDatabaseByMasterClientId($clientID); 
	
	if(!$selected)
	{  
		return -1;//indicate a db failure. 
	}
	
	/* Verify that user is in database */
	$q = 
"SELECT 
	u_id,
	user_phone_pass, 
	u_telephone,
	u_status
FROM 
	".TBL_USERS." 
WHERE 
	u_telephone = '".$callerID."'
AND u_status != 'DELETED'";

	
	//echo $callerID;
	$result = $this->sqlConvert_sql_query($q, $this->connectionClient) or die; 

	if(!$result || ($this->sqlConvert_sql_numrows($result) < 1))
	{			 
		return 1; //Indicates username failure
	}

	/* Retrieve password from result, strip slashes */
	$dbarray = $this->sqlConvert_sql_fetch_array($result); 
		
	$dbarray['user_phone_pass'] = stripslashes($dbarray['user_phone_pass']);
	$callerPassword = stripslashes($callerPassword);
	
	if(($callerPassword == $dbarray['user_phone_pass']) && ($dbarray['u_status'] == 'ACTIVE'))
	{
		// Return the user id (used for other look ups later)
		$varUserID = $dbarray['u_id'];
		return 0; //the caller is authorized. 
	}
	else if($dbarray['u_status'] == 'CREATE')
	{
		return 2;//Account has not yet been activated.  
	}
	else
	{
		return 3;//Wrong password. 
	}	
} // ConfirmCaller


/**
confirmUserPass - Checks whether or not the given
username is in the database, if so it checks if the
given password is the same password in the database
for that user. If the user doesn't exist or if the
passwords don't match up, it returns an error code
(1 or 2). On success it returns 0.
*/
function confirmUserPass($username, $password)
{
    /* Add slashes if necessary (for query) */
    if(!get_magic_quotes_gpc()) 
	{
	    $username = addslashes($username);
    }

	// Select the database for the user
	$selected = $this->selectClientDatabaseByUsername($username);
	if (!$selected)
	{
		// No database is associated with the user
		return 5;
	}

	/* Verify that user is in database */
	$q = 
"SELECT 
	u_password, 
	u_status 
FROM 
	".TBL_USERS." 
WHERE 
	user_login_name = '".$username."'
AND u_status != 'DELETED'";
	
	$result = $this->sqlConvert_sql_query($q, $this->connectionClient);
	if(!$result || ($this->sqlConvert_sql_numrows($result) < 1))
	{
		return 1; //Indicates username failure
	}

	/* Retrieve password from result, strip slashes */
	$dbarray = $this->sqlConvert_sql_fetch_array($result);
	$dbarray['u_password'] = stripslashes($dbarray['u_password']);
	$password = stripslashes($password);

	/* Validate that password is correct */
	if(($password == $dbarray['u_password']) && ($dbarray['u_status'] == 'ACTIVE'))
	{		
		// Set up the new session variables
		$this->setAuthenticatedSession();
		
		return 0; //Success! Username and password confirmed
	}
	else if ($dbarray['u_status'] <> 'ACTIVE')
	{
		if ($dbarray['u_status'] == 'CREATE')
		{
			return 3; // Not yet email confirmed
		}
		else
		{
			return 4; // Account not active
		}
	}
	else
	{
		return 2; //Indicates password failure
	}
} // confirmUserPass

// Set up the new session variables
function setAuthenticatedSession()
{
	$_SESSION['CLI_DB_CLIENT_ID'] 		= $this->db_client_id;
	$_SESSION['CLI_DB_DRIVER']			= $this->db_driver;
	$_SESSION['CLI_DB_TYPE']			= $this->db_type;
	$_SESSION['CLI_DB_HOST']			= $this->db_host;
	$_SESSION['CLI_DB_HOST_REMOTE']		= $this->db_host_remote;
	$_SESSION['CLI_DB_NAME']			= $this->db_name;
	$_SESSION['CLI_DB_USER']			= $this->db_user;
	$_SESSION['CLI_DB_PASSWORD']		= $this->db_password;
	$_SESSION['CLI_DB_CONNECT_OPTIONS']	= $this->db_connect_options;
	// A quick flag to see if we connected or not
	$_SESSION['CLI_DB_CONNECTED'] 		= true;
}

// Close the database session variables
function unsetAuthenticatedSession()
{
	unset($_SESSION['CLI_DB_CLIENT_ID']);  // Client Id Logged on as user
	unset($_SESSION['CLI_DB_DRIVER']);
	unset($_SESSION['CLI_DB_TYPE']);
	unset($_SESSION['CLI_DB_HOST']);
	unset($_SESSION['CLI_DB_HOST_REMOTE']);
	unset($_SESSION['CLI_DB_NAME']);
	unset($_SESSION['CLI_DB_USER']);
	unset($_SESSION['CLI_DB_PASSWORD']);
	unset($_SESSION['CLI_DB_CONNECT_OPTIONS']);
	// A quick flag to see if we connected or not
	unset($_SESSION['CLI_DB_CONNECTED']);
}


/**
confirmUserID - Checks whether or not the given
username is in the database, if so it checks if the
given userid is the same userid in the database
for that user. If the user doesn't exist or if the
userids don't match up, it returns an error code
(1 or 2). On success it returns 0. This is used for
ensuring the user is valid for logons etc.
*/
function confirmUserID($username, $userid)
{
    /* Add slashes if necessary (for query) */
    if(!get_magic_quotes_gpc()) 
	{
	    $username = addslashes($username);
    }

    /* Verify that user is in database */
    $q = "
SELECT 
	u_userid 
FROM 
	".TBL_USERS." 
WHERE 
	user_login_name = '".$username."' 
AND u_status = 'ACTIVE'";
	//echo ($q);	
	
    $result = $this->sqlConvert_sql_query($q, $this->connectionClient);

    if(!$result || ($this->sqlConvert_sql_numrows($result) < 1))
    {
       return 1; //Indicates username failure
    }

    /* Retrieve userid from result, strip slashes */
    $dbarray = $this->sqlConvert_sql_fetch_array($result);
    $dbarray['u_userid'] = stripslashes($dbarray['u_userid']);
    $userid = stripslashes($userid);

    /* Validate that userid is correct */
    if($userid == $dbarray['u_userid'])
    {
       return 0; //Success! Username and userid confirmed
    }
    else
	{
       return 2; //Indicates userid invalid
    }
} // confirmUserID


/**
getUserID - Returns userId for a user name.
Used for admin override
*/
function getUserID($username)
{
    // Find user
    $q = "
SELECT 
	u_userid 
FROM 
	".TBL_USERS." 
WHERE 
	user_login_name = '".$username."' 
AND u_status <> 'DELETED'";
	//echo ($q);	
	
    $result = $this->sqlConvert_sql_query($q, $this->connectionClient);
    if(!$result || ($this->sqlConvert_sql_numrows($result) < 1))
    {
       return false;
    }

    /* Retrieve userid from result, strip slashes */
    $dbarray = $this->sqlConvert_sql_fetch_array($result);
    return $dbarray['u_userid'];
} // getUserID

   
/**
* getCompanyInfo - Returns the result array from a mysql
* query asking for all information stored regarding
* the given Company id. If query fails, NULL is returned.
*/
function getCompanyInfo($companycode)
{
    $q = "
SELECT 
	* 
FROM 
	".TBL_COMPANY." 
WHERE 
	company_code = '".$companycode."'";
    $result = $this->sqlConvert_sql_query($q, $this->connectionClient);
	
    /* Error occurred, return given name by default */
    if(!$result || ($this->sqlConvert_sql_numrows($result) < 1))
    {
       return NULL;
    }
    /* Return result array */
    $dbarray = $this->sqlConvert_sql_fetch_array($result);
    return $dbarray;
}


function getCompanyInfoByID($companyid)
{
	$q = "
SELECT 
	* 
FROM 
	".TBL_COMPANY." 
WHERE company_id = '".$companyid."'";
	$result = $this->sqlConvert_sql_query($q, $this->connectionClient);
	
	/* Error occurred, return given name by default */
	if(!$result || ($this->sqlConvert_sql_numrows($result) < 1))
	{
		return NULL;
	}
	/* Return result array */
	$dbarray = $this->sqlConvert_sql_fetch_array($result);
	return $dbarray;
}

/**
* getCompanyUserTotal - Returns count of users for a companycode
*/
function getCompanyUserTotal($companycode)
{
	$q = "
SELECT 
	count(*) total 
FROM 
	".TBL_USERS." 
WHERE 
	u_status != 'DELETED'
AND	company_id = (SELECT 
						company_id 
					FROM 
						company 
					WHERE company_code = '".$companycode."') and u_status='ACTIVE'";
	// Debug echo "<br>".$q."<br>";
	
	$result = $this->sqlConvert_sql_query($q, $this->connectionClient);
	/* Error occurred, return given name by default */
	if(!$result || ($this->sqlConvert_sql_numrows($result) < 1))
	{
		return NULL;
	}
	/* Return result array */
	$dbarray = $this->sqlConvert_sql_fetch_array($result);
	return $dbarray;
} // getCompanyUserTotal


/**
* companycodeTaken - Returns true if the companycode has
* been taken already, false otherwise.
*/
function companycodeTaken($companycode)
{
	if(!get_magic_quotes_gpc())
	{
		$companycode = addslashes($companycode);
	}
	$q = "
SELECT 
	company_name 
FROM 
	".TBL_COMPANY." 
WHERE 
	company_code = '".$companycode."'";
	$result = $this->sqlConvert_sql_query($q, $this->connectionClient);
	return ($this->sqlConvert_sql_numrows($result) > 0);
} // companycodeTaken


/**
* addCompany - Inserts the given (username, password, email)
* info into the database. Appropriate user level is set.
* Returns true on success, false otherwise.
*/
function addCompany($companycode, $companyname, $companyownerid)
{
	$time = time();
	$q = 
"INSERT INTO ".TBL_COMPANY." 
(	company_code, 
	company_name, 
	company_owner_id
) 
VALUES 
(
	'$companycode', 
	'$companyname',
	$companyownerid
)";
	// Debug
	//echo '<br><br>'.$q.'<br><br>';
	return $this->sqlConvert_sql_query($q, $this->connectionClient);
}  // addCompany


// Log Call Event (anything the programmer wants to log for_
// eg:  $database->logCallEvent(...);
function logCallEvent(
					$evt_date,
					$evt_time,
					$evt_port,
					$evt_code,
					$evt_start,
					$call_duration,
					$logging_app,
					$site_code,
					$region_code,
					$evt_type,
					$evt_data1,
					$evt_data2,
					$evt_data3
				)
{
	// Defaults
	$log_version	= '400';

	/*
	// See if the table exists
	$q = "SHOW TABLES LIKE 'event_log'";
	$result = mysql_query($q);
	$row=mysql_fetch_array($result);
	if ($row == FALSE)
	{
		// Create the table
		$qCreate = TableCreate_Event_Log();
		$result = mysql_query($qCreate);
		// What happened?
		if ($result == FALSE) 
		{
			$responsecode = "Failed to Create Table";
			$elementHolder->setAttribute("action", $request_Command); 
			$elementHolder->setAttribute("result", $error); 
			$elementHolder->setAttribute("type",   $db); 
			// Add verbose TEST 
			$elementHolder->appendChild($domobject->createTextNode($responsecode));
		} 
	}
	*/
	
	// Log the entry
	$qInsert = "			
INSERT INTO event_log
	(
	evt_date,
	evt_time,
	evt_port,
	log_format_version,
	evt_code,
	evt_start,
	call_duration,
	logging_app,
	site_code,
	region_code,
	evt_type,
	evt_data1,
	evt_data2,
	evt_data3,
	evt_timestamp
	)
VALUES
	('".$evt_date."',
	 '".$evt_time."',
	  ".$evt_port.",
	 '".$log_version."',
	 '".$evt_code."',
	 '".$evt_start."',
	  ".$call_duration .",
	 '".$logging_app."',
	 '".$site_code."',
	 '".$region_code."',
	 '".$evt_type."',
	 '".$evt_data1."',
	 '".$evt_data2."',
	 '".$evt_data3."',
	 ".time()."	 
	)";

	$this->sqlConvert_sql_query($qInsert, $this->connectionClient); 
}	// logCallEvent
			

// Log access (users, automatic processes etc)
// eg:  $database->logAccess($_SERVER['REMOTE_ADDR'],'Login',$_SESSION['useruid'], $_SESSION['userclientid']);
function logAccess($pRemoteIP, $pSessionId, $pAccessType, $pUserUID, $pClientId, $pOptCommand, $pOptDetail)
{
	$mPage = $_SERVER['PHP_SELF'];
	
	// Insert 
	$qInsert = 
"INSERT INTO 
	".$this->sqlConvert_FROM(DATABASE_MASTER,'user_access_log')."
( 	ip_from,
	session_id,
	access_type,
	user_id,
	client_id,
	page_access,
	optional_command,
	optional_detail
)
VALUES 
(	'$pRemoteIP', 
	'$pSessionId', 
	'$pAccessType', 
	$pUserUID, 
	$pClientId, 
	'$mPage', 
	'$pOptCommand', 
	'$pOptDetail')";

	// Debug
	//echo '<br><br>'.$qInsert.'<br><br>'; die;
	return $this->sqlConvert_sql_query($qInsert,  $this->connectionSystem);
} // logAccess


// See if the user is allowed to access the system (top level)
// - ie: The are knocking on the front door
// return true to proceed to access test 
function securityAccessCheck($pRemoteIP, $pSessionId, &$varLockoutReason)
{
	global $globalSQLSrv_conn;
	// See LocalSettings_Appearance for these cLoginLock* variables
	
	// Make sure the system has a relevance
	if ((LOGINLOCKTESTMETHOD > 0) &&	// 0 = no security test 
		(LOGINLOCKATTEMPTS > 0) &&	// Attempts to test for
		(LOGINLOCKINRANGEMINS > 0))	// Within so many minutes
	{
		// IP Address Test level
		if (LOGINLOCKTESTMETHOD == 1)
		{
			$mTest = 'ip_from="'.$pRemoteIP.'"';
		}
		else
		{
		// Session ID level
		// $cLoginLockTestMethod == 2
			$mTest = 'session_id="'.$pSessionId.'"';		
		}

		// Find the greater of the times, and roll with that
		$mMinutesBackward = LOGINLOCKINRANGEMINS;
		if (LOGINLOCKOUTMINS > $mMinutesBackward)
		{
			// This should be the selection
			$mMinutesBackward = LOGINLOCKOUTMINS;
		}		
		
		// Get list of recent codes
		$q = 
"SELECT  
	user_access_log_id,
	access_type,
	ip_from,
	create_dtm_utc,
	".$this->sqlConvert_DateFormat_Time('create_dtm_host')." log_time
FROM
	".$this->sqlConvert_FROM(DATABASE_MASTER,'user_access_log')."
WHERE
	".$mTest."
AND create_dtm_utc > ".$this->sqlConvert_DateAdd_Seconds($this->sqlConvert_UTC_timestamp(),$mMinutesBackward*60*-1)."
ORDER BY
	user_access_log_id DESC";
		//echo ($q); die;
		
		$mReturnResult	= true;
		$mCount 		= 0;
		// Check against the master
		$result = $this->sqlConvert_sql_query($q, $this->connectionSystem);
		while ($dbarray = $this->sqlConvert_sql_fetch_array($result))
		{
			// Get the long values
			$mAccessType = ($dbarray['access_type']);
			if ($mAccessType == 'LoginFail')
			{
				// Have we exceeded the number of attempts allowed for the time allotment
				$mCount++;
				if ($mCount >= LOGINLOCKATTEMPTS)
				{
					// Failure and lockout
					$mReturnResult	= false;
					
					// We are locking out for X minutes, so mark that with an event
					$this->logAccess($pRemoteIP, $pSessionId, 'AccessLockout', 0, 0, $mTest, '');

					// Say what just happened
					$varLockoutReason = 'Lockout for '.LOGINLOCKOUTMINS.' minutes';
					break;
				}
			}
			elseif ($mAccessType == 'AccessLockout')
			{
				// Failure and lockout
				$mReturnResult	= false;
				$varLockoutReason = 'A lockout for '.LOGINLOCKOUTMINS.' minutes occurred at '.$dbarray['log_time'];
				break;			
			}
			else
			{
				// Anything but a fail, means we are fine to grant access
				// - eg: UnBlock will allow it
				break;
			}
		}

		// Return with the reason
		return $mReturnResult;
		
	}
	else
	{
		// No security check, return true
		return true;
	}
} // securityAccessCheck



// Media entry for recordings
// eg:  $database->mediaEntryInsert(
// NOTE: This entry must be created by the TeleFlow Server that creates the file.  No remote creates, because that simply defeats the ability to link file names to media entries
// FIND IN: xml_media_publish_body.php
function mediaEntryInsert($pClientId, 
						$pMediaOwnerUID,
						$pMediaFileAppStatus,
						// Sub folder from client media (could be a date for example)
						$pMediaFolder, 
						// Standard media information
						$pMediaFileName, 
						$pAppTypeName, 
						$pMediaFileType, 
						$pIVRComputerName,
						$pIVRPort,
						$pCallDirection,
						$pCalledNumber,
						$pCallingNumber,
						// Optional duration
						$pFileDuration,
						$pFileSize,	// Derived by calling program
						// Standard CallCapture
						$pRecordingStartTrigger, 
						$pRecordingStopTrigger,
						$pRecordingProtocol,
						$pCCAgentId,
						$pCCExtension,
						// File create information from other create methods 
						$pFileCreateStartDtmUtc, 
						$pFileCreateEndDtmUtc,
						$pFileCreateStartDtmClient,
						$pFileCreateEndDtmClient 						
						)
{
	global $cGlobalDebug;
	
	// Get client id offset
	$mClientUTCOffset 			= $this->getClientUTCOffset($pClientId);
	
	// First, we need to be certain that there are no previous entries that are active.  
	// -- Find the file name (there should be an index on the file name), and set both status to REPLACED
	$q = "
SELECT 
	media_file_id,
	file_duration,
	file_size	 	
FROM 
	media_file 
WHERE 
	media_file_name = '".$pMediaFileName."'";	
	//echo $q; die;
	
	$mReplace_MediaFileId = -1;
	// See if we have a previous entry
	$result = $this->sqlConvert_sql_query($q, $this->connectionClient);
	while ($row = $this->sqlConvert_sql_fetch_array($result))
	{
		// We have a previous entry, so we need to set it as replaced at the end
		$mReplace_MediaFileId 			= $row['media_file_id'];
		$mReplace_MediaFileDuration		= $row['file_duration'];
		$mReplace_MediaFileSize			= $row['file_size'];
		
		// Update 
		$this->mediaEntryUpdate($pClientId,
								$mReplace_MediaFileId,
								FA_REPLACED,
								FT_REPLACED,
								$mReplace_MediaFileDuration,
								$mReplace_MediaFileSize);
		
	} // While row

	// Derive any values that are not pre-populated
	$mMediaOwnerUID = $pMediaOwnerUID;
	if ($mMediaOwnerUID == '')
	{
		$mMediaOwnerUID = 'null';
	}
	$mFileCreateStartDtmUtc		= $pFileCreateStartDtmUtc;
	$mFileCreateEndDtmUtc		= $pFileCreateEndDtmUtc;
	$mFileCreateStartDtmClient	= $pFileCreateStartDtmClient;
	$mFileCreateEndDtmClient 	= $pFileCreateEndDtmClient;	
	if ($mFileCreateStartDtmUtc == '')
	{
		$mFileCreateStartDtmUtc		= $this->sqlConvert_DateFormat_FullTimeStamp_Now();
	}
	if ($mFileCreateStartDtmClient == '')
	{
		$mFileCreateStartDtmClient	= $this->sqlConvert_DateAdd_Seconds($this->sqlConvert_UTC_timestamp(),$mClientUTCOffset*60*60);
	}
	
	// Make sure the values have something
	if ($mFileCreateStartDtmUtc		== '') {$mFileCreateStartDtmUtc	  	= 'NULL';}
	if ($mFileCreateEndDtmUtc		== '') {$mFileCreateEndDtmUtc		= 'NULL';}
	if ($mFileCreateStartDtmClient	== '') {$mFileCreateStartDtmClient	= 'NULL';}
	if ($mFileCreateEndDtmClient 	== '') {$mFileCreateEndDtmClient	= 'NULL';}
	
	//echo 'htere'; die;
	
	// Insert 
	$qInsert = SQLCommandClean("
INSERT INTO media_file 
( 				
	media_file_directory,
	media_owner_u_id,	
	media_file_name, 				
	-- file_reference_id,
	app_type_name, 					
	media_file_type, 				
	ivr_computer_name, 				
	ivr_port, 			
	file_create_start_dtm_utc, 		
	file_create_end_dtm_utc, 		
	file_create_start_dtm_client, 	
	file_create_end_dtm_client, 		
	file_duration, 					
	file_size, 						
	file_app_status, 				
	-- app_status_update_dtm_utc, 		
	-- app_status_update_dtm_client,	
	file_transfer_status, 			
	-- xfer_status_update_dtm_utc, 		
	-- xfer_status_update_dtm_client, 	
	call_direction, 					
	called_number, 					
	calling_number,
	recording_start_trigger,
    recording_stop_trigger,
    recording_protocol,
	cc_agent_id,
	cc_extension
	-- create_dtm_utc, 					
	-- update_dtm_utc, 					
	-- create_dtm_client,				
	-- update_dtm_client
)
VALUES
(
	'$pMediaFolder',
	$mMediaOwnerUID,
	'$pMediaFileName',
	--,
	'$pAppTypeName',
	'$pMediaFileType',
	'$pIVRComputerName',
	$pIVRPort,
	$mFileCreateStartDtmUtc,
	$mFileCreateEndDtmUtc,
	$mFileCreateStartDtmClient,
	$mFileCreateEndDtmClient,
	$pFileDuration,
	$pFileSize,
	'$pMediaFileAppStatus',
	--,
	--,
	'NEW',
	--,
	--,
	'$pCallDirection',
	'$pCalledNumber',
	'$pCallingNumber',
	'$pRecordingStartTrigger', 
	'$pRecordingStopTrigger',
	'$pRecordingProtocol',
	$pCCAgentId,
	'$pCCExtension'	
	--,
	--,
	--,
	--
)");
	//echo ($qInsert); die;

	$result = $this->sqlConvert_sql_query($qInsert, $this->connectionClient);  
	if ($result)
	{
		$mMediaFileId = $this->sqlConvert_sql_insert_id($this->connectionClient);  //media_file_id, 
	}
	else
	{
		// Insert didnt happen.  
		$mMediaFileId = -1;
		
		// Log database entries in debug
		if ($cGlobalDebug)
		{
			// For debugging, lets catch the SQL statements being run.
			//LogTextToFileXMLTimeStamp('../'.SYSTEM_LOGFILES."Database-".Date('Y-m-d').".log", $pQuery);			
			LogTextToFileXMLTimeStamp(getRelativeSystemLogFilesPath()."Database-".Date('Y-m-d').".log", 
										'-- '.SYS_DATABASE_SYSTEM.chr(13).chr(10).
										$qInsert);		
		}		
	}

	// Return the insert id
	return $mMediaFileId;
} // mediaEntryInsert



// Media entry for recordings
// eg:  $database->mediaEntryUpdate(
// NOTE: This entry must be created by the TeleFlow Server that creates the file.  No remote creates, because that simply defeats the ability to link file names to media entries
function mediaEntryUpdate($pClientId,
						$pMediaFileId, 
						$pFileAppStatus, 
						$pFileTransferStatus,
						$pMediaFileDuration, 
						$pMediaFileSize
						)
{
	// Get client id offset
	$mClientUTCOffset = $this->getClientUTCOffset($pClientId);
	
	// Status to update
	$mAppStatus = '';
	$mTransferStatus = '';
	$mRecordingCompleted = '';
	
	// Only include if we have set a code for the fields.
	if ($pFileAppStatus <> '')
	{
		$mAppStatus = 'file_app_status	= "'.$pFileAppStatus.'",'.
"app_status_update_dtm_utc 	 = ".$this->sqlConvert_DateFormat_FullTimeStamp_Now().",
app_status_update_dtm_client = ".$this->sqlConvert_DateAdd_Seconds($this->sqlConvert_UTC_timestamp(),$mClientUTCOffset*60*60).',';		
	}
	
	// Update the datetimes for a code update
	if ($pFileTransferStatus <> '')
	{
		$mTransferStatus =  'file_transfer_status	= "'.$pFileTransferStatus.'",'; 
		// NOTE: We removed these fields because they were redundant
//"xfer_status_update_dtm_utc   = ".$this->sqlConvert_DateFormat_FullTimeStamp_Now().",
//xfer_status_update_dtm_client = ".$this->sqlConvert_DateAdd_Seconds($this->sqlConvert_UTC_timestamp(),$mClientUTCOffset*60*60).',';		
	}
	
	// Only update if we have a completed setting
	if ($pFileTransferStatus == 'COMPLETED')
	{
		$mRecordingCompleted = 'file_app_status	= "'.$pFileAppStatus.'",'.
"file_create_end_dtm_utc	= ".$this->sqlConvert_DateFormat_FullTimeStamp_Now().",		
file_create_end_dtm_client  = ".$this->sqlConvert_DateAdd_Seconds($this->sqlConvert_UTC_timestamp(),$mClientUTCOffset*60*60).',';		
	}
	
	
	// Update 
	$qUpdate = SQLCommandClean("
UPDATE 
	media_file 
SET
".$mRecordingCompleted."  
".$mAppStatus." 
".$mTransferStatus."
  file_duration		= ".$pMediaFileDuration.",
  file_size	 		= ".$pMediaFileSize."
WHERE
	media_file_id = ".$pMediaFileId); 
	//echo ($qUpdate); die;
	
	$result = $this->sqlConvert_sql_query($qUpdate, $this->connectionClient);  
	
	// If new, then indidate new data for a media transfer
	if (($result) && ($pFileAppStatus == 'NEW'))
	{
		$this->systemDatabaseIndicateNewData($pClientId, 'MEDIA_XFER', 0);
	}
	
	return ($result);
} // mediaEntryUpdate

   

// Get Client Info  - Get information about the client
/* Put the following around this call
// Note: This sets $_SESSION['userclientname']
$req_client_info  = $database->getClientInfo($_SESSION['userclientid']);
echo $req_client_info['client_type'];
*/
function getClientInfo($clientId)
{
	// Get details of the client from the client id
	$q = "
SELECT 
	* 
FROM 
	".TBL_CLIENT." 
WHERE 
	client_id = ".$clientId;
	
	//echo $q; die;
	
	$result = $this->sqlConvert_sql_query($q, $this->connectionClient);
	/* Error occurred, return given name by default */
	if(!$result || ($this->sqlConvert_sql_numrows($result) < 1))
	{
		return NULL;
	}
	/* Return result array */
	$dbarray = $this->sqlConvert_sql_fetch_array($result);
	
	//  If NOT set already, tke the opportunity to get the client name
	if (!isset($_SESSION['userclientname']))
	{
		$_SESSION['userclientname'] = $dbarray['client_name'];
	}
	
	return $dbarray;
} // getClientInfo

// Simple function to get the client name
function getClientName($clientId)
{
	if ($clientId > 0)
	{
		$req_client_info = $this->getClientInfo($clientId);
		return $req_client_info['client_name'];
	}
	else
	{
		return 'N/A';
	}
	
} // getClientName


// Get the name of the application from the web entries
function getApplicationNameForWebpage($pWebPage, &$varTypeDisplay, &$varTypeDescription )
{
	// select the application name from the application_type
	$q = "
SELECT 
	* 
FROM 
	".$this->sqlConvert_FROM(DATABASE_MASTER,'application_type ')."
WHERE 
	app_type_web_app LIKE '%".$pWebPage."'";	
	//echo $q; die;
	
	$result = $this->sqlConvert_sql_query($q, $this->connectionSystem);
	/* Error occurred, return given name by default */
	if(!$result || ($this->sqlConvert_sql_numrows($result) < 1))
	{
		return false;
	}
	/* Return result array */
	$dbarray = $this->sqlConvert_sql_fetch_array($result);

	// Populate the values
	$varTypeDisplay		= $dbarray['app_type_display'];
	$varTypeDescription	= $dbarray['app_type_desc'];
	
	return true;
}  // getApplicationNameForWebpage


// Get the UTC for the client
function getClientUTCOffset($clientid)
{
	$dbFields = $this->getClientInfo($clientid);
	return $dbFields['client_utc_offset'];
} //getClientUTCOffset


/**
* getClientUserTotal - Returns count of users for a client
*/
function getClientUserTotal($clientid)
{
	$q = "
SELECT 
	count(*) total 
FROM 
	".TBL_USERS." 
WHERE 
	client_id = ".$clientid." 
AND u_status = 'ACTIVE'";
	//echo "<br>".$q."<br>";
	
	$result = $this->sqlConvert_sql_query($q, $this->connectionClient);
	/* Error occurred, return given name by default */
	if(!$result || ($this->sqlConvert_sql_numrows($result) < 1))
	{
		return NULL;
	}
	/* Return result array */
	$dbarray = $this->sqlConvert_sql_fetch_array($result);
	return $dbarray['total'];
}

   
/**
* usernameTaken - Returns true if the username has
* been taken by another user, false otherwise.
*/
function usernameTaken($username)
{
	if(!get_magic_quotes_gpc())
	{
		$username = addslashes($username);
	}
	$q = "
SELECT 
	user_login_name 
FROM 
	".TBL_USERS." 
WHERE 
	user_login_name = '".$username."' 
AND u_status != 'DELETED'";
	
	$result = $this->sqlConvert_sql_query($q, $this->connectionClient);
	return ($this->sqlConvert_sql_numrows($result) > 0);
}
   
/**
* usernameExist - Returns true if the username has
* been created in the system, false otherwise.
*/
function usernameExists($username)
{
	if(!get_magic_quotes_gpc())
	{
		$username = addslashes($username);
	}
	$q = "
SELECT 
	user_login_name 
FROM 
	".TBL_USERS." 
WHERE 
	user_login_name = '".$username."' 
AND u_status != 'DELETED'";
	
	$result = $this->sqlConvert_sql_query($q, $this->connectionClient);
	return ($this->sqlConvert_sql_numrows($result) > 0);
}  // usernameExists
   
   
/**
* emailInUse - Returns true if the email has
* been used Before, false otherwise.
*/
function emailInUse($email)
{
	if(!get_magic_quotes_gpc())
	{
		$email = addslashes($email);
	}
	//$q = "SELECT u_email FROM ".TBL_USERS." WHERE user_login_name = '$email' and u_status='ACTIVE'";  // Client way
	$q = "
SELECT 
	user_login_name 
FROM 
	".$this->sqlConvert_FROM(DATABASE_MASTER,'user_master')."
WHERE 
	user_login_name = '".$email."'";  
	// echo ($q); die;
	
	$result = $this->sqlConvert_sql_query($q, $this->connectionSystem) or die(mysql_error());
	return ($this->sqlConvert_sql_numrows($result) > 0);
} // emailInUse
   

/**
* local - Returns true if the extlocal has
* been used Before, false otherwise.
*/
function extlocalInUse($extlocal)
{
	//echo ($q); die;
	$q = "
SELECT
	user_login_name						
FROM
	".TBL_USERS." a,
	".TBL_USER_EXTENSION." b
WHERE
	u_status != 'DELETED'
and a.u_id = b.u_id
and	virtual_ext='".$extlocal."'";
	
	$result = $this->sqlConvert_sql_query($q, $this->connectionClient);
	return ($this->sqlConvert_sql_numrows($result) > 0);
}  // extlocalInUse
   

/**
* usernameBanned - Returns true if the username has
* been banned by the administrator.
*/
function usernameBanned($username)
{
	if(!get_magic_quotes_gpc())
	{
		$username = addslashes($username);
	}
	$q = "
SELECT 
	user_login_name 
FROM 
	".TBL_BANNED_USERS." 
WHERE 
	user_login_name = '$username'";
	$result = $this->sqlConvert_sql_query($q, $this->connectionClient);
	return ($this->sqlConvert_sql_numrows($result) > 0);
}
   

/**
addNewUser - Inserts the given (username, password, email)
info into the database. Appropriate user level is set.
Returns true on success, false otherwise.
*/
function addNewUser($username, $client, $password, $namelast, $namefirst, $passnum, $email)
{
	$ulevel = USER_LEVEL;
	$uFirst = false;
	$time 	= time();

	// Step 1: Insert the user id to the system table
	// -- The user id must be checked before this point
	// @@@ Should do a double check in case two users are grabbing the same email address
	
	// Build the insert for System Master Level
	$q = 
"INSERT INTO 
	".$this->sqlConvert_FROM(DATABASE_MASTER,'user_master')."
(
	user_login_name, 
	client_id
) 
VALUES 
(
	'$username', 
	'$client'
)";
	//echo '<br><br>'.$q.'<br><br>'; die;	
	$this->sqlConvert_sql_query($q, $this->connectionSystem);
	$userMasterId = $this->sqlConvert_sql_insert_id($this->connectionSystem);
	
	// Select the database
	$selected = $this->selectClientDatabaseByMasterClientId($client); 
	
	// For a client that might have been just created for the client, make admin
	// -- If this is the first user for the client, make it an admin
	$req_total = $this->getClientUserTotal($client);
	if ($req_total == 0)
	{
		$ulevel = ADMIN_LEVEL;
		$uFirst = true;
	}

	// Make sure we have a valid phone number
	$mPhonePassword = $passnum;
	if ($passnum == '')
	{
		$mPhonePassword = 0;
	}
	
	// Create the touch tone equivalent 
	$tt_last  = TTLetterDigits($namelast);
	$tt_first = TTLetterDigits($namefirst);
	
	// Build the insert for Client
	$q = 
"INSERT INTO ".TBL_USERS." 
(
	client_id, 
	user_login_name, 
	u_last, 
	u_first, 
	u_full_name, 
	tt_last, 
	tt_first, 
	u_password, 
	u_userid, 
	u_permissions_level, 
	u_email, 
	u_timestamp, 
	u_status,
	user_phone_pass
) 
VALUES 
(
	'$client', 
	'$username', 
	'$namelast', 
	'$namefirst', 
	'".$namelast.", ".$namefirst."', 
	'$tt_last', 
	'$tt_first', 
	'$password', 
	'0', 
	$ulevel, 
	'$email', 
	$time, 
	'CREATE',
	$mPhonePassword
)";	
	//echo '<br><br>'.$q.'<br><br>'; die;

	$result = $this->sqlConvert_sql_query($q, $this->connectionClient);
	$user_id = $this->sqlConvert_sql_insert_id($this->connectionClient);
	
	// Do we need to update the company with the user if information
	if ($uFirst)
	{
		$dbFields = $this->getClientInfo($client);
		if (is_null($dbFields['primary_contact_u_id']))
		{
			// Add the user as the primary contact
			$this->updateClientField($client, 'primary_contact_u_id', $user_id);
		}	
	}
		
	// return the id of the user master record
	return $userMasterId;
} // addNewUser
   
   
/**
* addNewUserExtension - Inserts an extension with details for a user
* Returns true on success, false otherwise.
*/
function addNewUserExtension($u_id, $physical_ext, $virtual_ext, $phone_number, $transfer_type, 
							$primary_line_yn, $available_yn, $virt_ext_followme_yn, 
							$user_ext_followme_yn, $followme_ringcount, $ringcount, 
							$avail_start_time, $avail_end_time, $avail_days_of_week)
{
	// Make sure the number is clean
	$mTelephoneNumber =  stripNonNumeric($phone_number);
	
	//$req_company_info = $this->getCompanyInfo($company);
	$q = "INSERT INTO ".TBL_USER_EXTENSION."
(
  u_id,
  physical_ext,
  virtual_ext,
  phone_number,  
  transfer_type,
  primary_line_yn,  
  available_yn,  
  virt_ext_followme_yn,  
  user_ext_followme_yn,  
  followme_ringcount,  
  ringcount,  
  avail_start_time,  
  avail_end_time,  
  avail_days_of_week
)
values
( 
  $u_id,
  '$physical_ext',
  '$virtual_ext',
  '$mTelephoneNumber',  
  '$transfer_type',
  '$primary_line_yn',  
  '$available_yn',  
  '$virt_ext_followme_yn',  
  '$user_ext_followme_yn',  
  $followme_ringcount,  
  $ringcount,  
  '$avail_start_time',  
  '$avail_end_time',  
  '$avail_days_of_week'
)";
//--  user_ext_id,

	// Debug
	//echo '<br><br>'.$q.'<br><br>';
		
	return $this->sqlConvert_sql_query($q, $this->connectionClient);
}

/* deleteUserFromMaster - Delete the link entry in the master table 
-- NOTE: Because the master table is only a quick link to the client tables, no data is lost by deleting this entry,
as all information can be found in the log tables for each user.
*/
function deleteUserFromMaster(
				$login_user
				)
{
	// Build the insert for System Master Level
	$q = 
"DELETE FROM 
	".$this->sqlConvert_FROM(DATABASE_MASTER,'user_master')."
WHERE
	user_login_name = '".$login_user."'";
	//echo '<br><br>'.$q.'<br><br>'; die;	
	$this->sqlConvert_sql_query($q, $this->connectionSystem);
} // deleteUserFromMaster


/* userCreateOrReactivateToClient - Create or set status of a User (admin assumed) to 
a client database
*/
function adminUserCreateOrReactivateToClient(
				$pClient_Id,			// Client to insert new user, or reactivate with
				$pUserDetailsArray		// Array of user info from previous read
				)
{
	$mReturn = false;
	
	// Get the user name
	$mAdminUserName = $pUserDetailsArray['user_login_name'];
	
	// Must be an Admin
	if ($pUserDetailsArray['u_permissions_level'] == SUPER_ADMIN_LEVEL)
	{
		// select the Client environment
		if ($this->selectClientDatabaseByMasterClientId($pClient_Id))
		{
		
			// See if the user exists
			if ($this->usernameExists($mAdminUserName))
			{
				// User was activated, so we should be good.
				$mReturn = true;
			}
			else
			{
				// if user does not exist, then create it
				if ($this->addNewUser($mAdminUserName, 
									$pClient_Id,
									'', 		// Leave password blank
									$pUserDetailsArray['u_last'],
									$pUserDetailsArray['u_first'],
									'',			// Leave passname blank
									$pUserDetailsArray['u_email']))
				{
					// User was activated, so we should be good.
					$mReturn = true;
				}
			}
			
			if ($mReturn)
			{
				// Set the Admin user level - SUPER_ADMIN_LEVEL - this will enable the screen to be available to jump to other clients
				$q = "
UPDATE 
	".TBL_USERS." 
SET 
	u_permissions_level = ".SUPER_ADMIN_LEVEL.",  
	u_status = 'ACTIVE'
WHERE 
	user_login_name = '".$mAdminUserName."' 
AND u_status != 'DELETED'
AND client_id = ".$pClient_Id;

				//echo $q; die;		
				if (!$this->sqlConvert_sql_query($q, $this->connectionClient))
				{
					return false;
				}				

				// Will commit the current database selected as the correct session
				$this->setAuthenticatedSession();
	
				// Set these details to override user 
				// -- adminmode will be set until logout
				if (!isset($_SESSION['adminmode']))
				{
					// If we are not in admin mode, and leaving our home client then save the original client id
					if ($_SESSION['userclientid'] <> $pClient_Id)
					{
						$_SESSION['adminmode'] = $_SESSION['userclientid'];	
					}
				}
				else
				{
					// See if we are returning to the original client, after which we will turn things off
					if ($_SESSION['adminmode'] == $pClient_Id)
					{
						// We are returning, so clear the admin mode
						unset($_SESSION['adminmode']);
					}
				}
				
				// This will get set again later in getClientInfo
				unset($_SESSION['userclientname']);
				$_SESSION['username']	= $mAdminUserName;
				$_SESSION['userid']		= $this->getUserID($mAdminUserName);

				// Note: At the returned side, we need to force the session using
				// $session->checkLogin();
			}			
		}
	}

	return $mReturn;
} // adminUserCreateOrReactivateToClient




/**
* addClientDatabase - Creates a database for a client
* Returns true on success, false otherwise.
*/
function addClientDatabase(	
				$login_user,
				$company_name,
				$default_db_driver,					
				$default_db_type,					
				$default_db_host,	
				$default_db_host_remote,	
				$default_db_namestub,
				$default_db_user,				
				$default_db_password,		
				$default_db_connect_options,
				&$pNewClientId)
{
	// The following needs to occur:
	// * Get the encryption key
	// * Create new client entry in System client database table
	// * Create Client Database space

	// Encrypt key
	$mLoc = '../'.SYS_CRYPTO_LOCATE_KEYFILE;  // @@@ Should use some kind of session home variable. eg: $incPath
	if (!file_exists($mLoc))
	{
		$mLoc = '../../'.SYS_CRYPTO_LOCATE_KEYFILE;
	}
	
	// Get the crypt key to decrypt the stored passwor
	$cKey=$this->getCryptKey($mLoc); // Define found in database file
	if ($cKey) 
	{
		// Encrypt 
		$encrypted_user 	= ($this->padEncryptToHex($cKey, $default_db_user));
		$encrypted_password = ($this->padEncryptToHex($cKey, $default_db_password));
	}
	else
	{
		// Password was not encrypted, or someone has removed the cryptkey
		$encrypted_user 	= $default_db_user;
		$encrypted_password = $default_db_password;
		echo(providePhrase('cr', 523)); // ERROR: Encyption system missing key file
		die;		
	}

	// Start off as INACTIVE, because other tables will need to create to support the client.  This may take second to minutes depending on the application
	// Note: Needed because of cycling applications which check ACTIVE databases.
	$insertSQL = (
"INSERT INTO 
	".$this->sqlConvert_FROM(DATABASE_MASTER,'client')."	
(
  client_name,
  client_status,
  client_db_driver,
  client_db_type,
  client_db_host,
  client_db_host_remote,
  client_db_user,
  client_db_password,
  client_db_connect_options
)
VALUES
(
	'".$company_name."',
	'INACTIVE',
	'".$default_db_driver."',	
	'".$default_db_type."',	
	'".$default_db_host."',
	'".$default_db_host_remote."',
	'".$encrypted_user."',
	'".$encrypted_password."',
	'".$default_db_connect_options."'
)");
	//echo ($insertSQL); 

	// Insert into the System level
	$result = $this->sqlConvert_sql_query($insertSQL, $this->connectionSystem);
	if ($result == FALSE) 
	{
		// Database Error
		return FALSE;
	}

	// Update the database name (to be created) using the client_id (pk)
	$Client_Id = $this->sqlConvert_sql_insert_id($this->connectionSystem);
	if ($Client_Id > 0)
	{
		$pNewClientId = $Client_Id;
		
		$updateSQL = (
"UPDATE 
	".$this->sqlConvert_FROM(DATABASE_MASTER,'client')."	
SET
	client_db_name = '".$default_db_namestub.$Client_Id."'
WHERE 
	client_id=".$Client_Id);
		//echo($updateSQL); die;
		
		$result = $this->sqlConvert_sql_query($updateSQL, $this->connectionSystem);	
		if ($result == FALSE) 
		{
			// Database Error
			return FALSE;
		}
	
		// Create the Client Database space
		$createSQL = (
"CREATE DATABASE ".$default_db_namestub.$Client_Id);
		return $this->sqlConvert_sql_query($createSQL, $this->connectionSystem);
	}
	
	// Should not have got this far
	return FALSE;
} // addClientDatabase
   
   
   
/**
* addClientTableEntry - Inserts the given data into the database
* Returns true on success, false otherwise.
*/
function addClientTableEntry(	
					$client_id,
					$client_name,
					$client_utc_offset_yn,
					$client_observes_dst,
					$client_phone,
					$client_promo_code,
					$client_enews,
					$client_referrer,
					$triallength)
{
	// Convert the UTC offset to a code
	$mZoneCode		= getZoneCodeFromUTCOffset($client_utc_offset_yn);	

	// Create the insert
	$insertSQL = sprintf(
"INSERT INTO ".TBL_CLIENT."
(
  client_id,
  client_name,
  client_phone,
  client_send_callerid,
  client_utc_offset,
  client_timezone,
  client_observes_dst_yn,
  client_promo_code,
  client_enews,
  client_referrer,
  triallength
) 
VALUES
(             
  %s, 
  %s, 
  %s, 
  %s, 
  %s, 
  %s, 
  %s,
  %s, 
  %s, 
  %s, 
  %s 
)",
	GetSQLValueString($client_id, "text"),
	GetSQLValueString($client_name, "text"),
	GetSQLValueString($client_phone, "text"),
	GetSQLValueString($client_phone, "text"),  // Default the caller id here
	GetSQLValueString($client_utc_offset_yn, "text"), 
	GetSQLValueString($mZoneCode, "text"),
	GetSQLValueString($client_observes_dst, "text"), 
	GetSQLValueString($client_promo_code, "text"),
	GetSQLValueString($client_enews, "text"),
	GetSQLValueString($client_referrer, "text"),
	GetSQLValueString($triallength, "text")
	);  
	
	//echo '<br><br>'.$insertSQL.'<br><br>'; die;

	// Once the SINGLE client row is updated, we can set the entire database to active
	if ($this->sqlConvert_sql_query($insertSQL, $this->connectionClient))
	{
		// Update the client to ACTIVE in the master CLIENT table
		$updateSQL = (
"UPDATE 
	".$this->sqlConvert_FROM(DATABASE_MASTER,'client')."	
SET
	client_status = 'ACTIVE'
WHERE 
	client_id=".$client_id);
		//echo($updateSQL); die;
		
		$result = $this->sqlConvert_sql_query($updateSQL, $this->connectionSystem);	
		if ($result == FALSE) 
		{
			// Database Error
			return FALSE;
		}
	}
	
	// We got here by being successful
	return true;
}  // addClientTableEntry


/**
* updateClientField - Updates a field, specified by the field
* parameter, in the client's row of the database.
*/
function updateClientField($client_id, $field, $value)
{
	$q = "UPDATE ".TBL_CLIENT." SET ".$field." = '$value' WHERE client_id = ".$client_id;
	
	return $this->sqlConvert_sql_query($q, $this->connectionClient);
} // updateClientField



/*  // Create an array insert statement for a table
Parm 1: Table name
Parm 2: Array from Fetch
Parm 3: Ignor first field (used to copy contents from one identically names set of fields to another ... ignoring the primary key)
*/
function insert_sql_from_array($my_table, $my_array, $ignor_auto_values) 
{
    $keys = '';
	$i = 0;
	$keyend = 0;
	
	// Make sure we have an array
	if ($my_array)
	{
		foreach ( array_keys($my_array) as $key_item) 
		{
			$i++;
			if ($i % 2 == 0)
			{
				$keyend = $i;
				if (strtolower($key_item) == 'create_dtm_utc')  // standard first field for auto value creates
				{
					break;
				}
				
				// Ignor the PK
				if (!($ignor_auto_values && ($i <= 2)))
				{
					if ($keys)
					{
						$keys .= ', ';
					}
					$keys .= $key_item;
				}
			}
		}	
		
		$values = ''; 
		$i = 0;
		foreach ( array_values($my_array) as $key_item) 
		{
			$i++;
			if ($i % 2 == 0)
			{
				// Break if we are done 
				if ($i == $keyend)
				{
					break;
				}
				
				// Ignor the PK
				if (!($ignor_auto_values && ($i <= 2)))
				{
					if ($values)
					{
						$values .= ', ';
					}
					$values .= '"'.$key_item.'"';
				}
			}
		}
		$sql = 'INSERT INTO '.$my_table.' (' .$keys. ') VALUES ('.$values . ')';
		return ($sql);
	}
	else
	{
		return null;
	}
} // insert_sql_from_array

   
// Add an application to the Client
function addClientApp($client, $application_type_name)
{
	// Get the application id for app reminders
	$qry = "
SELECT 
	* 
FROM 
	".$this->sqlConvert_FROM(DATABASE_MASTER,'callout_application_defaults')."	
WHERE 
	app_type_name='".$application_type_name."'";
	
	if (!$res = $this->sqlConvert_sql_query($qry, $this->connectionSystem))
	{    
		return NULL;
	}
	else
	{
		// Get the application id to see if there is a settings row to add
		$row = $this->sqlConvert_sql_fetch_array($res);
		if ($row)
		{
			// See if it already exists
			$q = "
SELECT 
	*
FROM 
	callout_settings	
WHERE 
	app_type_name='".$application_type_name."'";
			$result = $this->sqlConvert_sql_query($q, $this->connectionClient);
			if ($this->sqlConvert_sql_numrows($result) < 1)
			{		
				$insertSQL = $this->insert_sql_from_array('callout_settings', $row, TRUE);
				//echo ('<br />---<br />'.$insertSQL.'<br />===<br />');
				
				// Insert attempt	
				return $this->sqlConvert_sql_query($insertSQL, $this->connectionClient);
			}
		}
		
		// If we get this far, exit
		return NULL;
	}
}  //addClientApp  


/**
* updateUserField - Updates a field, specified by the field
* parameter, in the user's row of the database.
*/
function updateUserField($username, $field, $value)
{
	// For added security, if the client_id is set, then add the check
	if ((isset($_SESSION['CLI_DB_CLIENT_ID'])) &&
		($_SESSION['CLI_DB_CLIENT_ID'] <> "") && 
		($_SESSION['CLI_DB_CLIENT_ID'] <> "0"))
	{
		// Make sure we select the client database
		$selected = $this->selectClientDatabaseByMasterClientId($_SESSION['CLI_DB_CLIENT_ID']);
		$q = "
UPDATE 
	".TBL_USERS." 
SET 
	".$field." = '$value' 
WHERE 
	user_login_name = '$username' 
AND u_status != 'DELETED'
AND client_id = ".$_SESSION['CLI_DB_CLIENT_ID'];
	}
	else
	{
		// This will happen when the user is signing up ... for example
		// --Make sure we select the client database
		$mMasterUserArray = $this->getUserInfoMaster($username);
		$mClient_Id = $mMasterUserArray['client_id'];
		$selected = $this->selectClientDatabaseByMasterClientId($mClient_Id);
		$q = "
UPDATE 
	".TBL_USERS." 
SET 
	".$field." = '$value' 
WHERE 
	user_login_name = '$username' 
AND u_status != 'DELETED'
AND client_id = ".$mClient_Id;
	}
	//echo $q; die;	
	
	return $this->sqlConvert_sql_query($q, $this->connectionClient);
}  // updateUserField

   
/**
* getUserIdData - Returns the result array from a mysql
* query asking for all information stored regarding
* the given user id. If query fails, NULL is returned.
*/
function getUserIdData($userid)
{
	$q = "
SELECT 
	* 
FROM 
	".TBL_USERS." 
WHERE 
	u_id = ".$userid;
	//echo $q; die;	
	
	$result = $this->sqlConvert_sql_query($q, $this->connectionClient) or die("Error: " . mysql_error()); 
	/* Error occurred, return given name by default */
	if(!$result || ($this->sqlConvert_sql_numrows($result) < 1))
	{
		return NULL;
	}
	/* Return result array */
	$dbarray = $this->sqlConvert_sql_fetch_array($result);
	return $dbarray;
} // getUserIdData


/**
* getAdminTotal - Returns the number of admins for the client
*/
function getAdminTotal($client)
{
	$q = "
SELECT 
	count(*) total 
FROM 
	".TBL_USERS." 
WHERE 
	client_id = ".$client."
AND u_status != 'DELETED'
AND u_permissions_level >= ".ADMIN_LEVEL;

	$result = $this->sqlConvert_sql_query($q, $this->connectionClient);
	/* Error occurred, return given name by default */
	if(!$result || ($this->sqlConvert_sql_numrows($result) < 1))
	{
		return NULL;
	}
	/* Return result array */
	$dbarray = $this->sqlConvert_sql_fetch_array($result);
	return $dbarray['total'];
}  // getAdminTotal
   
/*
HOW TO SELECT A USER DATABASE WHEN ALL YOU KNOW IS A USER NAME
if (($database->selectClientDatabaseByUsername($subemail)) &&
   ($database->emailInUse($subemail)))
 {
	// Load up the user info
	echo ('user:'.$subemail);
	$this->userinfo      = $database->getUserInfo($subemail);
}
*/
   
/**
getUserInfoMaster - Returns the result array from a mysql
query asking for all information stored regarding
the given username. If query fails, NULL is returned.
*/
function getUserInfoMaster($username)
{
	$q = "
SELECT 
	* 
FROM 
	".$this->sqlConvert_FROM(DATABASE_MASTER,'user_master')."	
WHERE 
	user_login_name = '".$username."'";	
	//echoDie(true, $q);
	
	$result = $this->sqlConvert_sql_query($q, $this->connectionSystem);

	/* Error occurred, return given name by default */
	if(!$result || ($this->sqlConvert_sql_numrows($result) < 1))
	{
		return NULL;
	}
	/* Return result array */
	$dbarray = $this->sqlConvert_sql_fetch_array($result);
	return $dbarray;
} // getUserInfoMaster



/**
getUserFromMasterByUserId - Returns the result array from a mysql
query asking for all information stored regarding
the given username. If query fails, NULL is returned.
*/
function getUserFromMasterByUserId($user_master_id)
{
	$q = "
SELECT 
	* 
FROM 
	".$this->sqlConvert_FROM(DATABASE_MASTER,'user_master')."	
WHERE 
	user_master_id = '".$user_master_id."'";
	$result = $this->sqlConvert_sql_query($q, $this->connectionSystem);

	/* Error occurred, return given name by default */
	if(!$result || ($this->sqlConvert_sql_numrows($result) < 1))
	{
		return NULL;
	}
	/* Return result array */
	$dbarray = $this->sqlConvert_sql_fetch_array($result);
	return $dbarray['user_login_name'];
}
   	
	
/**
getUserInfo - Returns the result array from a mysql
query asking for all information stored regarding
the given username. If query fails, NULL is returned.
*/
function getUserInfo($username)
{
	$q = "
SELECT 
	* 
FROM 
	".TBL_USERS." 
WHERE 
	user_login_name = '".$username."'
AND u_status != 'DELETED'";
	$result = $this->sqlConvert_sql_query($q, $this->connectionClient);

	/* Error occurred, return given name by default */
	if(!$result || ($this->sqlConvert_sql_numrows($result) < 1))
	{
		return NULL;
	}
	/* Return result array */
	$dbarray = $this->sqlConvert_sql_fetch_array($result);
	return $dbarray;
}  // getUserInfo


/**
* getNumMembers - Returns the number of signed-up users
* of the website, banned members not included. The first
* time the function is called on page load, the database
* is queried, on subsequent calls, the stored result
* is returned. This is to improve efficiency, effectively
* not querying the database when no call is made.
*/
function getNumMembers()
{
	if($this->num_members < 0)
	{
		$q = "
SELECT 
	* 
FROM 
	".TBL_USERS."
WHERE
	u_status != 'DELETED'";
		$result = $this->sqlConvert_sql_query($q, $this->connectionClient);
		$this->num_members = $this->sqlConvert_sql_numrows($result);
	}
	return $this->num_members;
}  // getNumMembers


   
/**
* calcNumActiveUsers - Finds out how many active users
* are viewing site and sets class variable accordingly.
*/
function calcNumActiveUsers()
{
	if ((!isset($_SESSION['CLI_DB_CONNECTED']))) return;

	/* Calculate number of users at site */
	$q = "
SELECT 
	count(*) total 
FROM ".TBL_ACTIVE_USERS;
	$result = $this->sqlConvert_sql_query($q, $this->connectionClient);
    //$dbarray = sqlConvert_sql_fetch_array($result);
	//return $dbarray['total'];
	$this->num_active_users = $this->sqlConvert_sql_numrows($result);
} // calcNumActiveUsers


   
/**
* calcNumActiveGuests - Finds out how many active guests
* are viewing site and sets class variable accordingly.
*/
function calcNumActiveGuests()
{
	if ((!isset($_SESSION['CLI_DB_CONNECTED']))) return;
	
	/* Calculate number of guests at site */
	$q = "
SELECT 
	* 
FROM ".TBL_ACTIVE_GUESTS;

	$result = $this->sqlConvert_sql_query($q, $this->connectionClient);
	$this->num_active_guests = $this->sqlConvert_sql_numrows($result);
} // calcNumActiveGuests


   
/**
* addActiveUser - Updates username's last active timestamp
* in the database, and also adds him to the table of
* active users, or updates timestamp if already there.
*/
function addActiveUser($username, $time)
{
	$q = "
UPDATE 
	".TBL_USERS." 
SET 
	u_timestamp = '".$time."' 
WHERE 
	user_login_name = '".$username."'
AND u_status != 'DELETED'";
	$this->sqlConvert_sql_query($q, $this->connectionClient);

	// Return if we are not tracking
	if ((!TRACK_VISITORS) || (!isset($_SESSION['CLI_DB_CONNECTED']))) return;

	// Remove and add 
	$q = "DELETE FROM ".TBL_ACTIVE_USERS." WHERE u_username = '".$username."'";
	$this->sqlConvert_sql_query($q, $this->connectionClient);
	
	$q = "INSERT INTO ".TBL_ACTIVE_USERS." VALUES ('".$username."','".$time."')";
	$this->sqlConvert_sql_query($q, $this->connectionClient);

	$this->calcNumActiveUsers();
} // addActiveUser

/* addActiveGuest - Adds guest to active guests table */
function addActiveGuest($ip, $time)
{
	// Return if we are not tracking
	if ((!TRACK_VISITORS) || (!isset($_SESSION['CLI_DB_CONNECTED']))) return;

	$q = "DELETE FROM ".TBL_ACTIVE_GUESTS." WHERE g_ip = '".$ip."'";
	$this->sqlConvert_sql_query($q, $this->connectionClient);
	$q = "INSERT INTO ".TBL_ACTIVE_GUESTS." VALUES ('".$ip."','".$time."')";
	$this->sqlConvert_sql_query($q, $this->connectionClient);

	$this->calcNumActiveGuests();
} // addActiveGuest
   

/* These functions are self explanatory, no need for comments */

/* removeActiveUser */
function removeActiveUser($username)
{
	if ((!TRACK_VISITORS) || (!isset($_SESSION['CLI_DB_CONNECTED']))) return;

	$q = "DELETE FROM ".TBL_ACTIVE_USERS." WHERE user_login_name = '".$username."'";
	$this->sqlConvert_sql_query($q, $this->connectionClient);
	$this->calcNumActiveUsers();
}
   
/* removeActiveGuest */
function removeActiveGuest($ip)
{
	if ((!TRACK_VISITORS) || (!isset($_SESSION['CLI_DB_CONNECTED']))) return;

	$q = "DELETE FROM ".TBL_ACTIVE_GUESTS." WHERE g_id = '".$ip."'";
	$this->sqlConvert_sql_query($q, $this->connectionClient);
	$this->calcNumActiveGuests();
}
   
/* removeInactiveUsers */
function removeInactiveUsers()
{
	if ((!TRACK_VISITORS) || (!isset($_SESSION['CLI_DB_CONNECTED']))) return;

	$timeout = time()-USER_TIMEOUT*60;
	$q = "DELETE FROM ".TBL_ACTIVE_USERS." WHERE u_timestamp < ".$timeout;
	$this->sqlConvert_sql_query($q, $this->connectionClient);
	$this->calcNumActiveUsers();
}

/* removeInactiveGuests */
function removeInactiveGuests()
{
	if ((!TRACK_VISITORS) || (!isset($_SESSION['CLI_DB_CONNECTED']))) return;

	$timeout = time()-GUEST_TIMEOUT*60;
	$q = "DELETE FROM ".TBL_ACTIVE_GUESTS." WHERE g_timestamp < ".$timeout;
	$this->sqlConvert_sql_query($q, $this->connectionClient);
	$this->calcNumActiveGuests();
}
   
/**
* query - Performs the given query on the database and
* returns the result, which may be false, true or a
* resource identifier.
*/
function query($query)
{
	return $this->sqlConvert_sql_query($query, $this->connectionClient);
}

	// END OF CLASS
};

/* Create database connection */
$database = new MySQLDB;

?>
