<?php
/*
 Session.php
 
 The Session class is meant to simplify the task of keeping
 track of logged in users and also guests.
 * Written by: Jpmaster77 a.k.a. The Grandmaster of C++ (GMC)
 *  Overhauled and updated by: vbaker@engenic.com
 Last Updated: April 13, 2010

===HOW LOGIN WORKS===
* User logs in on the cr_main.php screen 
* <form action="cr_process.php" method="POST"> is fired on the Login (submit) button
* cr_process -> procLogin is run which then calls session -> login
* session -> login then calls database -> confirmUserPass
* database -> confirmUserPass 
* If we get passed that , the management database is processed once, and then continued
 */
 
/* 
// Needed because this file is called from various locations
$includeFile = "LocalSettings.php";
if (!file_exists($includeFile))
{
	// Load required files relative
	include("../".$includeFile);
	// Email performed by Swift
	require_once "../swift_emailer/lib/Swift.php";
	require_once "../swift_emailer/lib/Swift/Connection/SMTP.php";
	require_once "../swift_emailer/lib/Swift/Authenticator/LOGIN.php";  
}
else
{
	// Load required files relative
	include($includeFile);
	// Email performed by Swift
	require_once "swift_emailer/lib/Swift.php";
	require_once "swift_emailer/lib/Swift/Connection/SMTP.php";
	require_once "swift_emailer/lib/Swift/Authenticator/LOGIN.php";
}
*/

// NOTE - SessionStart is found in the LocalSettings.php file

// Make sure we have the variable set
if (!isset($incPath))
{
	$incPath = '';
}

//echo $mSwift; die;

//require_once($incPath.'swift_emailer/lib/Swift.php');
require_once($incPath.'swift_emailer/lib/Swift.php');
require_once($incPath.'swift_emailer/lib/Swift/Connection/SMTP.php');
require_once($incPath.'swift_emailer/lib/Swift/Authenticator/LOGIN.php');

// Email templates created by system owner
require_once $incPath.'../'.'LocalSettings_EmailTemplates.php';

// Fires Contructors
require("database.php");
require("form.php");
	
class Session
{
	var $username;     //Username given on sign-up
	var $userid;       //Random value generated on current login
	var $useruid;      //Pk for user
	var $userclientid; //The client to which the user belongs
	var $userfirst;    //User first name
	var $userlevel;    //The level to which the user pertains
	var $time;         //Time user was last active (page loaded)
	var $logged_in;    //True if user is logged in, false otherwise
	var $url;          //The page url current being viewed
	var $referrer;     //Last recorded site page viewed
	// When in database mode
	var $databasemode	= true;
	var $appslist 		= array();  //The array holding applications
	var $userinfo 		= array();  //The array holding all user info
	var $userinfoMaster = array();  //The array holding all user info from the Master table
	
	/**
	* Note: referrer should really only be considered the actual
	* page referrer in cr_process.php, any other time it may be
	* inaccurate.
	*/

	/* Class constructor */
	function Session()
	{
		$this->time = time();
		$this->startSession();
	}

   
/**
startSession - Performs all the actions necessary to 
initialize this session object. Tries to determine if the
the user has logged in already, and sets the variables 
accordingly. Also takes advantage of this page load to
update the active visitors tables.
*/
	function startSession()
	{
		global $database, $logoutPage, $thisWebPage, $incPath; //The database connection
		// See Local_Settings for session_start()

		///////////////////////////
		// Session security prevention
		///////////////////////////
		// Regenerate the ID to ensure no hijacking
		if (!isset($_SESSION['initiated']))
		{
			// Regenerate Ids should only happen for web sessions
			if (HTTP_MODE)
			{
				session_regenerate_id();
			}
			$_SESSION['initiated'] = true;
		}
		
		
		// Ensure the User Agent is 
		if (isset($_SESSION['HTTP_USER_AGENT']))
		{
			if ($_SESSION['HTTP_USER_AGENT'] != md5($_SERVER['HTTP_USER_AGENT']))
			{
				// Log the user out, if we are not on the login page
				if  ($thisWebPage <> 'cr_main.php')
				{
					header("Location: ".$logoutPage);
				}
			}
		}
		else
		{
			// If we are a robot, we wont be able to get this
			if (isset($_SERVER['HTTP_USER_AGENT']))
			{
				$_SESSION['HTTP_USER_AGENT'] = md5($_SERVER['HTTP_USER_AGENT']);
			}
		}
		
		/* Determine if user is logged in */
		$this->logged_in = $this->checkLogin();
		if(!$this->logged_in)
		{
			$this->username     = $_SESSION['username']     = GUEST_NAME;
			$this->userclientid = $_SESSION['userclientid'] = 0;
			$this->userlevel = GUEST_LEVEL;
		}

		///////////////////////////////
		// This section applies to USERS only
		// - Determine a page or user (using GET.  If a POST is done with gettype... then its certainly an XML request fired off by AJAX 
		///////////////////////////////
		if (!isset($_POST['gettype']))
		{
			// For web logins only
			if (HTTP_MODE)
			{
				/**
				* Set guest value to users not logged in, and update
				* active guests table accordingly.
				*/
				if(!$this->logged_in)
				{
					$database->addActiveGuest($_SERVER['REMOTE_ADDR'], $this->time);
				}
				else
				{
					/* Remove inactive visitors from database */
					$database->removeInactiveUsers();
					$database->removeInactiveGuests();

					/* Update users last active timestamp */
					$database->addActiveUser($this->username, $this->time);
				}
      		}
			
			// Set referrer page if not an XML post
			if (isset($_SESSION['url']))
			{
				$this->referrer = $_SESSION['url'];
			}
			else
			{
				$this->referrer = "/";
			}

			/* Set current url */
			$this->url = $_SESSION['url'] = $_SERVER['PHP_SELF'];
		}
		
	} // startSession

/**
sessionActiveClient - Ensure the session is alive, and not signed out., and that the page 
is loading with the correct security management.
Take measures if that has occured.  Return the client id otherwise.
*/
	function sessionActiveClientSecurity()
	{
		global $logoutPage, $cLocationAccess, $cGlobalSecure;

		// Only allow access for some locations
		if (allowAccessForIPListOrLocalIPs($cLocationAccess ))
		{
			// We are good to go
		}
		else
		{
			// Allow the system to try other access types
			if ($cGlobalSecure) 
			{
				redirectToHTTPS();
			}

			if(!$this->checkLogin())
			{
				header("Location: ".$logoutPage);
				//OR .... echo "Error 403: Not authorized from ".$_SERVER["REMOTE_ADDR"];
				//die;
			}
		}		
		
		if ($_SESSION['userclientid'] <= 0)
		{
			header("Location: ".$logoutPage);
		}
		else
		{
			return $_SESSION['userclientid'];
		}
	}  // sessionActiveClientSecurity
	
/**
checkLogin - Checks if the user has already previously
logged in, and a session with the user has already been
established. Also checks to see if user has been remembered.
If so, the database is queried to make sure of the user's 
authenticity. Returns true if the user has logged in.
*/
	function checkLogin()
	{
		global $database, $incPath;  //The database connection

		/* Check if user has been remembered */
		if ((!isset($_SESSION['adminmode']) &&
			(isset($_COOKIE['cookname']) && isset($_COOKIE['cookid']))))
		{

			$this->username     = $_SESSION['username']    = $_COOKIE['cookname'];
			$this->userid       = $_SESSION['userid']      = $_COOKIE['cookid'];
		}
		
		/* Must be signed on to the Client Database Username and userid have been set and not guest */
		if (isset($_SESSION['CLI_DB_CONNECTED']) &&
			isset($_SESSION['username']) && 
			isset($_SESSION['userid']) &&
			$_SESSION['username'] != GUEST_NAME)
		{
			/* Confirm that username and userid are valid */
			if($database->confirmUserID($_SESSION['username'], $_SESSION['userid']) != 0)
			{
				/* Variables are incorrect, user not logged in */
				//unset($_SESSION['username']);
				//unset($_SESSION['userid']);
				//unset($_SESSION['useruid']);
				//unset($_SESSION['userclientid']);

				$this->unsetSession();
				
				// Get rid of cookies for autolongin
				//setcookie("cookname", "", time()-COOKIE_EXPIRE, COOKIE_PATH);
				//setcookie("cookid",   "", time()-COOKIE_EXPIRE, COOKIE_PATH);
				
				// Return to where user re-enters data
				return false;
			}

			/* User is logged in, set class variables */
			// Load Array
			$this->userinfo      = $database->getUserInfo($_SESSION['username']);

			// Read array fields
			$this->username      = $this->userinfo['user_login_name'];
			$this->userid        = $this->userinfo['u_userid'];
			$this->useruid       = $_SESSION['useruid']  = $this->userinfo['u_id'];			
			$this->userlevel     = $this->userinfo['u_permissions_level'];
			$this->userclientid  = $_SESSION['userclientid'] = $this->userinfo['client_id'];
			$this->userfirst     = $this->userinfo['u_first'];

			return true;
		}
		/* User not logged in */
		else
		{
			return false;
		}
	}

	// Unset all the session variables related to the user
	function unsetSession()
	{
		global $database;  //The database connection
		
		// Client/User standard numbers
		unset($_SESSION['username']);		// eg: vbaker@gmail.com
		unset($_SESSION['userid']);			// eg: vbaker@gmail.com  (note, this used to be a user name, but in following with the new approach.. we combined them
		unset($_SESSION['useruid']);		// eg: 1 - this is the PK in the user_details table
		unset($_SESSION['userclientid']);	// eg: 3042
		unset($_SESSION['userclientname']);	// eg: engenic (gets set in database.php)
		
		// Kill off any admin modes
		unset($_SESSION['adminmode']);
		
		// Client Autologon (will be set, only if the client autologons with (database->selectByIPAddressClientDatabase)
		unset($_SESSION['CLIENTID_BY_IP_ADDR']); 

		// Client db 
		$database->unsetAuthenticatedSession();
		
		// Kill of session variable information eg: $_SESSION
		session_unset();
		session_destroy();
		
	}    
	
	
/**
checkUserName - Checks if the username exists in System User Master
*/
	function checkSystemUserNameExists($pSystemUserName)
	{
		global $database;  //The database connection
   
		$this->userinfoMaster = $database->getUserInfoMaster($pSystemUserName);
		if (!$result)
		{
			return NULL;
		}
		else
		{
			return $this->userinfoMaster['client_id'];
		}
		
	}

/**
Critical and general Error Page
*/
	// Display a total and complete failure in a way that is at least somewhat respectable.  Send email to admin
	// Note: $pProgrammerInfo is not to be seen by the user... its just notes from the program to help resolve
	function SessionErrorPageDisplay($pCritical, $pServiceCode, $pErrorCode, $pDescription, $pDetail = '', $pProgrammerInfo = '')
	{
		// Weve got a whole lot here... because, we are essentially running the webpage out of sync... to get a half decent looking error page
		global	$incPath, $cGlobalSecure,$cGlobalMode,
				$webClickTo, $webLogo, $websiteName, $webLogoWidth, $webLogoHeight, $webCSSMenuGraphic,
				$webCSS, $webMainPage, $webTitleOverride,
				$webCopyright, $webTelephone, $webMailTo, $webEmailSubject;
				
		// Describe issue;
		if ($pCritical == true)
		{
			$mIssue = 'Critical Error';
		}
		else
		{
			$mIssue = 'Session Error';		
		}
		
		// Standard create
		include($incPath."include/header.php"); 
		include($incPath."include/header_divs.php"); 

		?>	
		<div id="topbar">
		<a class="active"><span><div id="bartitle"><?php echo($mIssue);?></div></span></a>	
		</div>
		<?php
		echo('<p id="layoutdims">&nbsp;'.'</p>');
		echo('<br />');
		
		// Put everything in a box
		echo (displayContainerShadedHeader(true));
		echo('<p><span class="important">'.$pErrorCode.' - '.$pDescription.'</span></p>');
		echo('<br />');
		// If there is some detail, put that in too
		if ($pDetail <> '')
		{
			echo('<p>'.$pDetail.'</p>'.
				'<br />');
		}
		
		if ($pCritical == true)	
		{
			// For critical errors, provide extra detail
			echo('<p>The system administrator has been alerted to this problem.  Time to resolution is
					dependent on the nature of the problem.  Please check back later, and hopefully it 
					will be a better experience then.</p>');
					
			// If a password is available for the email system, then it is probably good to go
			if (defined("EMAIL_STMP_PASS"))
			{
				// Send off an email stating what just happend (this shouldnt occur)
				$this->sendSessionEmail(EMAIL_ADMIN_ADDR, 
											$mIssue.' - '.$pServiceCode.' - ERROR: '.$pErrorCode,
											$pDescription.' ... '.$pDetail.' ... '.$pProgrammerInfo);
			}
		}
		else
		{
			// Provide login information for other errors
			$mGo = 'location.href="'.$incPath.'cr_main.php"';
			echo("<p><input type=\"button\" value=\"Login\" onclick='".$mGo."'></p><br />");
		}
		
		echo (displayContainerShadedFooter());

		// Toss a few spaces in to give the expanding box some vertical space to grow into
		echo('<br />');
		echo('<br />');
		echo('<br />');
		echo('<br />');
		echo('<br />');
		echo('<br />');
		echo('<br />'); // Enough to deal with any Detail the programmer may have provided
		echo('<br />');
		echo('<br />');
		echo('<br />');
		include($incPath."include/footer.php");
		
		
		// IMPORTANT: The very next code following this function call must be:
		/*
		die();
		exit();
		*/
	} // CriticalErrorDisplay
	
	
/**
login - The user has submitted his username and password
through the login form, this function checks the authenticity
of that information in the database and creates the session.
Effectively logging in the user if all goes well.
*/
	function login($subuser, $subpass, $subremember)
	{
		global $database, $form;  //The database and form object

		// Is the system locking out 
		$mLockoutReason = '';
		if ($database->securityAccessCheck($_SERVER['REMOTE_ADDR'],session_id(),$mLockoutReason))
		{
			/* Username error checking */	  
			$field = "user";  //Use field name for username
			if(!$subuser || strlen($subuser = trim($subuser)) == 0)
			{
				$form->setError($field, providePhrase('cr', 640)); // Username not entered
			}
			else
			{
				/* Check if username is not alphanumeric */
				if(!isValidEmail($subuser))
				{
					$form->setError($field, providePhrase('cr', 641)); // Username email address is not valid
				}
			}

			/* Password error checking */
			$field = "pass";  //Use field name for password
			if(!$subpass)
			{
				$form->setError($field, providePhrase('cr', 640)); // Password not entered
			}
					
			/* Return if form errors exist */
			if($form->num_errors > 0)
			{
				// Log the access attempt
				$database->logAccess($_SERVER['REMOTE_ADDR'],session_id(),'LoginFail',0,0,'FormError',$subuser);		
				return false;
			}			
		}
		else
		{
			// We have been locked out
			$field = "user";  //Use field name for username
			$form->setError($field, providePhrase('cr', 643).$mLockoutReason); // LOCKOUT reason error
			
			return false;
		}


		/* Checks that username is in database and password is correct */
		$subuser = stripslashes($subuser);
		// NOTE: This sets up: Session CLI_DB_CLIENT_ID
		$result = $database->confirmUserPass($subuser, md5($subpass));

		/* Check error codes */
		if($result == 1)
		{
			$field = "user";
			$form->setError($field, providePhrase('cr', 644)); // Username not found
		}
		else if($result == 2)
		{
			$field = "pass";
			$form->setError($field, providePhrase('cr', 645)); // Invalid password
		}
		else if($result == 3)
		{
			$field = "user";
			$form->setError($field, providePhrase('cr', 646)); // Account not yet confirmed (see email)
		}
		else if($result == 4)
		{
			$field = "user";
			$form->setError($field, providePhrase('cr', 647)); // Account not currently active
		}
		else if($result == 5)
		{
			$field = "user";
			$form->setError($field, providePhrase('cr', 644)); // Username not found
		}
		

		/* Return if form errors exist */
		if($form->num_errors > 0)
		{
			$database->logAccess($_SERVER['REMOTE_ADDR'],session_id(),'LoginFail',0,0,'CheckError'.$result,$subuser);		
			return false;
		}
	  
	  
		/* Username and password correct, register session variables */
		$this->userinfo      = $database->getUserInfo($subuser);
		// Check to see if we have matched the user id to the client (it is possible that there is a user name with a different client id)
		if (!$this->userinfo)
		{
			$field = "user";
			$form->setError($field, providePhrase('cr', 649)); // User record not located
			return false;
		}

		// Make sure we came back with something real
		if ($this->userinfo['client_id'] == '')
		{
			echo ('ERROR: System Installation Incorrect - Client ID not returned for '.$subuser); 
			die;
		}
		$this->username      = $_SESSION['username'] = $this->userinfo['user_login_name'];
		$this->userid        = $_SESSION['userid']   = $this->generateRandID();
		$this->useruid       = $_SESSION['useruid']  = $this->userinfo['u_id'];
		$this->userlevel     = $this->userinfo['u_permissions_level'];
		$this->userclientid  = $_SESSION['userclientid'] = $this->userinfo['client_id'];
		$this->userfirst     = $this->userinfo['u_first'];

		/* Insert userid into database and update active users table */
		$database->updateUserField($this->username, "u_userid", $this->userid);
		$database->addActiveUser($this->username, $this->time);
		$database->removeActiveGuest($_SERVER['REMOTE_ADDR']);		
		
		// Log the access
		$database->logAccess($_SERVER['REMOTE_ADDR'],session_id(),'Login',$_SESSION['useruid'], $_SESSION['userclientid'],'',$_SESSION['username']);		
/**
* This is the cool part: the user has requested that we remember that
* he's logged in, so we set two cookies. One to hold his username,
* and one to hold his random value userid. It expires by the time
* specified in constants.php. Now, next time he comes to our site, we will
* log him in automatically, but only if he didn't log out before he left.
*/
		if ($subremember)
		{			
			setcookie("cookname", $this->username, time()+COOKIE_EXPIRE, COOKIE_PATH);
			setcookie("cookid",   $this->userid,   time()+COOKIE_EXPIRE, COOKIE_PATH);
		}
		else
		{
			// Unset them for next time
			setcookie("cookname", "", time()-COOKIE_EXPIRE, COOKIE_PATH);
			setcookie("cookid",   "", time()-COOKIE_EXPIRE, COOKIE_PATH);
		}

		/* Login completed successfully */
		return true;
	} // login

	
/**
logout - Gets called when the user wants to be logged out of the
website. It deletes any cookies that were stored on the users
computer as a result of him wanting to be remembered, and also
unsets session variables and demotes his user level to guest.
*/
	function logout()
	{
		global $database;  //The database connection
		
		// Log the access
		$database->logAccess($_SERVER['REMOTE_ADDR'],session_id(),'Logout',$_SESSION['useruid'], $_SESSION['userclientid'],'',$_SESSION['username']);		

		/* Unset PHP session variables */
		$this->unsetSession();

		/* Reflect fact that user has logged out */
		$this->logged_in = false;
      
	  
		/**
		* Remove from active users table and add to
		* active guests tables.
		*/
		$database->removeActiveUser($this->username);
		$database->addActiveGuest($_SERVER['REMOTE_ADDR'], $this->time);

		/* Set user level to guest */
		$this->username  = GUEST_NAME;
		$this->userlevel = GUEST_LEVEL;
	} // logout

	
/**
AccessLockoutCheck - Tests to see if the system has locked out accesses for this IP or Session
*/
	function AccessLockoutCheck(&$pLockoutMessage)
	{
		global $database;  //The database connection
			
		// Is the system locking out 
		$mLockoutReason = '';
		
		// If we didnt pass the access check, then we have been locked out
		if (!$database->securityAccessCheck($_SERVER['REMOTE_ADDR'],session_id(),$mLockoutReason))
		{
			$pLockoutMessage = $mLockoutReason;
			return true;
		}
		// We are not locked out
		return false;
	} // AccessLockoutCheck
	
/**
accessUnLockCode - Generate a code for the user to unlock the system by
*/
	function accessUnlockCode($pUser)
	{
		global $database, $form;  //The database and form object

		// See calcworkunit for code to build the unlock code
		$mHash		= buildUnblockCode($pUser);
		
		/* Username error checking */	  
		$field = "user";  //Use field name for username
		if((!$pUser || strlen($pUser = trim($pUser)) == 0) ||
			(!$database->getUserInfoMaster($pUser)))
		{
			$form->setError($field, providePhrase('cr', 650)); // Unable to locate user
		}
		
		return $mHash;
	}  // accessUnlockCode
	
/**
accessUnLock - Test for valid unlock code, and allow user to attempt a new password if valid
*/
	function accessUnlock($pUser, $pUnlockcode)
	{
		global $database, $form;  //The database and form object

		
		// See calcworkunit for code to build the unlock code
		$mHash		= buildUnblockCode($pUser);

		// If the hash matches the unlock code, then we are good to go
		if ($mHash	 == $pUnlockcode)
		{		
			// Is the system locking out 
			$mLockoutReason = '';
			if (!$database->logAccess($_SERVER['REMOTE_ADDR'],session_id(),'AccessUnlock',0,0,$pUnlockcode,$pUser))
			{
				/* Username error checking */	  
				$field = "user";  //Use field name for username
				if(!$subuser || strlen($subuser = trim($subuser)) == 0)
				{
					$form->setError($field, providePhrase('cr', 651)); // Unable to unlock
				}
			}
		
		}
		else
		{
			/* Username error checking */	  
			$field = "user";  //Use field name for username
			$form->setError($field, providePhrase('cr', 654)); // Unlock user/code mismatch or expiry
		}
	} // accessUnlock

/**
addDefaultClientApplications - Add all applications for the system type (eg: MR) for a client
- Will create entries to the client_application_subscription table then giving client access to an application
**/   
	function addDefaultClientApplications($subclient, $pSystemCode)
	{
		global $database;
		
		// Insert entries into the client subscription, and give a count
		return $database->installApplicationCodesForSystem($subclient, $pSystemCode);	
	} // addDefaultClientApplications


/**
Add an Application Capability to the client 
- Will create an entry to the client_application_subscription table then giving client access to an application
**/   
	function addClientApplication($subclient,
								  $subapplicationtypename)
	{
		global $database, $form;  //The database and  form object
		
		// Does the application exist?
		if ($database->testApplicationCodeExists($subapplicationtypename))
		{
			// First, add the application to the list so that it shows on the menu in cr_main etc
			// Note: This reuses code from the system that indicates new data to process in order to create the application code
			if ($database->systemDatabaseIndicateNewData($subclient,
									  $subapplicationtypename,
									  0))
			{
				// Add any additional call outs settings entires etc
				if ($database->addClientApp($subclient,
										  $subapplicationtypename))		   
				{
					return 0;  //Registration attempt success
				}
				else
				{
					return 1;  //Registration attempt failed
				}
			}
			else
			{
				return 1;  //Registration attempt failed
			}
		}		
		else
		{
			return 2;  //Registration attempt failed because it does not exist
		}
	} // addClientApplication
   
   
/**
resetpass - Gets called when the user has just submitted the
password reset request form. Determines if there were any errors with
the entry fields, if so, it records the errors and returns
1. If no errors were found, it emails the user and
returns 0. Returns 2 if reset failed.
*/
	function resetpass($subemail, 
					 $subnamelast, 
					 $subnamefirst)
	{
		global $database, $form;  //The database and  form object

		/* First NameLast error checking */
		$field = "namelast";  //Use field name for namelast
		if(!$subnamelast || strlen($subnamelast = trim($subnamelast)) == 0)
		{
			$form->setError($field, providePhrase('cr', 655)); // Last name not entered
		}
		else
		{
			/* Spruce up NameLast, check length */
			$subnamelast = stripslashes($subnamelast);
			if(strlen($subnamelast) > 20)
			{
				$form->setError($field, providePhrase('cr', 656)); // Last name above 20 characters
			}
			/* Check if NameLast is not alphanumeric */
			else if(!eregi("^([0-9a-z-])+$", $subnamelast))
			{
				$form->setError($field, providePhrase('cr', 657)); // Last name not alphanumeric
			}
		}


		/* First NameFirst error checking */
		$field = "namefirst";  //Use field name for namefirst
		if(!$subnamefirst || strlen($subnamefirst = trim($subnamefirst)) == 0)
		{
			$form->setError($field, providePhrase('cr', 658)); // First name not entered
		}
		else
		{
			/* Spruce up NameFirst, check length */
			$subnamefirst = stripslashes($subnamefirst);
			if (strlen($subnamefirst) > 20)
			{
				$form->setError($field, providePhrase('cr', 659)); // First name above 20 characters
			}
			/* Check if NameFirst is not alphanumeric */
			else if(!eregi("^([0-9a-z-])+$", $subnamefirst))
			{
				$form->setError($field, providePhrase('cr', 660)); // First name not alphanumeric
			}
		}

      
		/* Email error checking */
		$field = "email";  //Use field name for email
		if(!$subemail || strlen($subemail = trim($subemail)) == 0)
		{
			$form->setError($field, providePhrase('cr', 666)); // Email not entered
		}
		else
		{
			/* Check if valid email address */
			if(!isValidEmail($subemail))
			{
				$form->setError($field, providePhrase('cr', 667)); // Email invalid
			}
			/* Check if username is already in use */
			else if(!$database->emailInUse($subemail))
			{
				$form->setError($field, providePhrase('cr', 664)); // Email address not found.
			}
			// If the email is in use, find the client it belongs to, and check to see if everything matches
			else if (($database->selectClientDatabaseByUsername($subemail)) &&
					($database->emailInUse($subemail)))
			{
				// Load up the user info
				$this->userinfo      = $database->getUserInfo($subemail);			
				//echo $this->userinfo['u_last'].'-'.$subnamelast.' ... '.$this->userinfo['u_first'].'-'.$subnamefirst; die;
				if (($this->userinfo['u_last']  <> $subnamelast) ||
					($this->userinfo['u_first'] <> $subnamefirst))
				{
					$form->setError($field, providePhrase('cr', 665)); //  Email and names do not match.
				}
			}		 
			$subemail = stripslashes($subemail);
		}
	  	  

		/* Errors exist, have user correct them */
		if($form->num_errors > 0)
		{
			return 1;  //Errors with form
		}
		/* No errors, add the new account to the */
		else
		{
			// Add request, and send emails
			$mResult = $this->resetpassemail($subemail, 0, $subnamelast, $subnamefirst);
			
			$field = "email";  //Use field name for email			
			if ($mResult == 2)
			{
				$form->setError($field, providePhrase('cr', 661)); // Unable to email (system issue)
				return 1;  //Errors with form
			}
			else if ($mResult == 3)
			{
				$form->setError($field, providePhrase('cr', 652)); // User id is not currently active
				return 1;  //Errors with form
			}
			else if ($mResult == 4)
			{
				$form->setError($field, providePhrase('cr', 653)); // User id not located as a client
				return 1;  //Errors with form
			}
			return 0; // We are good
		}
	} // resetpass

   
/**
setnewpass - Gets called when the user has just submitted the
registration form for a new password.  Determines if there were any errors with
the entry fields, if so, it records the errors and returns
1. If no errors were found, it registers the new user and
returns 0. Returns 2 if registration failed.
*/
	function setnewpass($subuserid, // WARNING: This is the ID at the Client Table level
						$subpass, 
						$subpass2) 
	{
		global $database, $form;  //The database and  form object

		/* Password error checking */
		$field = "pass";  //Use field name for password
		if(!$subpass)
		{
			$form->setError($field, providePhrase('cr', 686)); // Password not entered
		}
		else
		{
			/* Spruce up password and check length*/
			$subpass = stripslashes($subpass);
			if(strlen($subpass) < 4)
			{
				$form->setError($field, providePhrase('cr', 687)); // Password too short
			}
			/* Check if password is not alphanumeric */
			else if(!eregi("^([0-9a-z-])+$", ($subpass = trim($subpass))))
			{
				$form->setError($field, providePhrase('cr', 688)); // Password not alphanumeric
			}
			else if($subpass <> $subpass2)
			{
				$form->setError($field, providePhrase('cr', 689)); // Passwords do not match
			}
		}

		/* Errors exist, have user correct them */
		if($form->num_errors > 0)
		{
			return 1;  //Errors with form
		}
		/* No errors, add the new account to the */
		else
		{
			// Update database password
			$this->userinfo      = $database->getUserIdData($subuserid);	
			//echo $this->userinfo['user_login_name']; die;
			
			if ($this->userinfo)
			{
				$result = $database->updateUserField($this->userinfo['user_login_name'], 'u_password', md5($subpass));
				if ($result == 1) 
				{
					// Log that a reset password occured
					$database->logAccess($_SERVER['REMOTE_ADDR'],
										session_id(),'SetNewPass', 
										$this->userinfo['u_id'], 
										$this->userinfo['client_id'],
										'',
										'');				
					return 0;  // Success
				}
				else
				{
					$field = "pass";  //Use field name for password
					$form->setError($field, providePhrase('cr', 662)); // Database Error 1 - Contact Support
					return 2;   // General Failure
				}
			}
			else
			{
				$field = "pass";  //Use field name for password
				$form->setError($field, providePhrase('cr', 663)); // Database Error 2 - Contact Support
				return 2;   // General Failure		
			}
		}
	}  // setnewpass
   

/**
createClientDatabase - Gets called when the a client create webpage 
has been filled in.  Will check to make sure things are good to go before creating.  
This must be followed up with the createClientTables and other commands to complete the process
1. If no errors were found, it registers the new user and
returns 0. Returns 2 if registration failed.
*/
	function createClientDatabase(
					$sublogin_user,
					$subcompany_name,
					&$ref_created_client_id)
	{
		global $database, $form;  //The database and form object

		/* Email error checking */
		$field = "email";  //Use field name for email
		if(!$sublogin_user || strlen($sublogin_user = trim($sublogin_user)) == 0)
		{
			$form->setError($field, providePhrase('cr', 666)); // Email not entered
		}
		else
		{
			/* Check if valid email address */
			if(!isValidEmail($sublogin_user))
			{
				$form->setError($field, providePhrase('cr', 667)); // Email invalid
			}
			/* Check if username is already in use */
			else if($database->emailInUse($sublogin_user))
			{
				$form->setError($field, providePhrase('cr', 668)); // Email address already in use
			}
			$sublogin_user = stripslashes($sublogin_user);
		}

		/* Errors exist, have user correct them */
		if($form->num_errors > 0)
		{
			return 1;  //Errors with form
		}
		/* No errors, add the new account to the */
		else
		{
			// Register client
			$ref_client_id = 0;
			if($database->addClientDatabase(
							$sublogin_user,
							$subcompany_name,
							SYS_DEFAULT_DB_DRIVER,	
							SYS_DEFAULT_DB_TYPE,
							SYS_DEFAULT_DB_HOST,
							SYS_DEFAULT_DB_HOST_REMOTE,
							SYS_DEFAULT_DB_NAMESTUB,
							SYS_DEFAULT_DB_USER,
							SYS_DEFAULT_DB_PASSWORD,
							SYS_DEFAULT_DB_CONNECT_OPTIONS,
							$ref_client_id
							))	   
			{

				// Remember the last add.  This will enable the program to continue with additional registration etc.
				// Remember the client ID
				$_SESSION["client_id"] = $ref_client_id;

				// Connect to next Client Database
				$database->selectClientDatabase(
							SYS_DEFAULT_DB_TYPE,
							SYS_DEFAULT_DB_HOST,
							SYS_DEFAULT_DB_NAMESTUB.$_SESSION["client_id"],
							SYS_DEFAULT_DB_USER,
							SYS_DEFAULT_DB_PASSWORD,
							SYS_DEFAULT_DB_CONNECT_OPTIONS
							);
				
				// Return back the new client id 
				$ref_created_client_id = $ref_client_id;
				
				return 0;  //Registration attempt success
			}
			else
			{
				return 2;  //Registration attempt failed
			}
		}
	} // createClientDatabase

	
	/**
	Create a database space for a new client
	- Should immediately follow: createClientDatabase
	**/
	function createClientDatabaseTables()
	{
		global $database, $incPath, $appCompanySvcCode;  //The database object
		
		$mErrorsList = '';
		
		// Loop through all the files in a SQL create folder
		$dirPath = $incPath.'cr_sql_statements/'.SYS_DATABASE_SYSTEM; // Defined in database settings
		$dir = opendir($dirPath);
		while (($file = readdir($dir)) !== false)
		{
			// Only SQL files, no dirs
			if ((!is_dir($file)) && (strtoupper(rightstr($file,4)) == '.SQL'))
			{
				$fs=($dirPath."/".$file);
				//echo ($fs);
				$moddate=date('F j, Y',filemtime($fs));
				// filesize($fs)
				// formatReadableSize($fs)
				
				// Load and explode the sql file
				$f = fopen($fs,"r");  // r: read only  - r+: read and write
				$sqlFile = fread($f,filesize($fs));
				$sqlArray = explode(';',$sqlFile);			
				
				// Pass the Array to the SQL Create 
				$mErrorDesc = '';
				$mErrorLevel = $this->createClientTables($sqlArray, $mErrorDesc);
				
				// If we got an error at this point, it could be a problem
				if ($mErrorLevel > 0)
				{
					$mErrorsList .= 'ERROR: '.$mErrorLevel.' - '.$mErrorDesc.addCRLF();
				}
				
			}
		}
		closedir($dir);	
		
		
		// Send an email if there is a problem... as this will certainly require some review
		if ($mErrorsList <> '')
		{
			// If a password is available for the email system, then it is probably good to go
			if (defined("EMAIL_STMP_PASS"))
			{
				// Send off an email stating what just happened (this shouldnt ever occur... means that SQL create statements were not tested)
				$this->sendSessionEmail(EMAIL_ADMIN_ADDR, 
										$appCompanySvcCode.' TABLES CREATE ERROR',
										'Review database log files for the following ERROR(S):'.addCRLF().
										$mErrorsList);
			}
		}
		
	} // createClientDatabaseTables
	
	
	// Create the client tables in the client Database
	function createClientTables($sqlArray, &$varErrorDesc)
	{
		global $database;
		
		$mErrorDesc = '';
		$mErrorLevel = $database->createClientTables($sqlArray, $mErrorDesc);	
		
		// Note any errors
		if ($mErrorLevel > 0)
		{
			$varErrorDesc = $mErrorDesc;
		}
		
		return $mErrorLevel;
	} // createClientTables
	
	
	// Select the database based on client id
	function selectClientDatabaseByClientId($pClientId)
	{
		global $database, $appCompanySvcCode;
	
		// If the client has become 0 - chances are that the user has timed out
		if ($pClientId == 0)
		{
			// Display a polite screen
			$this->SessionErrorPageDisplay(false,
							$appCompanySvcCode, 
							'TIMEOUT', 
							'Your session has timed out, or your client database connection has been reset', 
							'For security purposes, or due to updates to the database environment, your session
							information has expired.  In order to continue, please log back into the system.', 
							'selectClientDatabaseByClientId');			
			die;
		}
		else
		{	
			// Connect to Client Database
			$database->selectClientDatabase(
						SYS_DEFAULT_DB_TYPE,
						SYS_DEFAULT_DB_HOST,
						SYS_DEFAULT_DB_NAMESTUB.$pClientId,
						SYS_DEFAULT_DB_USER,
						SYS_DEFAULT_DB_PASSWORD,
						SYS_DEFAULT_DB_CONNECT_OPTIONS
						);
		}
	} // selectClientDatabaseByClientId
	
	

	// Now, create the client data entries in the table
	function addClientTableEntry(
						$subclient_id,
						$subclient_name,
						$subclient_utc_offset_yn,
						$subclient_observes_dst,
						$subclient_phone,
						$subclient_promo_code,
						$subclient_enews,
						$subclient_referrer,
						$subtriallength)
	{
		global $database;  //The database object

		// Register client,
		if($database->addClientTableEntry(
						$subclient_id,
						$subclient_name,
						$subclient_utc_offset_yn,
						$subclient_observes_dst,
						$subclient_phone,
						$subclient_promo_code,
						$subclient_enews,
						$subclient_referrer,
						$subtriallength))		   
		{
			return 0;  //Registration attempt success
		}
		else
		{
			return 2;  //Registration attempt failed
		}
	}
	
/**
register - Gets called when the user has just submitted the
registration form. Determines if there were any errors with
the entry fields, if so, it records the errors and returns
1. If no errors were found, it registers the new user and
returns 0. Returns 2 if registration failed.
*/
	function register($subuser, 
					 $subclient, 
					 $subpass, 
					 $subpass2, 
					 $subnamelast, 
					 $subnamefirst, 
					 $subpassnum, 
					 $subemail,
					 $subregtype)
	{
		global $database, $form;  //The database and  form object

		/////////////////////////////////////////
		// OVERRIDE !!!! Take note - we decided to simply use the email address as the username
		$subuser = $subemail;
		/////////////////////////////////////////


		/* First NameLast error checking */
		$field = "namelast";  //Use field name for namelast
		if(!$subnamelast || strlen($subnamelast = trim($subnamelast)) == 0)
		{
			$form->setError($field, providePhrase('cr', 670)); // Last name not entered
		}
		else
		{
			/* Spruce up NameLast, check length */
			$subnamelast = stripslashes($subnamelast);
			if(strlen($subnamelast) > 20)
			{
				$form->setError($field, providePhrase('cr', 671)); // Last name above 20 characters
			}
			/* Check if NameLast is not alphanumeric */
			else if(!eregi("^([0-9a-z-])+$", $subnamelast))
			{
				$form->setError($field, providePhrase('cr', 672)); // Last name not alphanumeric
			}
		}


		/* First NameFirst error checking */
		$field = "namefirst";  //Use field name for namefirst
		if(!$subnamefirst || strlen($subnamefirst = trim($subnamefirst)) == 0)
		{
			$form->setError($field, providePhrase('cr', 673)); // First name not entered
		}
		else
		{
			/* Spruce up NameFirst, check length */
			$subnamefirst = stripslashes($subnamefirst);
			if(strlen($subnamefirst) > 20)
			{
				$form->setError($field, providePhrase('cr', 674)); // First name above 20 characters
			}
			/* Check if NameFirst is not alphanumeric */
			else if(!eregi("^([0-9a-z-])+$", $subnamefirst))
			{
				$form->setError($field, providePhrase('cr', 675)); // First name not alphanumeric
			}
		}

		/* Password error checking */
		$field = "pass";  //Use field name for password
		if(!$subpass)
		{
			$form->setError($field, providePhrase('cr', 686)); // Password not entered
		}
		else
		{
			/* Spruce up password and check length*/
			$subpass = stripslashes($subpass);
			if(strlen($subpass) < 4)
			{
				$form->setError($field, providePhrase('cr', 687)); // Password too short
			}
			/* Check if password is not alphanumeric */
			else if(!eregi("^([0-9a-z-])+$", ($subpass = trim($subpass))))
			{
				$form->setError($field, providePhrase('cr', 688)); // Password not alphanumeric
			}
			else if($subpass <> $subpass2)
			{
				$form->setError($field, providePhrase('cr', 689)); // Passwords do not match
			}
		 /**
		  * Note: I trimmed the password only after I checked the length
		  * because if you fill the password field up with spaces
		  * it looks like a lot more characters than 4, so it looks
		  * kind of stupid to report "password too short".
		  */
		}


		/* PassNum error checking */
		$field = "passnum";  //Use field name for password
		if(strlen($subpassnum) > 0) 
		{ // Passnumber not mandetory
			 /* Spruce up password and check length*/
			 $subpassnum = stripslashes($subpassnum);
			 if((strlen($subpassnum) < 4))
			 {
				$form->setError($field, providePhrase('cr', 690)); // Pass number too short
			 }
			 /* Check if password is not numeric */
			 else if(!eregi("^([0-9])+$", ($subpassnum = trim($subpassnum))))
			 {
				$form->setError($field, providePhrase('cr', 691)); // Pass number not numeric
			 }
			 /**
			  * Note: I trimmed the password only after I checked the length
			  * because if you fill the password field up with spaces
			  * it looks like a lot more characters than 4, so it looks
			  * kind of stupid to report "password too short".
			  */
		}

      
		/* Email error checking */
		$field = "email";  //Use field name for email
		if(!$subemail || strlen($subemail = trim($subemail)) == 0)
		{
			$form->setError($field,providePhrase('cr', 666)); // Email not entered
		}
		else
		{
			/* Check if valid email address */
			if(!isValidEmail($subemail))
			{
				$form->setError($field, providePhrase('cr', 667)); // Email invalid
			}
			/* Check if username is already in use */
			else if($database->emailInUse($subemail))
			{
				// Is this user in the client database, or another?
				$this->userinfo      = $database->getUserInfo($subemail);
				if ($this->userinfo)
				{
					// It is in this database
					$form->setError($field, providePhrase('cr', 668)); // Email address already in use
				}
				else
				{
					// It is in another client database
					$form->setError($field, providePhrase('cr', 669)); // Email address already in use for another client on this system
				}
			}
			
			$subemail = stripslashes($subemail);
		}

		/* Errors exist, have user correct them */
		if($form->num_errors > 0)
		{
			return 1;  //Errors with form
		}
		/* No errors, add the new account to the */
		else
		{
			// Add user to database, and send emails
			if ($subregtype == REG_BY_SELF)
			{
				return $this->registerUser($subuser, 
											$subclient, 
											$subpass, 
											$subnamelast, 
											$subnamefirst, 
											$subpassnum, 
											$subemail);
			}
			else
			{
				// An ADMINISTATOR for a company is inviting a user
				// -- REG_BY_INVITE
				return $this->registerUserInvite($subuser, 
												$subclient, 
												$subpass, 
												$subnamelast, 
												$subnamefirst, 
												$subpassnum, 
												$subemail);
			}
		}
	} // Register

   
   
/**
registeruser - Calls the database to register a new user
returns 0. Returns 2 if registration failed.
*/
	function registerUser($psubuser, $psubclient, $psubpass, $psubnamelast, $psubnamefirst, $psubpassnum, $psubemail)
	{
		global $database, $form, $incPath;  //The database and form object

		$mLogFile = $incPath.SYSTEM_LOGFILES."SystemRegisterUser.log";
		// Register user, with a hashed password
		$hashpass = md5($psubpass);
		
		// Note a log file to be made
		LogTextToFileXMLTimeStamp($mLogFile, "Attempting Add: ".$psubuser.",".$psubclient.",".$hashpass.",".$psubnamelast.",".$psubnamefirst.",".$psubpassnum.",".$psubemail);

		$userMasterId = $database->addNewUser($psubuser, $psubclient, $hashpass, $psubnamelast, $psubnamefirst, $psubpassnum, $psubemail);
		if ($userMasterId > 0)
		{
			// Remember the last add.  This will enable the program to continue with additional registration etc.
			$_SESSION["new_user_id"] = $userMasterId;

			$mBody = 'master_u_id='.$userMasterId.', '.
					'user='.$psubuser.', '.
					'client='.$psubclient.', '.
					'name_last='.$psubnamelast.', '.
					'name_first='.$psubnamefirst;
					
			// Log response 
			LogTextToFileXMLTimeStamp($mLogFile, $mBody);

			// New users must validate their account using a link
			// -- Must be in this order
			// -- MD5 the pass a second time
			$url = EMAIL_WEBACTIVATE.'?hash='.md5($hashpass).'&id='.$_SESSION["new_user_id"].'&stamp='.base64_encode(time());
			// Log it for review if there is a problem
			LogTextToFileXMLTimeStamp($mLogFile, $url);

			// See if we are allowed to use email
			if ((defined("SYSTEM_INVITEUSEREMAIL")) && (SYSTEM_INVITEUSEREMAIL == true))
			{
				//Connect to SMTP
				$smtp = new Swift_Connection_SMTP(EMAIL_SMTP, EMAIL_PORT);
				$smtp->setUsername(EMAIL_STMP_USER);
				$smtp->setpassword(EMAIL_STMP_PASS);
				$smtp->attachAuthenticator(new Swift_Authenticator_LOGIN());
				$swift = new Swift($smtp);

				// Create the message using LocalSettings_EmailTemplates.php
				$mInvitationTitle = '';
				$msg = CreateEmailBody_UserRegistration($psubnamefirst, $psubnamelast, $url, $mInvitationTitle);
				$message = new Swift_Message($mInvitationTitle, $msg, "text/html"); 
				 
				//Now check if Swift actually sends it
				if ($swift->send($message, $psubemail, EMAIL_FROM_ADDR))
				{	
					//echo "Sent!!";
					return 0;  //New user added succesfully
				}
				else
				{
					//else echo "Failed";
					return 2;  //Registration attempt failed
				}
			}
			else
			{
				// Email not allowed, so, save this request to a session variable, and display it for the administrator at:
				// -- cr_admin_invite
				$_SESSION['admin_link'] = $url;
				return 0;  //New user added succesfully
			}
		}
		else
		{
			return 2;  //Registration attempt failed
		}
	} // registerUser


/**
registerUserInvite -  Registers an intived user, and askes them to set their password
returns 0. Returns 2 if registration failed.
*/
	function registerUserInvite($psubuser, $psubclient, $psubpass, $psubnamelast, $psubnamefirst, $psubpassnum, $psubemail)
	{
		global $database, $form;  //The database and form object

		// Register user, with a hashed password
		$hashpass = md5($psubpass);

		//echo $psubclient;
		$userMasterId = $database->addNewUser($psubuser, $psubclient, $hashpass, $psubnamelast, $psubnamefirst, $psubpassnum, $psubemail);
		if ($userMasterId > 0)
		{
			// Remember the last add.  This will enable the program to continue with additional registration etc.
			$_SESSION["new_user_id"] = $userMasterId;

			// New users must validate their account using a link
			// -- Must be in this order
			// -- MD5 the pass a second time
			$url = EMAIL_WEBINVITE.'?hash='.md5($hashpass).'&id='.$_SESSION["new_user_id"].'&stamp='.base64_encode(time());

			// See if we are allowed to use email
			if ((defined("SYSTEM_INVITEUSEREMAIL")) && (SYSTEM_INVITEUSEREMAIL == true))
			{
				// Users will be sent an email where they can reset the password to their account

				//Connect to SMTP
				$smtp = new Swift_Connection_SMTP(EMAIL_SMTP, EMAIL_PORT);
				$smtp->setUsername(EMAIL_STMP_USER);
				$smtp->setpassword(EMAIL_STMP_PASS);
				$smtp->attachAuthenticator(new Swift_Authenticator_LOGIN());
				$swift = new Swift($smtp);


				// Create the message using LocalSettings_EmailTemplates.php
				$mInvitationTitle = '';
				$msg = CreateEmailBody_UserInvitation($psubnamefirst, $psubnamelast, $url, $mInvitationTitle);			
				$message = new Swift_Message($mInvitationTitle, $msg, "text/html"); 
				 
				//Now check if Swift actually sends it
				if ($swift->send($message, $psubemail, EMAIL_FROM_ADDR))
				{	
					//echo "Sent!!";
					return 0;  //Reset pass sent succesfully
				}
				else
				{
					//else echo "Failed";
					return 2;  //Reset pass attempt failed
				}
			}
			else
			{
				// Email not allowed, so, save this request to a session variable, and display it for the administrator at:
				// -- cr_admin_invite
				$_SESSION['admin_link'] = $url;
				return 0;  //New user added succesfully
			}
			
		}
		else
		{
			return 3;  // Failed user creation
		}
	} // registerUserInvite
   
   
   
/**
* resetpassemail - Sends the user a password reset option
* returns 0. Returns 2 if failed
*/
	function resetpassemail($psubemail, $psubclient, $psubnamelast, $psubnamefirst )
	{
		global $database, $form;  //The database and form object

		// Get the master level detail	
		$this->userinfoMaster = $database->getUserInfoMaster($psubemail);
		$mMasterUserId = $this->userinfoMaster['user_master_id'];

		// Select the client database
		if (($psubemail <> '') &&
			($database->selectClientDatabaseByUsername($psubemail)))
		{
			// Get user info, and sent them 
			$this->userinfo      = $database->getUserInfo($psubemail);	
			if (($this->userinfo['u_password'] <> '') &&
				($this->userinfo['u_status']  == 'ACTIVE'))
			{
				// Users will be sent an email where they can reset the password to their account

				//Connect to SMTP
				$smtp = new Swift_Connection_SMTP(EMAIL_SMTP, EMAIL_PORT);
				$smtp->setUsername(EMAIL_STMP_USER);
				$smtp->setpassword(EMAIL_STMP_PASS);
				$smtp->attachAuthenticator(new Swift_Authenticator_LOGIN());
				$swift = new Swift($smtp);

				// Must be in this order
				// MD5 the pass a second time
				$url = EMAIL_WEBSETPASS.'?hash='.md5($this->userinfo['u_password']).'&id='.$mMasterUserId.'&stamp='.base64_encode(time());

				// Create the message using LocalSettings_EmailTemplates.php
				$mInvitationTitle = '';
				$msg = CreateEmailBody_UserPasswordReset($psubnamefirst, $psubnamelast, $url, $mInvitationTitle);				
				$message = new Swift_Message($mInvitationTitle, $msg, "text/html"); 
				 
				//Now check if Swift actually sends it
				if ($swift->send($message, $psubemail, EMAIL_FROM_ADDR))
				{	
					//echo "Sent!!";
					return 0;  //Reset pass sent succesfully
				}
				else
				{
					//else echo "Failed";
					return 2;  //Reset pass attempt failed
				}
			}
			else
			{
				return 3;  // User not active 
			}
		}
		else
		{
			return 4;  //User not located in the client database
		}
	} // resetpassemail
   
/**
sendSessionEmail - Send an email using swift
*/
	function sendSessionEmail($pToEmailAdd, $pTitle, $pMsg)
	{
		//Connect to SMTP
		$smtp = new Swift_Connection_SMTP(EMAIL_SMTP, EMAIL_PORT);
		$smtp->setUsername(EMAIL_STMP_USER);
		$smtp->setpassword(EMAIL_STMP_PASS);
		$smtp->attachAuthenticator(new Swift_Authenticator_LOGIN());
		$swift = new Swift($smtp);

		// Create the message using LocalSettings_EmailTemplates.php
		$message = new Swift_Message($pTitle, $pMsg, "text/html"); 
		 
		//Now check if Swift actually sends it
		if ($swift->send($message, $pToEmailAdd, EMAIL_FROM_ADDR))
		{	
			//echo "Sent!!";
			return 0;  //Reset pass sent succesfully
		}
		else
		{
			//else echo "Failed";
			return 2;  //Reset pass attempt failed
		}
	} // sendSessionEmail
	
	
/**
activateUser - validates the user based on a passed entry from mail above
$passhash = Password saved at the client database user_details
$timestamp = Timestamp
$userid = System level user_master pk
*/
	function activateUser($userid, $passhash, $timestamp)
	{
		global $database, $form;  //The database and form object

		// First, we need to connect to the database of the userid pk, and get the 
		$user_login_name = $database->getUserFromMasterByUserId($userid);

		// Select the client database
		if (($user_login_name) &&
			($database->selectClientDatabaseByUsername($user_login_name)))
		{			
			// Get the user data from the client databse.
			$this->userinfo      = $database->getUserInfo($user_login_name);		

			// See if the password matches
			if ((md5($this->userinfo['u_password']) == $passhash))
			{
				$hoursecs = 3600;  // 3600 = 1 hour
				if ($this->userinfo['u_status'] == 'ACTIVE')
				{
					// Already activated
					return 2;
				}
				// See if we have the valid time
				else if ((($this->userinfo['u_timestamp']+24*$hoursecs) > $timestamp) &&  // Email must have occured within 24 hours
						 (($this->userinfo['u_timestamp']+24*$hoursecs) > time()))        // User must have done this within 24 hours too
				{
					//echo('3-'.$user_login_name.' '.$this->userinfo['user_login_name'] ); die;				
					// Attempt to update
					return $database->updateUserField($this->userinfo['user_login_name'], 'u_status', 'ACTIVE');  // 1; 
				}
				else
				{			
					// Attempted to activate after 24 hours
					// -- Deactivate the entry
					$deactuser = '_deleted_'.$this->userinfo['user_login_name'];
					$database->updateUserField($this->userinfo['user_login_name'], 'user_login_name', $deactuser);
					
					return 3;
				}			
			}
			else
			{
				// Hash mismatch
				return 4;
			}
		}
		else
		{
			// Hash mismatch
			return 6; // General error
		}
		
	} // activateUser
   
   
/**
registerext - Gets called when the user has just submitted the
registration communications form.
returns 0. Returns 2 if registration failed.
*/
   function registerext($subuser, 
                        $sublocal, 
                        $subcell)
   {
		global $database, $form;  //The database and form object

			/* First NameLast error checking */
			$field = "extlocal";  //Use field name for extlocal
			if(!$sublocal || strlen($sublocal = trim($sublocal)) == 0)
			{
			 $form->setError($field, providePhrase('cr', 678)); // Virtual Extension not entered
			}
			else
			{
			 /* Spruce up sublocal, check length */
			 $sublocal = stripslashes($sublocal);
			 if(strlen($sublocal) > 10)
			 {
				$form->setError($field, providePhrase('cr', 679)); // Virtual Extension above 10 digits
			 }
			 /* Check if username is already in use */
			 else if($database->extlocalInUse($sublocal))
			 {
				$form->setError($field, providePhrase('cr', 680)); //  Virtual Extension has been taken.  Suggestion: Add a number to make it unique. eg: Jenny2
			 }
			}

			/* Errors exist, have user correct them */
			if($form->num_errors > 0)
			{
			 return 1;  //Errors with form
			}
			/* No errors, add the new account to the */
			else
			{
				// Add communications information
				$userExtensionId = ($database->addNewUserExtension( $subuser,
							   '',
							   $sublocal,
							   $subcell,  
							   'SUPER_SWITCH',
							   'Y',  
							   'Y',  
							   'N',  
							   'N',  
							   4,  
							   4,  
							   '10:00:00',  
							   '20:00:00',  
							   'MON,TUE,WED,THU,FRI,SAT,SUN'));
				if ($userExtensionId > 0)
				{
					// Remember the last add.  This will enable the program to continue with additional registration etc.
					$_SESSION["new_user_ext"] = $userExtensionId;
					
					return 0;  //New user ext added succesfully
				}
				else
				{
					return 2;  //Registration attempt failed
				}
			}
		
   } // registerext

   
/**
* editAccount - Attempts to edit the user's account information
* including the password, which it first makes sure is correct
* if entered, if so and the new password is in the right
* format, the change is made. All other fields are changed
* automatically.
*/
	function editAccount($subcurpass, $subnewpass, $subemail, $subpassnum, $subtelephone)
	{
		global $database, $form;  //The database and form object
		/* New password entered */
		if($subnewpass)
		{
			/* Current Password error checking */
			$field = "curpass";  //Use field name for current password
			if(!$subcurpass)
			{
				$form->setError($field, providePhrase('cr', 681)); // Current Password not entered
			}
			else
			{
				/* Check if password too short or is not alphanumeric */
				$subcurpass = stripslashes($subcurpass);
				if(strlen($subcurpass) < 4 ||
				   !eregi("^([0-9a-z])+$", ($subcurpass = trim($subcurpass))))
				{
				   $form->setError($field, providePhrase('cr', 682)); // Current Password incorrect
				}
				/* Password entered is incorrect */
				if($database->confirmUserPass($this->username,md5($subcurpass)) != 0)
				{
				   $form->setError($field, providePhrase('cr', 682)); // Current Password incorrect
				}
			}

			/* New Password error checking */
			$field = "newpass";  //Use field name for new password
			/* Spruce up password and check length*/
			$subpass = stripslashes($subnewpass);
			if(strlen($subnewpass) < 4)
			{
				$form->setError($field, providePhrase('cr', 683)); // New Password too short
			}
			/* Check if password is not alphanumeric */
			else if(!eregi("^([0-9a-z-])+$", ($subnewpass = trim($subnewpass))))
			{
				$form->setError($field, providePhrase('cr', 684)); // New Password not alphanumeric
			}
		}
		/* Change password attempted */
		else if($subcurpass)
		{
			/* New Password error reporting */
			$field = "newpass";  //Use field name for new password
			$form->setError($field, providePhrase('cr', 685)); // New Password not entered
		}

		/* Email error checking */
		$field = "email";  //Use field name for email
		if($subemail && strlen($subemail = trim($subemail)) > 0)
		{
			/* Check if valid email address */
			if(!isValidEmail($subemail))			
			{
				$form->setError($field, providePhrase('cr', 667)); // Email invalid
			}
			$subemail = stripslashes($subemail);
		}

		/* Telephone error checking */
		$field = "telephone";  //Use field name for email
		if($subtelephone && strlen($subtelephone = trim($subtelephone)) > 0)
		{
			/* Spruce up password and check length*/
			$subtelephone = stripslashes($subtelephone);
			 
			// Allows  only digits
			$allowed = "/[^0-9]/i";
			$subtelephone =  preg_replace($allowed,"",$subtelephone);		 
			 
			if((strlen($subtelephone) < 10))
			{
				$form->setError($field, providePhrase('cr', 676)); // Telephone number must be 10 digits
			}
			/* Check if telephone is not numeric */
			else if(!eregi("^([0-9])+$", ($subtelephone = trim($subtelephone))))
			{
				$form->setError($field, providePhrase('cr', 677)); // Telephone number not numeric
			}		 
		}


		/* Pass number error checking */
		$field = "pass";  //Use field name for pass number
		if((strlen($subpassnum) > 0) && ($subpassnum <> '0')) 
		{ // Passnumber not mandetory
			/* Spruce up password and check length*/
			$subpassnum = stripslashes($subpassnum);
			if((strlen($subpassnum) < 4))
			{
				$form->setError($field, providePhrase('cr', 690)); // Pass number too short
			}
			/* Check if password is not numeric */
			else if(!eregi("^([0-9])+$", ($subpassnum = trim($subpassnum))))
			{
				$form->setError($field, providePhrase('cr', 691)); // Pass number not numeric
			}
		}


		/* Errors exist, have user correct them */
		if($form->num_errors > 0)
		{
			return false;  //Errors with form
		}

		/* Update password since there were no errors */
		if($subcurpass && $subnewpass)
		{
			$database->updateUserField($this->username,"u_password",md5($subnewpass));
		}

		/* Change Email */
		if($subemail)
		{
			$database->updateUserField($this->username,"u_email",$subemail);
		}

		/* Change Telephone */
		if($subtelephone)
		{
		// DEBUG echo($subtelephone); die;
			$database->updateUserField($this->username,"u_telephone",$subtelephone);
		}

		/* Change tt pass*/
		if($subpassnum)
		{
			$database->updateUserField($this->username,"user_phone_pass",$subpassnum);
		}

		/* Success! */
		return true;
	}

/**
* editAccountAdminUser - Allows an administrator to update data directly for another user
* Similar to editAccount
*/
	function editAccountAdminUser($subuser, $subnewpass, $subemail, $subpassnum, $subtelephone)
	{
		global $database, $form;  //The database and form object
		/* New password entered */
		if($subnewpass)
		{
			/* New Password error checking */
			$field = "newpass";  //Use field name for new password
			/* Spruce up password and check length*/
			$subpass = stripslashes($subnewpass);
			if(strlen($subnewpass) < 4)
			{
				$form->setError($field, providePhrase('cr', 683)); // New Password too short
			}
			/* Check if password is not alphanumeric */
			else if(!eregi("^([0-9a-z-])+$", ($subnewpass = trim($subnewpass))))
			{
				$form->setError($field, providePhrase('cr', 684)); // New Password not alphanumeric
			}
		}

		/* Email error checking */
		$field = "email";  //Use field name for email
		if($subemail && strlen($subemail = trim($subemail)) > 0)
		{
			/* Check if valid email address */
			if(!isValidEmail($subemail))
			{
				$form->setError($field, providePhrase('cr', 667)); // Email invalid
			}
			$subemail = stripslashes($subemail);
		}

		/* Telephone error checking */
		$field = "telephone";  //Use field name for email
		if($subtelephone && strlen($subtelephone = trim($subtelephone)) > 0)
		{
			/* Spruce up password and check length*/
			$subtelephone = stripslashes($subtelephone);
			 
			// Allows  only digits
			$allowed = "/[^0-9]/i";
			$subtelephone =  preg_replace($allowed,"",$subtelephone);		 
			 
			if((strlen($subtelephone) < 10))
			{
				$form->setError($field, providePhrase('cr', 676)); // Telephone number must be 10 digits
			}
			/* Check if telephone is not numeric */
			else if(!eregi("^([0-9])+$", ($subtelephone = trim($subtelephone))))
			{
				$form->setError($field, providePhrase('cr', 677)); // Telephone number not numeric
			}		 
		}


		/* Pass number error checking */
		$field = "pass";  //Use field name for pass number
		if((strlen($subpassnum) > 0) && ($subpassnum <> '0')) 
		{ // Passnumber not mandetory
			/* Spruce up password and check length*/
			$subpassnum = stripslashes($subpassnum);
			if((strlen($subpassnum) < 4))
			{
				$form->setError($field, providePhrase('cr', 690)); // Pass number too short
			}
			/* Check if password is not numeric */
			else if(!eregi("^([0-9])+$", ($subpassnum = trim($subpassnum))))
			{
				$form->setError($field, providePhrase('cr', 691)); // Pass number not numeric
			}
		}


		/* Errors exist, have user correct them */
		if($form->num_errors > 0)
		{
			return false;  //Errors with form
		}

		/* Update password since there were no errors */
		// This code can be run by several locations, so make sure the variables exist
		
		if ($subnewpass)
		{
			$database->updateUserField($subuser,"u_password",md5($subnewpass));
		}

		/* Change Email */
		if ($subemail)
		{
			$database->updateUserField($subuser,"u_email",$subemail);
		}

		/* Change Telephone */
		if ($subtelephone)
		{
		// DEBUG echo($subtelephone); die;
			$database->updateUserField($subuser,"u_telephone",$subtelephone);
		}

		/* Change tt pass*/
		if($subpassnum)
		{
			$database->updateUserField($subuser,"user_phone_pass",$subpassnum);
		}

		/* Success! */
		return true;
	}

	
/**
* editAccountUserName - Allows an for the update of a name
* Similar to editAccount
*/
	function editAccountUserName($subuser, $subNameFirst, $subNameMiddle, $subNameLast)
	{				
		// Change if we have received any data
		if (($subNameFirst) || ($subNameLast))
		{
			$tt_last  	= TTLetterDigits($subNameLast);
			$tt_first 	= TTLetterDigits($subNameFirst);
			$full_name	= $subNameLast.', '.$subNameFirst;
		
			// Update all the details
			$database->updateUserField($subuser, "u_last", 		$subNameLast);
			$database->updateUserField($subuser, "u_first", 	$subNameFirst);
			$database->updateUserField($subuser, "u_full_name", $full_name);
			$database->updateUserField($subuser, "tt_last", 	$tt_last);
			$database->updateUserField($subuser, "tt_first", 	$tt_first);
		}	
	}  // editAccountUserName


	
/**
* isAdmin - Returns true if currently logged in user is
* an administrator, false otherwise.
*/
	function isAdmin()
	{
		// There is not the opportunity for the user to change their ID to ADMIN, so this should work.
		return (($this->userlevel == SUPER_ADMIN_LEVEL) ||
				($this->userlevel == ADMIN_LEVEL) ||
				($this->username  == ADMIN_NAME));
	}
   
/**
* generateRandID - Generates a string made up of randomized
* letters (lower and upper case) and digits and returns
* the md5 hash of it to be used as a userid.
*/
	function generateRandID()
	{
		return md5($this->generateRandStr(16));
	}
   
/**
* generateRandStr - Generates a string made up of randomized
* letters (lower and upper case) and digits, the length
* is a specified parameter.
*/
	function generateRandStr($length)
	{
		$randstr = "";
		for($i=0; $i<$length; $i++)
		{
			$randnum = mt_rand(0,61);
			if($randnum < 10)
			{
				$randstr .= chr($randnum+48);
			}
			else if($randnum < 36)
			{
				$randstr .= chr($randnum+55);
			}
			else
			{
				$randstr .= chr($randnum+61);
			}
		}
		return $randstr;
	}
};


/**
 * Initialize session object - This must be initialized before
 * the form object because the form uses session variables,
 * which cannot be accessed unless the session has started.
 */
$session = new Session;

/* Initialize form object */
$form = new Form;

?>
