<?php

include_once("TwitterAPIConnect.php"); 

/*
Purpose: 	The TwitterOauth class provides Oauth authication for the twitter API. The class inherits from TwitterAPIConnect the socket connection functions to connect to the twitter server. 
Notes:-	Steps required to obtain the AccessToken and AccessTokenSecret from a twitter user. 

		1. Create an instance of the TwitterOauth class with the $ConsumerKey and $ConsumerSecret provided to you by twitter. 
		2. Call ObtainRequestTokens() 		-Save the return value in a php session or on disk so that tokens can be retrieved after a url redirect. 
		3. Call ObtainRedirectURL() 		-The Return value of this function is the url of the twitter server the user should be redirect to. 
								On the twitter server the user will authorize the application and is then redirected back to the php application. 
		4. Create a new instance of the TwitterOauth Class. This time with both the Consumer key/secret and the RequestTokens stored on disk or in a php session. 
		5. Call ObtainAccessTokens() 		-This function will return a array containing the accesstoken/accesstokensecret these values should be stored on disk 
								since those tokens are required to access the user's account. 
		6.The accessTokens can now be used to upate the user status on the twitter website. 
*/
class TwitterOauth extends TwitterAPIConnect
{	
	private $oAuthNonce; //= /*"8f5d5dcddab500b806fdd9744b10291d";"72d9738119236f639cab576e0711c94b";//*/md5(mt_rand() . microtime()); //fine
	private $oAuthTimeStamp; //= /*"1248461190" ;"1248386247";//*/time(); //fine
	private $oAuthRequestToken;
	private $oAuthRequestTokenSecret;
	private $oAuthSignature;					
	private $oAuthSignatureBaseString;
	
	private $oAuthSignatureMethod = "HMAC-SHA1";		
	private $oAuthVersion = "1.0";

	//twitter API Specific. 
	private $oAuthRequestServer = "http://twitter.com/oauth/request_token";	
	private $oAuthAuthorizeServer = "http://twitter.com/oauth/access_token";
	private $oAuthRequestRedirect = "http://twitter.com/oauth/authorize?";
	private $oAuthRequest = "/oauth/request_token?";	
	private $oAuthAuthorize = "/oauth/access_token?";
	
	private $oAuthServerConnectMethod = "GET";//ok
	private $oAuthElements = array("oauth_version=","&oauth_nonce=","&oauth_timestamp=","oauth_consumer_key=","&oauth_signature_method=","&oauth_signature=","&oauth_token=");	
	
	//@@@ move all http into twitterAPIConnect.php class. 
	private  $httpTwitterStatus;
	private  $httpTwitterResponce;//valid xml 
	
	function TwitterOauth($consumerKey = NULL, $consumerSecret = NULL,$token = NULL, $tokenSecret = NULL)
	{
		$this->oAuthNonce = $this->SetNonce();
		$this->oAuthTimeStamp = $this->SetTimeStamp(); 
		
		$this->oAuthConsumerSecret = $consumerSecret;
		$this->oAuthConsumerKey = $consumerKey; 
		$this->oAuthRequestToken = $token;
		$this->oAuthRequestTokenSecret = $tokenSecret;
	}	
	
	//Name: 	ObtainRequestTokens
	//Purpose:	Obtains the request tokens from the twitter server. Both the 'RequestToken' & 'RequestTokenSecret' are returned in an array.
	//Notes:	-The 'RequestToken' is passed to twitter through a redirect URL, so that the application can be registered with twitter. 
	//		-The 'RequestTokenSecret' is used to sign the Request for the Access token needed to update the user's twitter account. 
	//		-ObtainRequestTokens should only be called during the user sign up routine. 
	public function ObtainRequestTokens()
	{
		//Reseed the Nonce and timestamp for the request. 
		$this->oAuthNonce = $this->SetNonce();
		$this->oAuthTimeStamp = $this->SetTimeStamp(); 
		
		$httpGetRequest = $this->CreateHTTPGetRequestToken();
		
		$twitterResponce = parent::HTTPSendGet($httpGetRequest); 
		$requestTokens = parent::HTTPParseRequestTokens($twitterResponce); 
		
		$this->oAuthRequestToken = $requestTokens["requestToken"];
		$this->oAuthRequestTokenSecret = $requestTokens["requestTokenSecret"];
		
		return $requestTokens;// is an array
	}
	
	//Name:	ObtainAccessTokens
	//Purpose:	Retrieves the Access Tokens for the user from the twitter server. An array is returned with the following fields: 'oauth_token', 'oauth_token_secret',' user_id' and 'screen_name'
	//Notes:	-The 'oauth_token' & 'oauth_token_secret' should be stored in the database since these values are needed to update a twitter user's status. 
	//		-The 'oauth_token' & 'oauth_token_secret'  are equivalent to a user's password/username credentials.  
	public function ObtainAccessTokens()
	{
		//Reseed the Nonce and timestamp for the request. 
		$this->oAuthNonce = $this->SetNonce();
		$this->oAuthTimeStamp = $this->SetTimeStamp(); 
		
		$httpGetAccess = $this->CreateHTTPGetAccessToken(); 
		$twitterResponce = parent::HTTPSendGet($httpGetAccess);
		$accessTokens = parent::HTTPParseAccessTokens($twitterResponce); 
		
		return $accessTokens; 
	}
	
	protected function ObtainAccount($accessToken,$accessTokenSecret) // 0 and 3 need padding. 
	{
		$this->oAuthNonce = /*"652f01507c721f6df310e97836fa0649";*/$this->SetNonce();
		$this->oAuthTimeStamp = /*"1248798990";*/$this->SetTimeStamp(); 
		
		$url = "http://twitter.com/account/verify_credentials.xml"; 
		$this->oAuthSignature	= "GET" ."&".$this->encodeRFC3986($url) ."&"
								. $this->encodeRFC3986($this->oAuthElements[3] . $this->oAuthConsumerKey 
								. $this->oAuthElements[1] . $this->oAuthNonce . $this->oAuthElements[4]
								. $this->oAuthSignatureMethod . $this->oAuthElements[2] . $this->oAuthTimeStamp 
								. $this->oAuthElements[6] . $accessToken. "&" . $this->oAuthElements[0] . $this->oAuthVersion);
								//. "&status=" . $this->encodeRFC3986($statusUpdate));
								
		//echo "<br>account : oAuthSignature before encryption:  $this->oAuthSignature<br><br>";
		
		$this->oAuthSignature = base64_encode(hash_hmac('sha1',$this->oAuthSignature,$this->oAuthConsumerSecret."&".$accessTokenSecret,true));
		$this->oAuthSignature = $this->encodeRFC3986($this->oAuthSignature); //string is now URL Ready. 
		
		$this->oAuthSignatureBaseString = $this->oAuthElements[0] . $this->encodeRFC3986($this->oAuthVersion) . $this->oAuthElements[1]
							. $this->encodeRFC3986($this->oAuthNonce) . $this->oAuthElements[2] . $this->encodeRFC3986($this->oAuthTimeStamp) 
							. "&" . $this->oAuthElements[3] . $this->encodeRFC3986($this->oAuthConsumerKey)  
							. $this->oAuthElements[6]. $this->encodeRFC3986($accessToken) . $this->oAuthElements[4] 
							. $this->encodeRFC3986($this->oAuthSignatureMethod) . $this->oAuthElements[5] . $this->oAuthSignature; 
		//echo ">>>>>>acount : final;: $this->oAuthSignatureBaseString <br><br>";	
		
		return $this->oAuthSignatureBaseString;
	}
	
	//Name:	ObtainStatusUpdateSignature
	//Purpose:	Creates the contents for the HTTP POST request to the twitter server to update the user's status. The Contents are returned in a string variable. 
	//Notes:	-The oauthSignature is generated here. 
	protected function ObtainStatusUpdateSignature($statusUpdate,$accessToken, $accessTokenSecret)
	{
		//Reseed the Nonce and timestamp for the request. 
		$this->oAuthNonce = /*"652f01507c721f6df310e97836fa0649";*/$this->SetNonce();
		$this->oAuthTimeStamp = /*"1248798990";*/$this->SetTimeStamp(); 
		
		//Creating the signature to be hashed. 
		$this->oAuthSignature	= "POST" ."&".$this->encodeRFC3986("http://twitter.com/statuses/update.xml") ."&"
								. $this->encodeRFC3986($this->oAuthElements[3] . $this->oAuthConsumerKey 
								. $this->oAuthElements[1] . $this->oAuthNonce . $this->oAuthElements[4]
								. $this->oAuthSignatureMethod . $this->oAuthElements[2] . $this->oAuthTimeStamp 
								. $this->oAuthElements[6] . $accessToken. "&" . $this->oAuthElements[0] . $this->oAuthVersion
								. "&status=" . $this->encodeRFC3986($statusUpdate)); 	//<<error at status update. 
								
		//echo "<br>oAuthSignature before encryption:  $this->oAuthSignature<br><br>";
		
		//Use the secret access token from the user with the Consumer secret key from the consumer to create the hash of the signature. 
		$this->oAuthSignature = base64_encode(hash_hmac('sha1',$this->oAuthSignature,$this->oAuthConsumerSecret."&".$accessTokenSecret,true));
		$this->oAuthSignature = $this->encodeRFC3986($this->oAuthSignature); //string is now URL Ready. 
		
		//Now OauthSignatureBaseString will contain the entire HTTP POST request body. 
		$this->oAuthSignatureBaseString = $this->oAuthElements[0] . $this->encodeRFC3986($this->oAuthVersion) . $this->oAuthElements[1]
							. $this->encodeRFC3986($this->oAuthNonce) . $this->oAuthElements[2] . $this->encodeRFC3986($this->oAuthTimeStamp) 
							. "&" . $this->oAuthElements[3] . $this->encodeRFC3986($this->oAuthConsumerKey) ."&status=" . $this->encodeRFC3986($statusUpdate) 
							. $this->oAuthElements[6]. $this->encodeRFC3986($accessToken) . $this->oAuthElements[4] 
							. $this->encodeRFC3986($this->oAuthSignatureMethod) . $this->oAuthElements[5] . $this->oAuthSignature; 
							
	//	echo ">>>>>> $this->oAuthSignatureBaseString";	
		
		return $this->oAuthSignatureBaseString;
	}
	
	//Name:	ObtainRedirectURL
	//Purpose: 	Creates the Redirect URL from the RequestToken Obtained. This url is the location the user has to goto to authorize the php application access to twitter. 
	//Note:	-Twitter will redirect the user after giving permission to a predefined URL entered on the twitter application registration page. 
	public function ObtainRedirectURL()//@@@pass in Request Token with default value NULL ??
	{
		$this->oAuthRequestRedirect.= "oauth_token=".$this->oAuthRequestToken;	
		return $this->oAuthRequestRedirect; 
	}
	
	//Name:	CreateHTTPGetRequestToken
	//Purpose:	Create the HTTP GET request to be send to the twitter server to receive the Request tokens back from the server. The GET request is returned as a variable string. 
	//Note:	-
	private function CreateHTTPGetRequestToken()//Create the Request Token Signarure to be send to the twitter server
	{
		$this->oAuthSignature = $this->oAuthServerConnectMethod ."&".$this->encodeRFC3986($this->oAuthRequestServer) ."&"
					. $this->encodeRFC3986($this->oAuthElements[3] . $this->oAuthConsumerKey 
					. $this->oAuthElements[1] . $this->oAuthNonce . $this->oAuthElements[4]
					. $this->oAuthSignatureMethod . $this->oAuthElements[2] . $this->oAuthTimeStamp 
					. "&" . $this->oAuthElements[0] . $this->oAuthVersion); 
		
		//Only create the hash with the ConsumerSecret since the Token Request secret is empty for now. 
		$this->oAuthSignature = base64_encode(hash_hmac('sha1',$this->oAuthSignature,$this->oAuthConsumerSecret."&",true)); 	
		$this->oAuthSignature = $this->encodeRFC3986($this->oAuthSignature); //string is now URL Ready. =/	
		
		$this->oAuthSignatureBaseString = $this->oAuthElements[0] . $this->encodeRFC3986($this->oAuthVersion) . $this->oAuthElements[1]
							. $this->encodeRFC3986($this->oAuthNonce) . $this->oAuthElements[2] . $this->encodeRFC3986($this->oAuthTimeStamp) 
							. "&" . $this->oAuthElements[3] . $this->encodeRFC3986($this->oAuthConsumerKey) . $this->oAuthElements[4]
							. $this->encodeRFC3986($this->oAuthSignatureMethod) . $this->oAuthElements[5] . $this->oAuthSignature; 
	
		$this->oAuthRequest .= $this->oAuthSignatureBaseString; 
		
		return $this->oAuthRequest; 
	}
	
	//Name:	CreateHTTPGetAccessToken
	//Purpose:	Creates the HTTP GET request string to obtain the Access tokens from the twitter server. A Get request is returned as a string. 
	//Notes:	-Requires the 'RequestTokenSecret' 
	private function CreateHTTPGetAccessToken()
	{
		$this->oAuthSignature = $this->oAuthServerConnectMethod ."&".$this->encodeRFC3986($this->oAuthAuthorizeServer) ."&"
								. $this->encodeRFC3986($this->oAuthElements[3] . $this->oAuthConsumerKey 
								. $this->oAuthElements[1] . $this->oAuthNonce . $this->oAuthElements[4]
								. $this->oAuthSignatureMethod . $this->oAuthElements[2] . $this->oAuthTimeStamp 
								. $this->oAuthElements[6] . $this->oAuthRequestToken . "&" . $this->oAuthElements[0] . $this->oAuthVersion);
		
		//Signature is hashed with the RequestTokenSecret obtained from a previous request. 
		$this->oAuthSignature = base64_encode(hash_hmac('sha1',$this->oAuthSignature,$this->oAuthConsumerSecret."&".$this->oAuthRequestTokenSecret,true));
		$this->oAuthSignature = $this->encodeRFC3986($this->oAuthSignature); //string is now URL Ready. 
	
		$this->oAuthSignatureBaseString = $this->oAuthElements[0] . $this->encodeRFC3986($this->oAuthVersion) . $this->oAuthElements[1]
										. $this->encodeRFC3986($this->oAuthNonce) . $this->oAuthElements[2] . $this->encodeRFC3986($this->oAuthTimeStamp) 
										. "&" . $this->oAuthElements[3] . $this->encodeRFC3986($this->oAuthConsumerKey) . $this->oAuthElements[6]  
										. $this->encodeRFC3986($this->oAuthRequestToken) . $this->oAuthElements[4] . $this->encodeRFC3986($this->oAuthSignatureMethod) 
										. $this->oAuthElements[5] . $this->oAuthSignature; 
	
		$this->oAuthAuthorize .= $this->oAuthSignatureBaseString; 
		return $this->oAuthAuthorize; 
	}
	
	//Name:	SetNonce
	//Purpose:	Returns a Random string. 
	private function SetNonce()
	{
		return md5(mt_rand() . microtime());
	}
	
	//Name: 	SetTimeStamp
	//Purpose:	Returns the Unix Epoch time. 
	private function SetTimeStamp()
	{
		return time();
	}
	
	//Name: 	EncodeRFC3986
	//Purpose: 	Encodes a string according to RFC3986 specifications. 
	//Note:	-This is the encoding required by the Oauth process. 
	private function encodeRFC3986($input) 
	{
		return str_replace('+', ' ',str_replace('%7E', '~', rawurlencode($input)));
	}
	
	protected function SetTwitterXML($twitterData)
	{
		$xmlHeader = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
		$responceArray = explode($xmlHeader,$twitterData); 
		
		$this->SetTwitterHTTPStatus($responceArray[0]);
		
		if(isset($responceArray[1]) == 1)//responceArray should hold the xml file returned by twitter. 
		{
			$this->httpTwitterResponce = $xmlHeader . $responceArray[1]; 			
		}
		else
		{	    
			$this->httpTwitterResponce = ""; 
		}	
	}
	
	protected function SetTwitterHTTPStatus($twitterHeader)
	{			
		$httpHeaderArray = explode("Status:",$twitterHeader); 
	
		if(isset($httpHeaderArray[1]))
		{
			$this->httpTwitterStatus = substr($httpHeaderArray[1],1,3);//Get the 'Status:' field from the http header send by the twitter server. 			
		}
		else
		{
			echo "badbad";
		}
		
	}	
	
	public function HTTPResponce()
	{
		return $this->httpTwitterResponce; 
	}
	
	public function HTTPStatus()
	{
		return $this->httpTwitterStatus;
	}


}
?>