<?php
/*
* TeleFlow LogIndex
*
* Copyright (C) 2008 engenic (engenic.com)
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
* http://www.gnu.org/copyleft/gpl.html
*/

/*
===Description===
This program is a simple Index-er which makes random access to a large log file possible.  
* Rows point to byte locations of row starts and ends.
* Errors and restarts are also pointed

===Issues===
* Program where a last line keeps on adding to the Index if Error or Step (Row seems okay)
:eg: 
:0000/00/00,00:00:00.000,00023688,1506646,Log,Logging End<br>
:0000/00/00,00:00:00.000,00023688,1506646,Log,Logging End<br>
:0000/00/00,00:00:00.000,00023688,1506646,Log,Logging End<br>
*/

/*
// eg:  http://localhost/TeleFlow/tflogindex.php?LogFile=Test08_2008-05-22.log
// eg:  http://localhost/TeleFlow/tflogindex.php?LogFile=TFVO43_2008-02-12.log
*/
require_once("cr_reference.php");
require_once($incPath.'LocalSettings.php'); 
//include($incPath."include/session.php");

require_once($websiteLibrary.'calcWorkUnit.php');  


// Save entry to an index file
function AddLogRow($pIc,$pSeek,$pType,$pTimeStamp,$pCode)
{
  $mDate = substr($pTimeStamp,0,10);
  $mTime = substr($pTimeStamp,11,12);
  
  // Ensure blank lines keep the index lined up nicely.  
  if ((trim($mDate) == '') or (substr($mDate,4,1) <> '/')) $mDate = '0000/00/00';
  if ((trim($mTime) == '') or (substr($mTime,2,1) <> ':')) $mTime = '00:00:00.000';
  
  //DEBUGGING: echo ($mDate.','.$mTime.','.rightstr('00000000'.$pIc,8).','.$pSeek.','.$pType.'<br>');
  
  // Date, Time, Row, ByteLoc, Type (Page,Issue), Data (Search info)
  if ($pType == 'Row')
  {
     return $mDate.','.$mTime.','.
       rightstr('00000000'.$pIc,8).','.$pSeek.','.$pType.',<br>'.chr(13).chr(10);
  }
  else
  {
     return $mDate.','.$mTime.','.
       rightstr('00000000'.$pIc,8).','.$pSeek.','.$pType.','.nl2br(trim($pCode)).'<br>'.chr(13).chr(10);   
  }
}

// Retreive the log index
function GetLogIndex($pLogFile)
{
  // Determine what the index file should be named
  $mLogIndex = str_ireplace('.log','.idx',$pLogFile);
  return $mLogIndex;
}

// Update the log index
function UpdateLogIndex($pLogFile)
{
// Defines for the LogFile scanner
$cStepTitle    = 29;  // Location where step title is displayed
$cUserOut      = 26;  // Location where user messages are displayed
$cStepOut      = 32;  // Location where step properties and info is displayed
$cFatalOut     = 35;  // Location where fatal error messages will occur
$cMsgFatal     = 'Attempting to jump to Fatal';
$cMsgFatalL    = strlen($cMsgFatal);
$cMsgRestart   = 'Application restart';
$cMsgRestartL  = strlen($cMsgRestart);
$cStepPickUp   = 'Pick Up';
$cStepHangUp   = 'Hang Up';
$cStepWaitCall = 'Wait For Call';


  $mLogFile = $pLogFile;
  //$mLogFile = "Test08_2008-05-22.log"; 
  //$mLogFile = "TFVO39_2008-05-23.log";

  ///////////////////////////////
  // Update the Log Index
  
  // Determine what the index file should be named
  $mLogIndex = str_ireplace('.log','.idx',$mLogFile);
  //echo $mLogFile;
  //echo $mLogIndex;
  $mIndexRows = '';
  $mIndexLast = '';
  
  $counter    = 1; 
  $ic         = 0;
  $ic_start   = 0; // start at row
  $ic_max     = 100000000; // stops after this number of rows
  $ic_buffer  = 1024*16;
  $ic_recs    = 0;
  if (File_Exists($mLogFile))
  {
    
    // Open the Log file
    $handle     = fopen($mLogFile, "r");    
    // Open or create the index file
    if (File_Exists($mLogIndex))
    {
      $handleIdx  = fopen($mLogIndex, "r");  // Open for Read
      // Load the index
      do
      {
         $buffer = fgets($handleIdx, $ic_buffer);
         $mIndexRows .= $buffer;
         $ic_recs++;
         if (trim($buffer) > 0)
         {
            $mIndexLast = $buffer;
         }
      } while (!feof($handleIdx));
      fclose($handleIdx);
    }
    
    // Position the file only if we have some valid data to work with
    $mSeekPos = 0;
    if (($mIndexLast <> '') and ($ic_recs > 3))
    { 
       $mArray = explode(',',$mIndexLast,5);
       $mSeekPos = $mArray[3];  // Byte position (at end of row)
       // Make sure we are valid
       if (is_numeric($mSeekPos))
       {
         $ic = $mArray[2];  // Row
         // Position to end of row
         fseek($handle, $mSeekPos);
         // Position for the next fgets()
         $ic++;
       }
       else
       {
         $mIndexRows = '';
       }
    }
    else
    {
       $mIndexRows = '';
    }
    
    // Read through the log, and update the index
    do 
    {  
       $buffer = fgets($handle, $ic_buffer);
    
       // For paging, get the seek location for every thousand rows
       if (($ic % 1000) == 0)
       {
          $mIndexRows .= AddLogRow($ic,ftell($handle),'Row',substr($buffer,0,23),substr($buffer,26,200));
       }
    
       if ($ic >= $ic_start)
       {
            // Do we have a new step?    [xx] Step Name
            if ((substr($buffer,$cStepTitle,1) == '[') and (!$mFormatted)) 
            {
             //echo 'Found:'.$buffer;
               if ((StrPos($buffer, $cStepPickUp) > 0) or 
                   (StrPos($buffer, $cStepHangUp) > 0) or 
                   (StrPos($buffer, $cStepWaitCall) > 0))
               {
                  $mIndexRows .= AddLogRow($ic,ftell($handle),'Step',substr($buffer,0,23),substr($buffer,26,200));
               }
            }
            // Do we have a user message or a Proceedure change?
            elseif ((substr($buffer,$cFatalOut,$cMsgFatalL) == $cMsgFatal)) 
            {
               $mIndexRows .= AddLogRow($ic,ftell($handle),'Error',substr($buffer,0,23),substr($buffer,26,200));
            }
            // Do we have an application start 
            elseif ((substr($buffer,26,$cMsgRestartL) == $cMsgRestart))
            {
               $mIndexRows .= AddLogRow($ic,ftell($handle),'App','',$cMsgRestart);
            }
            // Have we bumped into the extent of logging (for systems only logging errors - see tfconfig)
            elseif ((substr($buffer,1,1) == '<'))
            {
               if (StrPos($buffer, 'START') > 0)
               {
                   $mIndexRows .= AddLogRow($ic,ftell($handle),'Log','','Logging Start');
               }
               else
               {
                   $mIndexRows .= AddLogRow($ic,ftell($handle),'Log','','Logging End');
               }
            }
       }
    } while (!feof($handle) && ++$ic <= $ic_max );
    fclose($handle);
    
    // Save the Index
    // Open or create the index file
    //unlink($mLogIndex);
    $handleIdx  = fopen($mLogIndex, "w");  // Open for Write from beginning
    fwrite($handleIdx, $mIndexRows);
    fclose($handleIdx);
  }
  // Update Log index end
  ///////////////////////////////
  
  return $ic;
}  
?>