<?php
/* 
mediaWorkUnit - Used to determine and manipulate filenames for media for use with TeleFlowRelay
- Created by: Vern Baker
- Last updated: 2009
- GPL license
*/

// Generate the name of what the original MediaFile should be for the web site
function mediaFileNameGenerate($pClient_id, $pOwnerUID, $pAppTypeName, $pMediaFileType, $pTimeStamp, $pIVRPort, $pCalledNumber, $pCallingNumber)
{
	// Make sure we have some defaults
	$mClient_id 	= $pClient_id 		<> '' ? $pClient_id 	: 'NA'; 	// Client id
	$mOwnerUID		= $pOwnerUID		<> -1 ? $pOwnerUID		: 'NA';		// Owner User id
	$mAppTypeName 	= $pAppTypeName 	<> '' ? $pAppTypeName 	: 'GENERAL'; // Application that created the file
	$mMediaFileType = $pMediaFileType 	<> '' ? $pMediaFileType : 'RECORDING'; // File type FAX, VOICEMSG, TWITYAX etc
	// pTimeStamp needed always
	$mIVRPort		= $pIVRPort			<> '' ? $pIVRPort		: '---';
	$mCalledNumber 	= $pCalledNumber 	<> '' ? $pCalledNumber 	: 'NA';
	$mCallingNumber = $pCallingNumber 	<> '' ? $pCallingNumber : 'NA';
	
	// Pad the IVR port to make it line up nicely later
	$mIVRPort = rightstr('000'.$mIVRPort,3);
	
	// Create a unique file that cannot occur twice on a TeleFlowRelay system
	$mFileNameGenerate = $mAppTypeName.'_'.
						 $mClient_id.'_'.
						 $mMediaFileType.'_'.
						 $mOwnerUID.'_'.
						 $pTimeStamp.'_'.	
						 // Timestamp + the IVR port will ensure unique file names ... no matter what
						 trim($mIVRPort).'_'.
						 trim($mCalledNumber).'_'.
						 trim($mCallingNumber).'.WAV';  // Note: TeleFlow will create uppercase
						 
	// return 
	return $mFileNameGenerate;
} // webMediaFileNameGenerate


// Function for locating web location of client media
function clientMediaFolderLocate($pCreateFoldersYN, $pIncPath, $pClientId)
{
	$mClientMediaWeb = $pIncPath.CLIENT_MEDIA_AREA.'/'.'client_'.zeroFill($pClientId,5);

	// If we need create folders
	if ($pCreateFoldersYN == true)
	{
		// Does it exist?
		if (!is_dir($mClientMediaWeb))
		{
			// Does now
			mkdir($mClientMediaWeb);
		}					
	}
	
	$mMediaClientRoot = filePathForOS($mClientMediaWeb);

	return $mMediaClientRoot;
} // clientMediaFolderLocate


// Function find the original content folder
// - Note: These files will be located in the TeleFlow Server content creation location
function mediaFileFolderLocate($pCreateFoldersYN, $pClientId, $pFileFolder)
{
	// Determine the names of the files
	$mMediaRoot = TELEFLOW_CLIENT_MEDIA_AREA;
	
	// If we need create folders
	if ($pCreateFoldersYN == true)
	{
		// Does it exist?
		if (!is_dir($mMediaRoot))
		{
			// Does now
			mkdir($mMediaRoot);
		}					
	}
	
	// Location of the client 
	$mMediaClientRoot = $mMediaRoot.'/client_'.
						zeroFill($pClientId,5);
	// If we need create folders
	if ($pCreateFoldersYN == true)
	{
		// Does it exist?
		if (!is_dir($mMediaClientRoot))
		{
			// Does now
			mkdir($mMediaClientRoot);
		}				
	}
						
	// Add the sub folder, if defined
	if ($pFileFolder <> '')
	{
		$mMediaClientRoot .= '/'.$pFileFolder;
	}
	// If we need create folders
	if ($pCreateFoldersYN == true)
	{
		// Does it exist?
		if (!is_dir($mMediaClientRoot))
		{
			// Does now
			mkdir($mMediaClientRoot);
		}				
	}
	
	// Make the file name conform to Windows
	$mMediaClientRoot = filePathForOS($mMediaClientRoot);
	
	return $mMediaClientRoot;
} // mediaFileFolderLocate


// Ensure the file is ready to create
// eg: webMediaFileName($incPath, $_SESSION['userclientid'], $row_rsSearchResultDate['file_name'])
// - Note: It is assumed that a TeleFlow Server will create this file and that it will then be transfered to a remote server.  
// For that reason, TELEFLOW_CLIENT_MEDIA_AREA is used instead CLIENT_MEDIA_AREA (which is the web access)
function mediaFileNameCreate($pIncPath, $pClientId, $pMediaFieldOriginalFolder, $pMediaFileOriginalFileName, &$pMediaFileFullPath)
{
	// Create the full path
	$mMediaClientRoot = mediaFileFolderLocate(true, $pClientId, $pMediaFieldOriginalFolder);

	// Get the name to be created
	$mLocalFilename = $mMediaClientRoot.'/'.$pMediaFileOriginalFileName;
	// Remove if exists
	if (file_exists($mLocalFilename))
	{
		unlink($mLocalFilename);
	}

	// Make the file name conform to Windows
	$mLocalFilename = filePathForOS($mLocalFilename);
	
	// RETURN a filename via Reference
	$pMediaFileFullPath = $mLocalFilename;
	
	// RETURN the full path
	return true;
}  // mediaFileNameCreate

/*
	// Find the file root for the webside
	// $row_rsSearchResultDate['web_dir_loc']
	$clientMediaWeb = $pIncPath.TELEFLOW_CLIENT_MEDIA_AREA.'/'.'client_'.zeroFill($pClientId,5);				
		
	// Add the download reference - The new name would not include the _orig.
	$mMediaFile = $clientMediaWeb.'/recordings/'.
				untack_orig($pMediaFileOriginalFileName);
	return $mMediaFile;
*/	

// Return the filepath for the system
function filePathForOS($pFilePath)
{
	if (TELEFLOW_WINDOWS_INSTALL == true)
	{
		// Windows
		return slashToBackslash($pFilePath);
	}
	else
	{
		// Linux etc
		return backslashToSlash($pFilePath);
			
	}
} // filePathForOS


// Wrap file with link 
function webMediaFileLink($pMediaFile, $pFileLinkDetail, $pClass)
{
	$mMediaLink =	'<a href="'.$pMediaFile.'"';
	if ($pClass <> '')
	{
		$mMediaLink .=	' class="style10"';
	}
	// Wrap it
	$mMediaLink .=	'>'.$pFileLinkDetail.'</a>';
	return $mMediaLink;
}


// Generate the names of the media file to convert to
function getMediaFileNameMP3($pFileWAV)
{
	$mFileWAV =  str_replace(strtolower(FTPWAVEXT), '.mp3', $pFileWAV);   	// Lower case
	$mFileWAV =  str_replace(strtoupper(FTPWAVEXT), '.mp3', $mFileWAV);   	// Upper case
	return $mFileWAV;
}

function getMediaFileNameGSM($pFileWAV)
{
	return str_replace(FTPWAVEXT, '_GSM.wav', $pFileWAV);	// Lower case// Used for renaming files in transfer
}
// Used for transfering a file to an FTP location
function getFileNameForXfer($pFilename)
{
	return (str_replace(FTPWAVEXT, FTPXFEREXT, $pFilename));
}

						
function tack_orig($pfilename)
{
	return (str_replace('.', '_orig.', $pfilename));
}

function untack_orig($pfilename)
{
	return (str_replace('_orig.', '.', $pfilename));
}

///////////////////////////
// HTTP Send File
// -- Send the file using HTTP command 
///////////////////////////
function sendTeleFlowRelayFileViaHTTP(
						$pClientId, 
						$pApp_transfer_type,		// FileUpload, FileUploadTwitYax  etc
						$pHttp_web_server, 			// Comes from FAT table
						$pApp_type_name, 
						$pMedia_file_id, 
						$pMedia_file_directory, 
						$pFileToSend, 
						$pMediaTypeCode,
						$pMaxMBSize,
						$pOauthKey1,
						$pOauthKey2,
						&$varReceivedFileSize
						)	
{
	global $incPath;

	// Load the contents of the file to Mime encode
	// -WARNING: This file could be any size.  Should we put a limit on it somehow?
	$handle = fopen($pFileToSend, "rb");  // windows MUST have a 'b' for binary
	$contents = fread($handle, filesize($pFileToSend));
	fclose($handle);
	//output('Size: '.strlen($contents));
	//output(base64_encode($contents));
	//output('Size: '.strlen($contents));
	//output('End');
	
	// For the XML statement to send 
	$data1 = '<REQUEST action="'.$pApp_transfer_type.'" name="'.basename($pFileToSend).'" '.
				'client_id="'.$pClientId.'" '.
				'app_name_type="'.$pApp_type_name.'" '.
				'media_file_id="'.$pMedia_file_id.'" '.
				'media_type="'.$pMediaTypeCode.'" '.				
				'media_directory="'.trim($pMedia_file_directory).'" '.
				'oauth_key1="'.trim($pOauthKey1).'" '.
				'oauth_key2="'.trim($pOauthKey2).'">';
			
	$data2 = '<FileContents>'.base64_encode($contents).'</FileContents>';
	$data3 = '</REQUEST>';
	// Post the file
	//echo $data1.$data3; die;
	$result = ffl_XMLPostStream($pHttp_web_server, $data1.$data2.$data3);
	// Break up the result, and check success
	$mResultCode	='Nothing';
	$mFileSize 		= -1;

	// Log response 
	LogTextToFileXMLTimeStamp($incPath.SYSTEM_LOGFILES.$pApp_type_name."-".Date('Y-m-d').".log", $result['body']);
	
	
	// Scan the response for important information
	try 
	{
		$xml = new SimpleXMLElement($result['body']);
		foreach($xml->children() as $child)
		{
			//echo $child->getName() . ": " . $child . "<br />";
			if ($child->getName() == "SUCCESS")
			{
				// Get the name
				foreach($child->attributes() as $a => $b)
				{
					if ($a == "result")
					{
					   $mResultCode = $b;
					}
					if ($a == "filesize")
					{
					   $mFileSize = intval($b);
					}
				}
				// Get the result
				$content = trim($child); 
			}
			
			// Failure
			if ($child->getName() == "FAILURE")
			{
				// Get the name
				foreach($child->attributes() as $a => $b)
				{
					if ($a == "result")
					{
					   $mResultCode = $b;
					}
					if ($a == "filesize")
					{
					   $mFileSize = intval($b);
					}
				}
				// Get the result
				$content = trim($child);  
			}			
		}
	}
	catch(Exception $e)
	{
		$content  = 'Message: '.$e->getMessage();
		$content .= 'Body: '.$result['body'];
	}
	
	// Return the filesize
	$varReceivedFileSize = $mFileSize;
	// Return a verbose description
	return $content;
} // sendTeleFlowRelayFileViaHTTP

?>